/**********************************************************************
 *                ACTIVE SILICON LIMITED / DATACELL LIMITED           *
 *  File name   : d16ansq.c                                           *
 *  Function    : Snapper-DIG16 areascan example using callbacks      *
 *                and swapped image buffers.                          *
 *  Project     : ASLexamp                                            *
 *  Author      : Mike Harrison                                       *
 *  Systems     : Ansi C - OS independent                             *
 *  Written     : 24.Feb.98                                           *
 *  Updated     :                                                     *
 *  Version     :                                                     *
 *                                                                    *
 *  Copyright (c) 1993-99 Active Silicon Ltd                          *
 **********************************************************************
 * Comments:
 * This program demonstrates the set up & operation of Snapper-Dig16.  It
 * demonstrates all the possible modes of acquisition - from single shot
 * to interrupt driven.
 * 
 * Operating System Compiler Switch.  Options are :
 *   _DOS32     -
 *   _LYNXOS    - LynxOS 2.5 on _X86 or _PPC
 *   _MACINTOSH - Macintosh System 7.5 and later pn PowerPCs using MetroWerks
 *                CodeWarrior 9 and later.
 *   _SOLARIS2  - Solaris 2.6 and above.
 *   _VXWORKS   - VxWorks on PowerPC.
 *   _WIN95     - Windows 95 (must also define _WINDOWS).
 *   _WINNT     - Windows NT4 (must also define _WINDOWS).
 *
 *
 * Display type compiler switches.  Options are:
 *   _CONSOLE_ONLY     - No image display, just messages sent to stdout.
 *                       This over-rides all other display modes.
 *   _X_GRAPHICS       - Use X Graphics imaging and control.  This is
 *                       currently supported under _SOLARIS2, _LYNX and
 *                       _VXWORKS.
 *  _WIN32_DISPLAY_DDB - Use Device Dependent Bitmap (DDB) display method
 *                       under _WIN95 or _WINNT.
 *  _WIN32_DISPLAY_DIB - Use Device Independent Bitmap (DIB) display method
 *                       under _WIN95 or _WINNT.
 *  _WIN32_DISPLAY_DDI - Use DirectDraw Interface (DDI) display method under
 *                       _WIN95 or _WINNT.
 *
 * Acquisition Method compiler switches.  Options are:
 *  _SINGLE_IMAGE - acquire single image then quit
 *  _SYNC_LOOP    - blocked acquisition followed by image display.  This is
 *                  the simplest method but is not effecient as the
 *                  acquisition time cannot be used for image processing.
 *  _ASYNC_LOOP   - start asynchronous acquisition, display image then
 *                  complete acquisition.  This is more effecient than the
 *                  _SYNC_LOOP method as the previous image can be processed
 *                  whilst the current image is acquired.
 *  _CALLBACKS    - independent asynchronous acquisition and image display.
 *                  An interrupt function completes the asynchronous
 *                  acquisition, starts the next and then signals to a
 *                  separate thread that an image is available for display.
 *                  As this method doesn't rely on the processing time being
 *                  less than the acquisition time (as in the _ASYNC_LOOP
 *                  method), this method can be used when the AVERAGE
 *                  processing time is less than the AVERAGE acquisition
 *                  time.
 * 
 * 
 * The following global variables provide additional control:
 *
 * gdwDebugLevel - set in the range from 1 (most info) to 10 (least info) to
 *                 control the amount of debug information output to stdout.
 *
 * gdwCameraType - set the specific camera type.  The full list of supported
 *                 cameras can be found in DIG16INI.c, supplied as part of
 *                 the SDK.
 *
 * gdwRequiredBits - set the number of bits of data to acquire from the
 *                   video source.
 *
 * gdwHwSubSample - set the hardware sub sample ratio for the acquisition.
 *
 * gdwFramesRequired - set the number of frames of video data to acquire.
 *                     (A value of 0 is infinite).
 *
 * gdwSemWait - set the semaphore timeout period.
 *
 * gdwFileFormat - set the file format of the image to be saved.  The last
 *                 acquired image is always saved before the application
 *                 terminates.
 */

/* Active Silicon Include files */
#include <asl_inc.h>
#include <xpl_inc.h>


/* Select acquisition method */
/*#define _SINGLE_IMAGE  */
/*#define _SYNC_LOOP    */
#define _ASYNC_LOOP   
/*#define _CALLBACKS    */


/* Global Variables
 * ****************
 */
/* Specify the camera type we expect to be using with this program.
 * The full list can be found in DIG16ini.c, supplied with the SDK.
 */
ui32 gdwCameraType = DIG16_PULNIX_TM9700;
ui32 gdwFramesRequired = 0;         /* Specify number of frames to acquire - 0 is infinite */
ui32 gdwSemWait = 5;                /* Timeout period on semaphores */
ui32 gdwRequiredBits = 8;           /* Digital camera so data width can be specified up to max of 16 bits */
ui32 gdwHwSubSample = DIG16_SUB_X1; /* Specify the hardware image sub-sampling ratio required. */
ui32 gdwFileFormat = TMG_TIFF;      /* Specify the dump image file type */
ui32 gdwDebugLevel=10;              /* Set triviality level of debug messages */



/* Local Function Prototypes
 * *************************
 */
void EXPORT_FN MyCallback(Thandle hSnapper, ui32 dwIntSrc, void *pMyData);
Terr ConfigSnapperDynamic( struct GuiInfo*, struct Event* );
void TidyAndExit( struct Info *psInfo );


int main( ui16 argc, char** argv)
{
   static char *szVersion="4.00.00";
   static char *szCopyright="\nCopyright (C) 1998-1999 Active Silicon Ltd, Uxbridge.";
   static char *szFnName = "D16ansq";
   
   struct Info stiLocal;         /* Snapper / Imaging variables */
   struct Event *pseEventRoot;   /* Event queue message list */
   ui32 dwBufferNumber;          /* Current Buffer ID */
   Terr teStatus;                /* Snapper status flag */
   Terr teImageStatus;           /* Image status flag */


   /* Initialise port for debug output */
   DBG_Open( szFnName );


   /* Header message */
   DBG_WriteString( 10, "%s\n", szCopyright );
   DBG_WriteString( 10, "Snapper DIG16 area-scan repeated single image capture utility, version: %s\n", szVersion);


   /* Turn on the library error reporting.
    */
   GUI_EnableErrorHandler( TRUE );


   /* Initialise the GUI display info structure */
   if ( ASL_is_err( GUI_DisplayInitInfo( &stiLocal.sGuiInfo, argv[0] ) ) )
   {
      DBG_WriteString( 10 | XPL_DBG_PROMPT, "%s: Failed to configure GUI info structure\n", szFnName);
      TidyAndExit( &stiLocal );
   }


   /* SnapInfo structure init.
    */
   if( ASL_is_err ( GUI_SnapInit( &stiLocal.sSnapInfo ) ) )
   {
      DBG_WriteString( 10 | XPL_DBG_PROMPT, "%s: Failed to initialise SnapInfo structure\n", szFnName);
      TidyAndExit( &stiLocal );
   }

   
   /* Parse command-line options */
   if( ASL_is_err( GUI_SetCmdParams( &stiLocal, (ui16)argc, argv) ) )
   {
      DBG_WriteString( 10 | XPL_DBG_PROMPT, "%s: Failed to parse command line args\n", szFnName);
      TidyAndExit( &stiLocal );
   }


   /* TMG init.
    * Snapper absolutely requires the TMG imaging library just to store 
    * image data. Whether or not you use it for further processing is up to you. 
    */
   if( ASL_is_err ( GUI_TmgInit( &stiLocal.sGuiInfo ) ) )
   {
      DBG_WriteString( 10 | XPL_DBG_PROMPT, "%s: Failed to initialise TMG image structures\n", szFnName);
      TidyAndExit( &stiLocal );
   }


   /* Init display. 
    * Window sizing parameters from the command line are automatically processed.
    */
   if( ASL_is_err( GUI_DisplayInit( &stiLocal.sGuiInfo ) ) )
   {
      DBG_WriteString( 10 | XPL_DBG_PROMPT, "%s: Failed to initialise GUI display", szFnName);
      TidyAndExit( &stiLocal );
   }


   /* All Snapper initialisation here */
   if ( ASL_is_err( stiLocal.sSnapInfo.hBase = BASE_create( (ui16) stiLocal.sSnapInfo.dwBoardID ) ) )
   {
      DBG_WriteString( 10 | XPL_DBG_PROMPT, "%s: Failed to configure Baseboard and Snapper\n", szFnName);
      TidyAndExit( &stiLocal );
   }

   if ( ( stiLocal.sSnapInfo.hSnapper = BASE_get_parameter( stiLocal.sSnapInfo.hBase, BASE_MODULE_HANDLE )) == MODULE_NULL_HANDLE )
   {
      DBG_WriteString( 10 | XPL_DBG_PROMPT, "%s: Failed to initialize Snapper - probably not fitted\n", szFnName, 0);
      TidyAndExit( &stiLocal );
   }
   
   /* Test to check that the correct Snapper personality module is fitted. This
    * program drives a DIG16 so we only want a PCI/ISA DIG16 or a PMC DIG16. 
    */
   stiLocal.sSnapInfo.wModuleID = (ui16) ASL_get_ID( BASE_get_parameter( stiLocal.sSnapInfo.hBase, BASE_MODULE_FAMILY ) );
   if ( stiLocal.sSnapInfo.wModuleID != DIG16_ID )
   {
      DBG_WriteString( 10 | XPL_DBG_PROMPT, "%s: Incorrect Snapper module fitted (should be SNP-DIG16) \n", szFnName);
      TidyAndExit( &stiLocal );
   }

   /* Specific camera-type initialisation. 
    * For your camera, check that the file dig16ini.c in ~/lib/src/dig16 has your 
    * camera type listed (see table above) or create a new entry based on the ones given.
    */
   if ( ASL_is_err ( DIG16_initialize( stiLocal.sSnapInfo.hSnapper, (ui16) gdwCameraType, (ui16) gdwRequiredBits ) ) )
   {
      DBG_WriteString( 10 | XPL_DBG_PROMPT, "%s: Failed to initialise Snapper D16\n", szFnName );
      TidyAndExit( &stiLocal );
   }
   DBG_WriteString( 10, "%s: Snapper D16 Initialised OK\n", szFnName );


    /* Set sub-sampling factor */
   if( ASL_is_err( DIG16_set_capture(stiLocal.sSnapInfo.hSnapper, gdwHwSubSample ) ) )
   {
      DBG_WriteString( 10 | XPL_DBG_PROMPT, "%s: Set-capture failed \n", szFnName );
      TidyAndExit( &stiLocal );
   }

   /* format the data to the correct output foramt */
   if( DIG16_get_data_width( stiLocal.sSnapInfo.hSnapper, DIG16_IMAGE_DATA_WIDTH ) <= 8 )
      teStatus = DIG16_set_format( stiLocal.sSnapInfo.hSnapper, DIG16_FORMAT_Y8, stiLocal.sGuiInfo.wDisplayFormat );
   else
      teStatus = DIG16_set_format( stiLocal.sSnapInfo.hSnapper, DIG16_FORMAT_Y16, stiLocal.sGuiInfo.wDisplayFormat );
   
   if( ASL_is_err( teStatus ) )
   {
      DBG_WriteString( 10 | XPL_DBG_PROMPT, "%s: Set_format failed \n", szFnName );
      TidyAndExit( &stiLocal );
   }

   /* Store the image sizes and Lock the memory down to save time. */
   for( dwBufferNumber = 0; dwBufferNumber < NUM_VID_IMAGES; dwBufferNumber++ )
   {
      DIG16_set_image( stiLocal.sSnapInfo.hSnapper, GUI_TmgGetImage( &stiLocal.sGuiInfo, dwBufferNumber ) );
      TMG_image_set_flags( GUI_TmgGetImage( &stiLocal.sGuiInfo, dwBufferNumber ), TMG_LOCKED, TRUE);
   }
   
   /* Resize the window to be the full image
    */
   if( ASL_is_err( GUI_DisplayReSize( &stiLocal.sGuiInfo ) ) )
   {
      DBG_WriteString( 10 | XPL_DBG_PROMPT, "%s: Failed to ReSize the GUI display", szFnName);
      TidyAndExit( &stiLocal );
   }



#if defined _SINGLE_IMAGE
/************************/

   DBG_WriteString( 10, "%s: Entering single image acquisition \n", szFnName );
   
   /* Start timing */
   ASL_Thrd_Gettime( &stiLocal.sSnapInfo.dStart );

   teStatus = DIG16_capture_to_image( stiLocal.sSnapInfo.hSnapper, GUI_TmgGetImage( &stiLocal.sGuiInfo, stiLocal.sSnapInfo.dwFrameBufferNumber ), DIG16_START_AND_WAIT );
   if( teStatus != ASL_OK )
   {
      DBG_WriteString( 10 | XPL_DBG_PROMPT, "%s: Capture_to_image failed \n", szFnName );
      TidyAndExit( &stiLocal );
   }
   else
      stiLocal.sSnapInfo.dwFrameCount++;

   _UNUSED_VAR( pseEventRoot );
   _UNUSED_VAR( teImageStatus );


#elif defined _SYNC_LOOP
/***********************/

   DBG_WriteString( 10, "%s: Entering synchronous imaging loop \n", szFnName );
   
   /* Capture first image to have something to display at start */
   teStatus = DIG16_capture_to_image( stiLocal.sSnapInfo.hSnapper, GUI_TmgGetImage( &stiLocal.sGuiInfo, stiLocal.sSnapInfo.dwFrameBufferNumber ), DIG16_START_AND_WAIT );
      
   /* Start timing */
   ASL_Thrd_Gettime( &stiLocal.sSnapInfo.dStart );

   /* Acquire images */
   while ( !ASL_is_err(teStatus) && !SNAP_FramesFinished( stiLocal.sSnapInfo, gdwFramesRequired, stiLocal.sSnapInfo.dwFrameCount ) )
   {
      /* Start the next capture */
      stiLocal.sSnapInfo.dwFrameBufferNumber = ( stiLocal.sSnapInfo.dwFrameBufferNumber + 1 ) % NUM_VID_IMAGES;
      DIG16_capture_to_image( stiLocal.sSnapInfo.hSnapper, GUI_TmgGetImage( &stiLocal.sGuiInfo, stiLocal.sSnapInfo.dwFrameBufferNumber ), DIG16_START_AND_RETURN );

      /* Having acquired the next image, display it */
      GUI_DisplayImage( &stiLocal.sGuiInfo, stiLocal.sSnapInfo.dwDisplayBufferNumber );

      stiLocal.sSnapInfo.dwDisplayBufferNumber = ( stiLocal.sSnapInfo.dwDisplayBufferNumber + 1) %  NUM_VID_IMAGES;
      stiLocal.sSnapInfo.dwFrameCount++;

      /* Having displayed the last image, wait for the current one to complete reading */
      teStatus = DIG16_get_capture_status( stiLocal.sSnapInfo.hSnapper, DIG16_START_AND_WAIT );

      /* Handle user input */
      if( !ASL_is_err( teStatus ) )
         teStatus = GUI_CheckMessageQueue( &stiLocal.sGuiInfo, &pseEventRoot );
      if( !ASL_is_err( teStatus ) )
       /* Handle list of events returned */
         teStatus =  ConfigSnapperDynamic( &stiLocal.sGuiInfo, pseEventRoot );
       
   }

   _UNUSED_VAR( teImageStatus );


#elif defined _ASYNC_LOOP
/************************/

   DBG_WriteString( 10, "%s: Entering asynchronous imaging loop \n", szFnName );
   teImageStatus = ASL_OK;
   teStatus = DIG16_capture_to_image( stiLocal.sSnapInfo.hSnapper, GUI_TmgGetImage( &stiLocal.sGuiInfo, stiLocal.sSnapInfo.dwFrameBufferNumber ), DIG16_START_AND_RETURN);
   if( teStatus != ASL_OK )
   {
      DBG_WriteString( 10, "%s: Capture_to_image failed \n", szFnName );
      TidyAndExit( &stiLocal );
   }
   else
   {
      /* Clean up after the first asynch acquisition, we block until the capture is complete,
       * alternatively we could use START_AND_RETURN in a polling loop.
       */
      teStatus = DIG16_get_capture_status( stiLocal.sSnapInfo.hSnapper, DIG16_START_AND_WAIT );
      DBG_WriteString( 8, "%s: Initial image started OK \n", szFnName );
   }
   
   ASL_Thrd_Gettime( &stiLocal.sSnapInfo.dStart );
   
   /* Acquire images */
   while ( !ASL_is_err( teStatus ) && !SNAP_FramesFinished( stiLocal.sSnapInfo, gdwFramesRequired, stiLocal.sSnapInfo.dwFrameCount ) )
   {
      /* Read the next frame */
      stiLocal.sSnapInfo.dwFrameBufferNumber = ( stiLocal.sSnapInfo.dwFrameBufferNumber + 1 ) % NUM_VID_IMAGES;

      DIG16_capture_to_image( stiLocal.sSnapInfo.hSnapper, GUI_TmgGetImage( &stiLocal.sGuiInfo, stiLocal.sSnapInfo.dwFrameBufferNumber ), DIG16_START_AND_RETURN );
  
      /* While the Snapper is busy acquiring the next image, display the current image */
      GUI_DisplayImage( &stiLocal.sGuiInfo, stiLocal.sSnapInfo.dwDisplayBufferNumber );

      stiLocal.sSnapInfo.dwDisplayBufferNumber = ( stiLocal.sSnapInfo.dwDisplayBufferNumber + 1) %  NUM_VID_IMAGES;
      stiLocal.sSnapInfo.dwFrameCount++;
      stiLocal.sSnapInfo.dwDisplayCount++;

      /* Handle user input */
      teStatus = GUI_CheckMessageQueue( &stiLocal.sGuiInfo, &pseEventRoot );
      if( !ASL_is_err( teStatus ) )
         /* Handle list of events returned */
         teStatus = ConfigSnapperDynamic( &stiLocal.sGuiInfo, pseEventRoot );
      
      /* Get around to checking and cleaning up. If the loop is fast enough, we get here and wait, hopefully briefly.
       * If its not, then again the speed of acquisition is limited by the display/image processing call.
       */
      teImageStatus = DIG16_get_capture_status( stiLocal.sSnapInfo.hSnapper, DIG16_START_AND_WAIT );
      if ( ASL_is_err( teImageStatus ) )
         teStatus = ASL_ERROR;
   }

   
#elif defined _CALLBACKS
/************************/

#if defined _WINNT || defined _SOLARIS2 || defined _VXWORKS || defined _LYNXOS
   DBG_WriteString( 10, "%s: Entering asynchronous imaging loop using callbacks\n", szFnName );
   
   /* Create Semaphore */
   if( ASL_is_err( ASL_Sem_Create( &stiLocal.sSemInfo, 0, 10 ) ) )
   {
      DBG_WriteString( 10, "%s: Sem_Create failed \n", szFnName );
      TidyAndExit( &stiLocal );
   }

   /* Enable interrupts & register our callback function */
   SNAP_CallbackSetActive( stiLocal.sSnapInfo, FALSE );
   SNAP_CallbackSetStatus( stiLocal.sSnapInfo, ASL_OK );
   DIG16_set_callback(   stiLocal.sSnapInfo.hSnapper, DIG16_CALLBACK_SET, MyCallback, &stiLocal ); 
   DIG16_set_interrupts( stiLocal.sSnapInfo.hSnapper, DIG16_INT_CAPTURE_COMPLETE, TRUE);
   
   ASL_Thrd_Gettime( &stiLocal.sSnapInfo.dStart );
   
   /* Begin capture */
   teStatus = DIG16_capture_to_image( stiLocal.sSnapInfo.hSnapper, GUI_TmgGetImage( &stiLocal.sGuiInfo, stiLocal.sSnapInfo.dwFrameBufferNumber ), DIG16_START_AND_RETURN);
   if( teStatus != ASL_OK )
   {
      DBG_WriteString( 10, "%s: Capture_to_image failed \n", szFnName );
      TidyAndExit( &stiLocal );
   }
   else
   {
      /* No need to clean up with get_capture_status after the first asynch acquisition
       * because we use the callback function to do this immediately it gets notified 
       * an image has been acquired.
       */
      DBG_WriteString( 8, "%s: Initial image acquired OK \n", szFnName );
   }

   /* The use of callbacks has an extra advantage over the first two methods, by completely 
    * disconnecting the image processing loop from the acquisition loop, the acquisition can
    * proceed at full speed, even if the processing lags behind. The next image it processes
    * ( in this implementation ) will be the most recent image acquired. This does however mean
    * that some images, although acquired, will not be processed and so are lost. Get a faster 
    * display/processing routine if this is the case.
    */
   while ( !ASL_is_err( teStatus ) && !SNAP_FramesFinished( stiLocal.sSnapInfo, gdwFramesRequired, stiLocal.sSnapInfo.dwFrameCount ) )
   {
      /* Wait for a buffer to be filled by the callback. */
      teStatus = ASL_Sem_WaitWithTimeout( &stiLocal.sSemInfo, gdwSemWait );
      if( ASL_is_err( teStatus ) )
      {
         DBG_WriteString( 10, "%s: Failed to wait on callback capture %d\n", szFnName , stiLocal.sSnapInfo.dwFrameCount );
      }

      /* Check for Callback failure status.. */
      else if( ASL_is_err ( SNAP_CallbackGetStatus( stiLocal.sSnapInfo ) ) )
      {
         teStatus = SNAP_CallbackGetStatus( stiLocal.sSnapInfo );
      }

      else
      {
         /* Replace with image processing code as required */
         GUI_DisplayImage( &stiLocal.sGuiInfo,  stiLocal.sSnapInfo.dwDisplayBufferNumber );

         stiLocal.sSnapInfo.dwDisplayBufferNumber = ( stiLocal.sSnapInfo.dwDisplayBufferNumber + 1 ) % NUM_VID_IMAGES;
         stiLocal.sSnapInfo.dwDisplayCount++;   
      }

      /* Handle user input */
      if( !ASL_is_err( teStatus ) )
         if( ( teStatus = GUI_CheckMessageQueue( &stiLocal.sGuiInfo, &pseEventRoot ) ) == ASL_OK )
            /* Handle list of events returned */
            teStatus = ConfigSnapperDynamic( &stiLocal.sGuiInfo, pseEventRoot );
   
   }
   
   /* Stop interrupts & un-register callbacks */
   DIG16_set_interrupts( stiLocal.sSnapInfo.hSnapper, DIG16_INT_INIT, FALSE );
   DIG16_set_callback(   stiLocal.sSnapInfo.hSnapper, DIG16_CALLBACK_INIT, NULL, NULL); 

   /* Ensure that the Callback is not executing before tidying up.  This is only significant
    * if there is a system modal pop-up error message still active which has been generated
    * by the Callback routine.
    */
   while ( SNAP_CallbackGetActive( stiLocal.sSnapInfo ) == TRUE )
   {}

   /* If the user has terminated the processing loop by pressing 'Q',
    * then tidy up the last acquisition.  If the acquisition has already been tidied,
    * repeated calls to this function will not cause any harm.
    */
   DIG16_get_capture_status( stiLocal.sSnapInfo.hSnapper, DIG16_START_AND_WAIT );
   
   /* Destroy semaphore */
   ASL_Sem_Destroy( &stiLocal.sSemInfo );

   _UNUSED_VAR( teImageStatus );

#else
#if ! defined __MWERKS__
#error "CallBacks are only Implemented on WindowsNT, Solaris, VxWorks and LynxOS"
#endif
#endif

#else/* No loop acquisition */
#if ! defined __MWERKS__
#error "No Loop Type Defined"
#endif
#endif

   /* Get timing */
   ASL_Thrd_Gettime( &stiLocal.sSnapInfo.dFinish );
   
   /* Write image to file. Filetype determined by definition at top of file.
    * Insert file conversion routine here to guarantee writeability of TIFF images
    */
   if( stiLocal.sSnapInfo.dwFrameCount > 0 )
      GUI_TmgWriteFile( &stiLocal.sGuiInfo, szFnName, gdwFileFormat, (stiLocal.sSnapInfo.dwFrameCount - 1) % NUM_VID_IMAGES );
   else
      DBG_WriteString( 10, "%s: no images to write to file \n", szFnName );

   
   /* Image acquisition rate calculation */
   GUI_DisplayRates( &stiLocal );

   DBG_WriteString( 10 | XPL_DBG_PROMPT, "%s: Program completed OK\n", szFnName ); 
   TidyAndExit( &stiLocal );
   return 0;
} /* End Main */


/*
 * Interrupt callback function. Any image processing is
 * done in the foreground process - the ISR here is used to signal back
 * to the foreground. This routine is called by a high priority driver thread
 * and so has very low latency.
 */
void EXPORT_FN MyCallback(Thandle hSnapper, ui32 dwIntSrc, void *pMyData)
{
   struct Info *psInfo = (struct Info*) pMyData;
   ui32 dwNextBufferNumber;

   SNAP_CallbackSetActive( psInfo->sSnapInfo, TRUE );
   
   DBG_WriteString( 6, "Callback: IntSrc = %08lx, frame %d, buffer %d\n", dwIntSrc, psInfo->sSnapInfo.dwFrameCount, psInfo->sSnapInfo.dwFrameBufferNumber );   
   
   /* Have we finished yet? */
   if ( SNAP_FramesFinished( psInfo->sSnapInfo, gdwFramesRequired, psInfo->sSnapInfo.dwFrameCount ) )
   {
      /* Clean up last (possibly only) acquisition. */
      DIG16_get_capture_status( psInfo->sSnapInfo.hSnapper, DIG16_START_AND_WAIT );
      
      /* Signal main process */
      ASL_Sem_Signal( &psInfo->sSemInfo, 1 );
   }

   /* No...there's still more data to acquire */
   else
   {
      /* Keep track of the nulber of frames acquired */
      psInfo->sSnapInfo.dwFrameCount++;

      /* Clean up previous acquisition. There should be no penalty from using START_AND_WAIT
       * because the interrupt calling this function tells us that the acquisition is complete.
       */
      if (DIG16_get_capture_status( psInfo->sSnapInfo.hSnapper, DIG16_START_AND_WAIT ) == DIG16_CAPTURE_COMPLETE )
      {
         /* Avoid acquiring into the image buffer currently being used by the display routine 
          * by checking which buffer it is using. If we are about to lap its position in the circular
          * buffer stack, acquire into the last used buffer & DONT signal, otherwise our
          * signal queue gets infinitely deep because the display routine cannot keep up.
          */
         dwNextBufferNumber = ( psInfo->sSnapInfo.dwFrameBufferNumber + 1) % NUM_VID_IMAGES;
         if ( dwNextBufferNumber != psInfo->sSnapInfo.dwDisplayBufferNumber )
         {
            psInfo->sSnapInfo.dwFrameBufferNumber = dwNextBufferNumber;
            ASL_Sem_Signal( &psInfo->sSemInfo, 1 );
         }

         /* Begin next read */
         DIG16_capture_to_image( psInfo->sSnapInfo.hSnapper, GUI_TmgGetImage( &psInfo->sGuiInfo, psInfo->sSnapInfo.dwFrameBufferNumber ), DIG16_START_AND_RETURN );
      }
      else
      {
         SNAP_CallbackSetStatus( psInfo->sSnapInfo, ASL_ERROR );
         ASL_Sem_Signal( &psInfo->sSemInfo, 1 );
      }
   }

   SNAP_CallbackSetActive( psInfo->sSnapInfo, FALSE );
   
   _UNUSED_VAR( hSnapper );
}


/* Add messages as required. Check message translation function in Gui.c for
 * the current list, expand there as required also.
 */
Terr ConfigSnapperDynamic( struct GuiInfo* psGui, struct Event *pseEventRoot )
{
   static char *szFnName = "ConfigSnapperDynamic";
   struct Event* pseEventPtr = pseEventRoot;
   struct Event* pseLastEventPtr;
   Terr teStatus = ASL_OK;

   DBG_WriteString( 1, "\n%s: Entering..", szFnName); 
   while ( pseEventPtr != NULL )
   {
      DBG_WriteString( 1, "\n%s: dwCmd = [0x%x]", szFnName, pseEventPtr->dwCmd);
      switch( pseEventPtr->dwCmd )
      {
         case 'Q': 
         case 'q':teStatus = ASL_ERROR;
            break;
         default :
            break;
      }
      pseLastEventPtr = pseEventPtr;
      pseEventPtr = pseEventPtr->pseNextEvent;
      free( pseLastEventPtr );
   }

   _UNUSED_VAR( psGui );

   return teStatus;
}


/* Tidy up all the resources and then exit
 */
void TidyAndExit( struct Info *psInfo )
{
   BASE_destroy( psInfo->sSnapInfo.hBase );
   GUI_TmgClose( &psInfo->sGuiInfo );
   GUI_DisplayClose( &psInfo->sGuiInfo );
   DBG_Close();
   exit(0);
}
