/**********************************************************************
 *                ACTIVE SILICON LIMITED / DATACELL LIMITED           *
 *  File name   : SnapScan.c                                          *
 *  Function    : Utility to Scan for all available Snapper boards    *
 *  Project     : ASLexamp                                            *
 *  Author      : Mike Harrison                                       *
 *  Systems     : Ansi C - OS independent                             *
 *  Written     : 24.Feb.98                                           *
 *  Updated     :                                                     *
 *  Version     :                                                     *
 *                                                                    *
 *  Copyright (c) 1993-99 Active Silicon Ltd                          *
 **********************************************************************
 * Comments:
 * This utility scans for all available Snapper boards and displays all
 * available boards information.
 * It also displays library revision information.
 * This utility only requires console I/O, and henece there are no compiler
 * switches.
 */

/* Active Silicon Include files */
#include <asl_inc.h>
#include <xpl_inc.h>



/* Local Protrotypes
 * *****************
 */
void DisplayBaseInfo( Thandle hBase );
void DisplaySnapperInfo( Thandle hBase, Thandle hSnapper );
void DisplayLibraryInfo( void );


/* Global Variables
 * ****************
 */
ui32 gdwDebugLevel = 10;


int main ( int argc, char* argv[] )
{
#define MAX_BOARDS 9

   static char *szVersion="4.00.00";
   static char *szCopyright="\nCopyright (C) 1998-1999 Active Silicon Ltd, Uxbridge.";
   static char *szFnName = "SnapScan";

   struct Info stiLocal;         /* Snapper / Imaging variables */
   ui16 aBaseAddress[MAX_BOARDS] = {
         0x300, 0x320, 0x340,                               /* ISA Devives */
         BASE_DEVICE | 1, BASE_DEVICE | 2, BASE_DEVICE | 3, /* PCI or SBus Devices */
         BASE_DEVICE | 4, BASE_DEVICE | 5, BASE_DEVICE | 6};
//         BASE_AUTO, BASE_AUTO, BASE_AUTO, /* PCI or SBus Devices */
//         BASE_AUTO, BASE_AUTO, BASE_AUTO};
   ui16 *pBaseAddress;
   int nBoard, nMaxBoards;
   int nNumBaseDevices = 0;
   int nNumSnapperDevices = 0;
   Thandle hBase, hSnapper;
   Tboolean EnableErrorHandler = FALSE;


   /* Initialise port for debug output */
   DBG_Open( szFnName );


   /* Header message */
   DBG_WriteString( 10, "\n%s", szCopyright );
   DBG_WriteString( 10, "\nSnapper Information Utility, version: %s", szVersion);


   /* Parse command-line options.
    * The only parameter of interest is the baseboard address.  Set that to the default
    * value before parsing the program arguments.  If the user has set a specific
    * base address, dwBoardID will be overwritten by GUI_SetCmdParams.
    */
   stiLocal.sSnapInfo.dwBoardID = BASE_AUTO;
   if( ASL_is_err( GUI_SetCmdParams( &stiLocal, (ui16)argc, argv) ) )
   {
      DBG_WriteString( 10 | XPL_DBG_PROMPT, "%s: Failed to parse command line args\n", szFnName);
      DBG_Close();
      exit (0);
   }
   DBG_WriteString( 1, "\n%s: Board Base Address = 0x%08x", szFnName, stiLocal.sSnapInfo.dwBoardID);


   /* If the user has specified a particular board address, then just check that one.
    * Also the user probably expects the board to exist, therefore do not turn
    * off error reporting during the initialisation.
    */
   if (stiLocal.sSnapInfo.dwBoardID != BASE_AUTO)
   {
      DBG_WriteString( 10 , "\n%s: Just scanning for board at address 0x%08x", szFnName, stiLocal.sSnapInfo.dwBoardID);
      EnableErrorHandler = TRUE;
      pBaseAddress = (ui16 *) &stiLocal.sSnapInfo.dwBoardID;
      nMaxBoards = 1;
   }
   else
   {
      DBG_WriteString( 10 , "\n%s: Scanning for all available devices", szFnName);
      /* Attempt to open all available Base devices */
      pBaseAddress = aBaseAddress;
      nMaxBoards = MAX_BOARDS;
   }


   for (nBoard = 0; nBoard < nMaxBoards; nBoard++)
   {
      /* Explicity turn off error handling.
       * The reason being that we repeatedly ask the device driver for the next 
       * available Snapper device. When there are no more to assign, the device 
       * driver will return an error, which gets displayed. 
       * Since the error doesn't mean anything to us at the moment, we turn it off.
       * However, if there are problems detecting cards known to exist, turn it on 
       * to help debugging.
       */
      GUI_EnableErrorHandler( EnableErrorHandler );
      hBase = BASE_create( *pBaseAddress );
      GUI_EnableErrorHandler( TRUE );

      DBG_WriteString( 1, "\nhBase = 0x%08x, for address = 0x%08x", hBase, *pBaseAddress );
      if ( ASL_is_ok( hBase )  )
      {
         nNumBaseDevices++;
         DBG_WriteString( 10, "\n\nBoard information for #%d:", nNumBaseDevices );
         DisplayBaseInfo( hBase );
         if ( ( hSnapper = (Thandle) BASE_get_parameter( hBase, BASE_MODULE_HANDLE ) ) != MODULE_NULL_HANDLE )
         {
            nNumSnapperDevices++;
            DisplaySnapperInfo( hBase, hSnapper );
         }
      }

      pBaseAddress++;
   }

   DisplayLibraryInfo();

   /* Destroy all base device instances */
   BASE_destroy( BASE_ALL_HANDLES );

   DBG_WriteString( 10 | XPL_DBG_PROMPT, "\n");
   DBG_Close();
   return (0);
}


void DisplayBaseInfo( Thandle hBase )
{
#define MAX_PROPERTIES  8
#define ISA_PROPERTIES  0
#define PCI_PROPERTIES  1
#define SBUS_PROPERTIES 2
   static char *pszPropertyNames[][MAX_PROPERTIES] = {
/* ISA */   { "\0"},
/* PCI */   { "mapdate", "mapfpga_name", "mapfpga", "micro", "micro_revlevel", "revlevel", "snapclk", "\0"},
/* SBus */  { "mapfpga", "mapfpga_name", "mapfpga", "micro", "micro_revlevel", "revlevel", "snapclk", "\0"} };
   char szPropertyValue[256];
   char *pszCurrentProperty;
   int  nPropertyIndex = 0;
   int  nBoardIndex = 0;
   int  nBaseBoardType;


   DBG_WriteString( 10, "\n\tsnapper type          : " );
   nBaseBoardType = (int) BASE_get_parameter( hBase, BASE_BASEBOARD_TYPE );
   switch ( nBaseBoardType )
   {
      case (int) BASE_ISA_BIB:
         DBG_WriteString( 10, "ISA-BIB" );
         nBoardIndex = ISA_PROPERTIES;
         break;
      case (int) BASE_ISA_JPG:
         DBG_WriteString( 10, "ISA-JPEG" );
         nBoardIndex = ISA_PROPERTIES;
         break;
      case (int) BASE_PCI_BIB:
         DBG_WriteString( 10, "PCI-BIB (or PMC)" );
         nBoardIndex = PCI_PROPERTIES;
         break;
      case (int) BASE_PCI_JPG:
         DBG_WriteString( 10, "PCI-JPEG (or PMC)" );
         nBoardIndex = PCI_PROPERTIES;
         break;
      case (int) BASE_SBUS_BIB:
         DBG_WriteString( 10, "SBus-BIB" );
         nBoardIndex = PCI_PROPERTIES;
         break;
      default:
         DBG_WriteString( 10, "Unknown\n" );
         break;
   }

   pszCurrentProperty = pszPropertyNames[nBoardIndex][nPropertyIndex];
   while ( *pszCurrentProperty != 0 )
   {
      DBG_WriteString( 1, "\nSearching for \"%s\" property", pszCurrentProperty);
      if ( BASE_get_property( hBase, pszCurrentProperty, szPropertyValue ) == ASL_OK )
         DBG_WriteString( 10, "\n\t%-21s : %s", pszCurrentProperty, szPropertyValue);

      nPropertyIndex++;
      pszCurrentProperty = pszPropertyNames[nBoardIndex][nPropertyIndex];
   }
}


void DisplaySnapperInfo( Thandle hBase, Thandle hSnapper )
{
   ui8 IDRev;

   DBG_WriteString(  4, "\n\tbase handle           : %d", hBase );
   DBG_WriteString(  4, "\n\tsnapper handle        : %d", hSnapper );
   DBG_WriteString( 10, "\n\tsnapper type          : " );
   IDRev =  (ui8) ASL_get_ret ( BASE_get_parameter( hBase,  BASE_MODULE_IDREV_VALUE) );
   switch ( (int) ASL_get_ID( IDRev ) )
   {
      case (int) SNP16_ID:     DBG_WriteString( 10, "Snapper-16" );        break;
      case (int) SNP24_ID:     DBG_WriteString( 10, "Snapper-24" );        break;
      case (int) SNP8_ID:      DBG_WriteString( 10, "Snapper-8" );         break;
      case (int) PMC_SNP24_ID: DBG_WriteString( 10, "Snapper-PMC-24" );    break;
      case (int) PMC_SNP8_ID:  DBG_WriteString( 10, "Snapper-PMC-8" );     break;
      case (int) SNP_PCI24_ID: DBG_WriteString( 10, "Snapper-PCI-24" );    break;
      case (int) SNP_PCI8_ID:  DBG_WriteString( 10, "Snapper-PCI-8" );     break;
      case (int) DIG16_ID:     DBG_WriteString( 10, "Snapper-Dig16" );     break;                                                 
      case (int) PMC_DIG16_ID: DBG_WriteString( 10, "Snapper-PMC-Dig16" ); break;                                                 
      default:                 DBG_WriteString( 10, "Unknown\n" );         break;
   }

   DBG_WriteString( 10, "\n\tsnapper revlevel      : 0x%02x", (int) IDRev );
}


void DisplayLibraryInfo( void )
{
   ui32 dwFPGARevLevel;
   ui32 dwParam;
   Thandle hDisplay;

   DBG_WriteString( 10, "\n\nSoftware version information:" );

   /* Null call to software to get software version number in first BIB struct*/
   DBG_WriteString( 10, "\n\tSnapper DLL version   : %d", (ui16) (ui32) BASE_get_parameter( BASE_NULL_HANDLE, BASE_LIBRARY_REV_LEVEL ) );
   
   FPGA_get_rev_level( &dwFPGARevLevel);
   DBG_WriteString( 10, "\n\tFPGA version          : %ld", dwFPGARevLevel );

   DBG_WriteString( 10, "\n\tTMG Imaging Library   : %ld", (long) TMG_image_get_parameter( TMG_NULL, TMG_LIBRARY_REV_LEVEL ) );


   /* Get display / environment information & print it*/
   DBG_WriteString( 10, "\n\nEnvironment information:" );

   hDisplay = TMG_display_create();

   /* Display pixel format */
   dwParam = TMG_display_get_parameter( hDisplay, TMG_PIXEL_FORMAT );
   DBG_WriteString( 10, "\n\tDisplay Pixel Format  : ");
   switch ((int) dwParam )
   {
      case (int) TMG_RGB15:  DBG_WriteString( 10, "RGB15");   break;
      case (int) TMG_RGB16:  DBG_WriteString( 10, "RGB16");   break;
      case (int) TMG_BGR24:  DBG_WriteString( 10, "BGR24");   break;
      case (int) TMG_BGRX32: DBG_WriteString( 10, "BGRX32");  break;
      default:               DBG_WriteString( 10, "Unknown"); break;
   }

   /* DD/DCI support */
   dwParam = TMG_display_get_parameter( hDisplay, TMG_DISPLAY_DIRECT_CAPS ); 
   DBG_WriteString( 10, "\n\tDirectDraw/DCI        : %s", ((int) dwParam)? "ON":"OFF" );

   /*close down screen support again*/
   TMG_display_destroy(hDisplay);
}

