/**********************************************************************
 *                ACTIVE SILICON LIMITED / DATACELL LIMITED           *
 *  File name   : d16ansq.c                                           *
 *  Function    : Snapper-SNP16 areascan                              * 
 *  Project     : ASLexamp                                            *
 *  Author      : Mike Harrison                                       *
 *  Systems     : Ansi C - OS independent                             *
 *  Written     : 24.Feb.98                                           *
 *  Updated     :                                                     *
 *  Version     :                                                     *
 *                                                                    *
 *  Copyright (c) 1993-99 Active Silicon Ltd                          *
 **********************************************************************
 * Comments:
 * This program demonstrates the set up & operation of Snapper-16.  It
 * demonstrates all the possible modes of acquisition - from single shot
 * to interrupt driven.
 * 
 * Operating System Compiler Switch.  Options are :
 *   _DOS32     -
 *   _LYNXOS    - LynxOS 2.5 on _X86 or _PPC
 *   _MACINTOSH - Macintosh System 7.5 and later pn PowerPCs using MetroWerks
 *                CodeWarrior 9 and later.
 *   _SOLARIS2  - Solaris 2.6 and above.
 *   _VXWORKS   - VxWorks on PowerPC.
 *   _WIN95     - Windows 95 (must also define _WINDOWS).
 *   _WINNT     - Windows NT4 (must also define _WINDOWS).
 *
 * Display type compiler switches.  Options are:
 *   _CONSOLE_ONLY     - No image display, just messages sent to stdout.
 *                       This over-rides all other display modes.
 *   _X_GRAPHICS       - Use X Graphics imaging and control.  This is
 *                       currently supported under _SOLARIS2, _LYNX and
 *                       _VXWORKS.
 *  _WIN32_DISPLAY_DDB - Use Device Dependent Bitmap (DDB) display method
 *                       under _WIN95 or _WINNT.
 *  _WIN32_DISPLAY_DIB - Use Device Independent Bitmap (DIB) display method
 *                       under _WIN95 or _WINNT.
 *  _WIN32_DISPLAY_DDI - Use DirectDraw Interface (DDI) display method under
 *                       _WIN95 or _WINNT.
 *
 * Acquisition Method compiler switches.  Options are:
 *  _SINGLE_IMAGE - acquire single image then quit
 *  _SYNC_LOOP    - blocked acquisition followed by image display.  This is
 *                  the simplest method but is not effecient as the
 *                  acquisition time cannot be used for image processing.
 *  _ASYNC_LOOP   - start asynchronous acquisition, display image then
 *                  complete acquisition.  This is more effecient than the
 *                  _SYNC_LOOP method as the previous image can be processed
 *                  whilst the current image is acquired.
 *
 *
 * The following global variables provide additional control:
 *
 * gdwDebugLevel - set in the range from 1 (most info) to 10 (least info) to
 *                 control the amount of debug information output to stdout.
 *
 * gdwCameraFormat - set the camera output format.  Select from
 *                   SNP16_FORMAT_YUV422 - full YUV422 colour acquisition
 *                   SNP16_FORMAT_MONO8  - 8 bit mono acquisition
 *
 * gdwHwSubSample - set the hardware sub sample ratio for the acquisition.
 *
 * gdwFramesRequired - set the number of frames of video data to acquire.
 *                     (A value of 0 is infinite).
 *
 * gdwSemWait - set the semaphore timeout period.
 *
 * gdwFileFormat - set the file format of the image to be saved.  The last
 *                 acquired image is always saved before the application
 *                 terminates.
 */


/* Active Silicon Include files */
#include <asl_inc.h>
#include <xpl_inc.h>


/* Select acquisition method */
/*#define _SYNC_LOOP    */
#define _ASYNC_LOOP   


/* Global Variables
 * ****************
 */
ui32 gdwFramesRequired = 0;                    /* Specify number of frames to acquire - 0 is infinite */
ui32 gdwSemWait = 5;                           /* Timeout period on semaphores */
ui32 gdwCameraFormat = SNP16_FORMAT_YUV422;     /* Specify camera format */
ui32 gdwHwSubSample = SNP16_SUB_X2;            /* Specify the hardware image sub-sampling ratio required. */
ui32 gdwFileFormat = TMG_TIFF;                 /* Specify the dump image file type */
ui32 gdwDebugLevel = 10;                       /* Set triviality level of debug messages */


/* Local Function Prototypes
 * *************************
 */
Terr ConfigSnapperDynamic( struct GuiInfo*, struct Event* );
void TidyAndExit( struct Info *psInfo );


int main( ui16 argc, char** argv)
{
   static char *szVersion="4.00.00";
   static char *szCopyright="\nCopyright (C) 1998-1999 Active Silicon Ltd, Uxbridge.";
   static char *szFnName = "S16ansq";
   
   struct Info stiLocal;         /* Snapper / Imaging variables */
   struct Event *pseEventRoot;   /* Event queue message list */
   ui32 dwBufferNumber;          /* Current Buffer ID */
   Terr teStatus;                /* Snapper status flag */
   Terr teImageStatus;           /* Image status flag */


   /* Initialise port for debug output */
   DBG_Open( szFnName );


   /* Header message */
   DBG_WriteString( 10, "\n%s", szCopyright );
   DBG_WriteString( 10, "\nSnapper SNP16 area-scan image capture utility, version: %s", szVersion);


   /* Turn on the library error reporting */
   GUI_EnableErrorHandler( TRUE );


   /* Initialise the GUI display info structure */
   if ( ASL_is_err( GUI_DisplayInitInfo( &stiLocal.sGuiInfo, szFnName ) ) )
   {
      DBG_WriteString( 10 | XPL_DBG_PROMPT, "\n%s: Failed to configure GUI info structure", szFnName);
      TidyAndExit( &stiLocal );
   }

   /* SnapInfo structure init.
    */
   if( ASL_is_err ( GUI_SnapInit( &stiLocal.sSnapInfo ) ) )
   {
      DBG_WriteString( 10 | XPL_DBG_PROMPT, "\n%s: Failed to initialise SnapInfo structure", szFnName);
      TidyAndExit( &stiLocal );
   }

   /* Parse command-line options */
   if( ASL_is_err( GUI_SetCmdParams( &stiLocal, (ui16)argc, argv) ) )
   {
      DBG_WriteString( 10 | XPL_DBG_PROMPT, "\n%s: Failed to parse command line args", szFnName);
      TidyAndExit( &stiLocal );
   }


   /* TMG init.
    * Snapper absolutely requires the TMG imaging library just to store 
    * image data. Whether or not you use it for further processing is up to you. 
    */
   if( ASL_is_err ( GUI_TmgInit( &stiLocal.sGuiInfo ) ) )
   {
      DBG_WriteString( 10 | XPL_DBG_PROMPT, "\n%s: Failed to initialise TMG image structures", szFnName);
      TidyAndExit( &stiLocal );
   }



   /* Init display. 
    * Window sizing parameters from the command line are automatically processed.
    */
   if( ASL_is_err( GUI_DisplayInit( &stiLocal.sGuiInfo ) ) )
   {
      DBG_WriteString( 10 | XPL_DBG_PROMPT, "\n%s: Failed to initialise GUI display", szFnName);
      TidyAndExit( &stiLocal );
   }

   if ( ASL_is_err( stiLocal.sSnapInfo.hBase = BASE_create( (ui16) stiLocal.sSnapInfo.dwBoardID ) ) )
   {
      DBG_WriteString( 10 | XPL_DBG_PROMPT, "\n%s: Failed to configure Baseboard and Snapper", szFnName);
      TidyAndExit( &stiLocal );
   }

   if ( ( stiLocal.sSnapInfo.hSnapper = BASE_get_parameter( stiLocal.sSnapInfo.hBase, BASE_MODULE_HANDLE )) == MODULE_NULL_HANDLE )
   {
      DBG_WriteString( 10 | XPL_DBG_PROMPT, "\n%s: Failed to initialize Snapper - probably not fitted", szFnName, 0);
      TidyAndExit( &stiLocal );
   }

   /* Test to check that the correct Snapper personality module is fitted. This
    * program drives a SNP16 so we only want a PCI/ISA SNP16.
    */
   stiLocal.sSnapInfo.wModuleID = (ui16) ASL_get_ID( BASE_get_parameter( stiLocal.sSnapInfo.hBase, BASE_MODULE_FAMILY ) );
   if ( stiLocal.sSnapInfo.wModuleID != SNP16_ID )
   {
      DBG_WriteString( 10 | XPL_DBG_PROMPT, "\n%s: Incorrect Snapper module fitted (should be SNP16 family member)", szFnName);
      TidyAndExit( &stiLocal );
   }

   /* Apply user specified options */
   SNP16_set_subsample( stiLocal.sSnapInfo.hSnapper, (ui16) gdwHwSubSample );


   /* Snapper-16 only supports YUV422 or MONO formats.
    * For the YUV422 formats, there is no datamapper available, therefore the application
    * must convert the image in software before display.
    */
   stiLocal.sGuiInfo.wDisplayFormat = ASL_get_ret( TMG_display_get_parameter( stiLocal.sGuiInfo.hDisplay, TMG_PIXEL_FORMAT ) );
   if (gdwCameraFormat == SNP16_FORMAT_YUV422)
   {
      SNP16_set_format( stiLocal.sSnapInfo.hSnapper, gdwCameraFormat, TMG_YUV422 );
      stiLocal.sGuiInfo.fConvertForDisplay = TRUE;
   }
   else
      SNP16_set_format( stiLocal.sSnapInfo.hSnapper, gdwCameraFormat, stiLocal.sGuiInfo.wDisplayFormat );

   
   /* Store the image sizes and Lock the memory down to save time. */
   if (ASL_is_err( SNP16_capture( stiLocal.sSnapInfo.hSnapper ) ) )
   {
      DBG_WriteString( 10 | XPL_DBG_PROMPT, "\n%s: Initial capture failed", szFnName);
      TidyAndExit( &stiLocal );
   }

   for( dwBufferNumber = 0; dwBufferNumber < NUM_VID_IMAGES; dwBufferNumber++ )
   {
      SNP16_set_image( stiLocal.sSnapInfo.hSnapper, GUI_TmgGetImage( &stiLocal.sGuiInfo, dwBufferNumber ) );
      SNP16_read_video_data( stiLocal.sSnapInfo.hSnapper, GUI_TmgGetImage( &stiLocal.sGuiInfo, dwBufferNumber ), TMG_RUN );
      teStatus = TMG_image_set_flags( GUI_TmgGetImage( &stiLocal.sGuiInfo, dwBufferNumber ), TMG_LOCKED, TRUE);
   }

   /* Resize the window to be the full image
    */
   if( ASL_is_err( GUI_DisplayReSize( &stiLocal.sGuiInfo ) ) )
   {
      DBG_WriteString( 10 | XPL_DBG_PROMPT, "\n%s: Failed to ReSize the GUI display", szFnName);
      TidyAndExit( &stiLocal );
   }


#if defined _SYNC_LOOP
/***********************/

   DBG_WriteString( 10, "\n%s: Entering synchronous imaging loop", szFnName );
      
   /* Start timing */
   ASL_Thrd_Gettime( &stiLocal.sSnapInfo.dStart );

   /* Acquire images */
   while ( !ASL_is_err(teStatus) && !SNAP_FramesFinished( stiLocal.sSnapInfo, gdwFramesRequired, stiLocal.sSnapInfo.dwFrameCount ) )
   {
      /* Capture the image... */
      if ( ASL_is_ok( teStatus = SNP16_capture( stiLocal.sSnapInfo.hSnapper ) ) )
      {
         /* ..Then read the video data */
         teStatus = SNP16_read_video_data( stiLocal.sSnapInfo.hSnapper, GUI_TmgGetImage( &stiLocal.sGuiInfo, stiLocal.sSnapInfo.dwFrameBufferNumber ), TMG_RUN );
         stiLocal.sSnapInfo.dwFrameBufferNumber = ( stiLocal.sSnapInfo.dwFrameBufferNumber + 1 ) % NUM_VID_IMAGES;

         if ( ASL_is_ok( teStatus ) )
         {
            /* Having acquired the next image, display it */
            teStatus = GUI_DisplayImage( &stiLocal.sGuiInfo, stiLocal.sSnapInfo.dwDisplayBufferNumber );

            stiLocal.sSnapInfo.dwDisplayBufferNumber = ( stiLocal.sSnapInfo.dwDisplayBufferNumber + 1) %  NUM_VID_IMAGES;
            stiLocal.sSnapInfo.dwFrameCount++;
            stiLocal.sSnapInfo.dwDisplayCount++;
         }
      }
      
      /* Handle user input */
      if( ASL_is_ok ( teStatus ) )
         if ( ASL_is_ok( teStatus = GUI_CheckMessageQueue( &stiLocal.sGuiInfo, &pseEventRoot ) ) )
            teStatus =  ConfigSnapperDynamic( &stiLocal.sGuiInfo, pseEventRoot );
   }

   _UNUSED_VAR( teImageStatus );


#elif defined _ASYNC_LOOP
/************************/

   DBG_WriteString( 10, "\n%s: Entering asynchronous imaging loop", szFnName );
   teImageStatus = ASL_OK;

//   if (ASL_is_err( teStatus = SNP16_capture( stiLocal.sSnapInfo.hSnapper ) ) )
   if (ASL_is_err( teStatus = SNP16_start_capture( stiLocal.sSnapInfo.hSnapper ) ) )
   {
      DBG_WriteString( 10, "\n%s: Capture failed", szFnName );
      TidyAndExit( &stiLocal );
   }   

   ASL_Thrd_Gettime( &stiLocal.sSnapInfo.dStart );
   
   /* Acquire images */
   while ( !ASL_is_err( teStatus ) && !SNAP_FramesFinished( stiLocal.sSnapInfo, gdwFramesRequired, stiLocal.sSnapInfo.dwFrameCount ) )
   {
      /* Wait for the previous acquisition to complete */
      while ( !SNP16_is_capture_complete( stiLocal.sSnapInfo.hSnapper ) )
      {}

      teStatus = SNP16_read_video_data( stiLocal.sSnapInfo.hSnapper, GUI_TmgGetImage( &stiLocal.sGuiInfo, stiLocal.sSnapInfo.dwFrameBufferNumber ), TMG_RUN );
      stiLocal.sSnapInfo.dwFrameBufferNumber = ( stiLocal.sSnapInfo.dwFrameBufferNumber + 1 ) % NUM_VID_IMAGES;

      if ( ASL_is_ok ( teStatus ) )
         teImageStatus = SNP16_start_capture( stiLocal.sSnapInfo.hSnapper );

      if ( ASL_is_ok( teStatus ) )
      {
         GUI_DisplayImage( &stiLocal.sGuiInfo, stiLocal.sSnapInfo.dwDisplayBufferNumber );

         stiLocal.sSnapInfo.dwDisplayBufferNumber = ( stiLocal.sSnapInfo.dwDisplayBufferNumber + 1) %  NUM_VID_IMAGES;
         stiLocal.sSnapInfo.dwFrameCount++;
         stiLocal.sSnapInfo.dwDisplayCount++;
      }

      /* Handle user input */
      if ( ASL_is_ok ( teStatus ) )
         if ( ASL_is_ok( teStatus = GUI_CheckMessageQueue( &stiLocal.sGuiInfo, &pseEventRoot ) ) )
            teStatus =  ConfigSnapperDynamic( &stiLocal.sGuiInfo, pseEventRoot );
   }


#else/* No loop acquisition */
#if ! defined __MWERKS__
#error "No Loop Type Defined"
#endif
#endif


   /* Get timing */
   ASL_Thrd_Gettime( &stiLocal.sSnapInfo.dFinish );

   /* Write image to file. Filetype determined by definition at top of file.
    * Insert file conversion routine here to guarantee writeability of TIFF images
    */
   if( stiLocal.sSnapInfo.dwFrameCount > 0 )
      GUI_TmgWriteFile( &stiLocal.sGuiInfo, szFnName, gdwFileFormat, (stiLocal.sSnapInfo.dwFrameCount - 1) % NUM_VID_IMAGES );
   else
      DBG_WriteString( 10, "\n%s: no images to write to file", szFnName );

   /* Image acquisition rate calculation */
   GUI_DisplayRates( &stiLocal );

   DBG_WriteString( 10 | XPL_DBG_PROMPT, "\n%s: Program completed OK", szFnName ); 
   TidyAndExit( &stiLocal );
   return 0;
} /* End Main */


/* Add messages as required. Check message translation function in Gui.c for
 * the current list, expand there as required also.
 */
Terr ConfigSnapperDynamic( struct GuiInfo* psGuiInfo, struct Event *pseEventRoot )
{
   static char *szFnName = "ConfigSnapperDynamic";
   struct Event* pseEventPtr = pseEventRoot;
   struct Event* pseLastEventPtr;
   Terr teStatus = ASL_OK;

   DBG_WriteString( 1, "\n%s: Entering..", szFnName); 
   while ( pseEventPtr != NULL )
   {
      DBG_WriteString( 1, "\n%s: dwCmd = [0x%x]", szFnName, pseEventPtr->dwCmd);
      switch( pseEventPtr->dwCmd )
      {
         case 'Q': 
         case 'q':teStatus = ASL_ERROR;
            break;
         default :
            break;
      }
      pseLastEventPtr = pseEventPtr;
      pseEventPtr = pseEventPtr->pseNextEvent;
      free( pseLastEventPtr );
   }

   _UNUSED_VAR( psGuiInfo );

   return teStatus;
}


/* Tidy up all the resources and then exit
 */
void TidyAndExit( struct Info *psInfo )
{
   BASE_destroy( psInfo->sSnapInfo.hBase );
   GUI_TmgClose( &psInfo->sGuiInfo );
   GUI_DisplayClose( &psInfo->sGuiInfo );
   DBG_Close();
   exit(0);
}


