/**********************************************************************
 *                ACTIVE SILICON LIMITED / DATACELL LIMITED           *
 *  File name   : d16ansq.c                                           *
 *  Function    : Snapper-SNP24 areascan                              * 
 *  Project     : ASLexamp                                            *
 *  Author      : Mike Harrison                                       *
 *  Systems     : Ansi C - OS independent                             *
 *  Written     : 24.Feb.98                                           *
 *  Updated     :                                                     *
 *  Version     :                                                     *
 *                                                                    *
 *  Copyright (c) 1993-99 Active Silicon Ltd                          *
 **********************************************************************
 * Comments:
 * This program demonstrates the set up & operation of Snapper-Dig16.  It
 * demonstrates all the possible modes of acquisition - from single shot
 * to interrupt driven.
 * 
 * Operating System Compiler Switch.  Options are :
 *   _DOS32     -
 *   _LYNXOS    - LynxOS 2.5 on _X86 or _PPC
 *   _MACINTOSH - Macintosh System 7.5 and later pn PowerPCs using MetroWerks
 *                CodeWarrior 9 and later.
 *   _SOLARIS2  - Solaris 2.6 and above.
 *   _VXWORKS   - VxWorks on PowerPC.
 *   _WIN95     - Windows 95 (must also define _WINDOWS).
 *   _WINNT     - Windows NT4 (must also define _WINDOWS).
 *
 *
 * Display type compiler switches.  Options are:
 *   _CONSOLE_ONLY     - No image display, just messages sent to stdout.
 *                       This over-rides all other display modes.
 *   _X_GRAPHICS       - Use X Graphics imaging and control.  This is
 *                       currently supported under _SOLARIS2, _LYNX and
 *                       _VXWORKS.
 *  _WIN32_DISPLAY_DDB - Use Device Dependent Bitmap (DDB) display method
 *                       under _WIN95 or _WINNT.
 *  _WIN32_DISPLAY_DIB - Use Device Independent Bitmap (DIB) display method
 *                       under _WIN95 or _WINNT.
 *  _WIN32_DISPLAY_DDI - Use DirectDraw Interface (DDI) display method under
 *                       _WIN95 or _WINNT.
 *
 * Acquisition Method compiler switches.  Options are:
 *  _SINGLE_IMAGE - acquire single image then quit
 *  _SYNC_LOOP    - blocked acquisition followed by image display.  This is
 *                  the simplest method but is not effecient as the
 *                  acquisition time cannot be used for image processing.
 *  _ASYNC_LOOP   - start asynchronous acquisition, display image then
 *                  complete acquisition.  This is more effecient than the
 *                  _SYNC_LOOP method as the previous image can be processed
 *                  whilst the current image is acquired.
 *  _CALLBACKS    - independent asynchronous acquisition and image display.
 *                  An interrupt function completes the asynchronous
 *                  acquisition, starts the next and then signals to a
 *                  separate thread that an image is available for display.
 *                  As this method doesn't rely on the processing time being
 *                  less than the acquisition time (as in the _ASYNC_LOOP
 *                  method), this method can be used when the AVERAGE
 *                  processing time is less than the AVERAGE acquisition
 *                  time.
 * 
 * 
 * The following global variables provide additional control:
 *
 * gdwDebugLevel - set in the range from 1 (most info) to 10 (least info) to
 *                 control the amount of debug information output to stdout.
 *
 * gdwCameraType - set the specific camera type.  Select between
 *                 SNP24_EIA_DEFAULT and SNP24_CCIR_DEFAULT.
 *
 * gdwRequiredBits - set the number of bits of data to acquire from the
 *                   video source.
 *
 * gdwCameraFormat - set the camera output format.  Select from
 *                   SNP24_FORMAT_RGB - full RGB acquisition
 *                   SNP24_FORMAT_Y8_ON_RED - mono acquisition on the red input channel
 *                   SNP24_FORMAT_Y8_ON_GRN - mono acquisition on the green input channel
 *                   SNP24_FORMAT_Y8_ON_BLU - mono acquisition on the blue input channel
 *
 * gdwSyncSource - set the required sync source.  Select from
 *                 SNP24_SYNC_OFF_RED1 - sync from Red #1 input
 *                 SNP24_SYNC_OFF_GRN1 - sync from Green #1 input
 *                 SNP24_SYNC_0FF_VIDEO - sync from Green input of currently selected video channel
 *                 SNP24_SYNC_OFF_CSYNC_NEG - sync from negative edge of CSync input
 *
 * gdwHwSubSample - set the hardware sub sample ratio for the acquisition.
 *
 * gdwFramesRequired - set the number of frames of video data to acquire.
 *                     (A value of 0 is infinite).
 *
 * gdwSemWait - set the semaphore timeout period.
 *
 * gdwFileFormat - set the file format of the image to be saved.  The last
 *                 acquired image is always saved before the application
 *                 terminates.
 */


/* Active Silicon Include files */
#include <asl_inc.h>
#include <xpl_inc.h>


/* Select acquisition method */
/*#define _SINGLE_IMAGE */
/*#define _SYNC_LOOP    */
#define _ASYNC_LOOP   
/*#define _CALLBACKS    */


/* Global Variables
 * ****************
 */
/* Specify the camera type we expect to be using with this program.
 * The full list can be found in DIG16ini.c, supplied with the SDK.
 */
ui32 gdwCameraType = SNP24_EIA_DEFAULT;        /* Alternatively SNP24_CCIR_DEFAULT */
ui32 gdwFramesRequired = 0;                    /* Specify number of frames to acquire - 0 is infinite */
ui32 gdwSemWait = 5;                           /* Timeout period on semaphores */
ui32 gdwRequiredBits = 8;                      /* Digital camera so data width can be specified up to max of 16 bits */
ui32 gdwCameraFormat = SNP24_FORMAT_RGB;       /* Specify camera format */
ui32 gdwHwSubSample = SNP24_SUB_X1;            /* Specify the hardware image sub-sampling ratio required. */
//ui32 gdwSyncSource = SNP24_SYNC_OFF_VIDEO;   /* Specify sync source */
ui32 gdwSyncSource = SNP24_SYNC_OFF_CSYNC_NEG; /* Specify sync source */
ui32 gdwFileFormat = TMG_TIFF;                 /* Specify the dump image file type */
ui32 gdwDebugLevel = 10;                       /* Set triviality level of debug messages */


/* Local Function Prototypes
 * *************************
 */
void EXPORT_FN MyCallback(Thandle hSnapper, ui32 dwIntSrc, void *pMyData);
Terr ConfigSnapperDynamic( struct GuiInfo*, struct Event* );
void TidyAndExit( struct Info *psInfo );


int main( ui16 argc, char** argv)
{
   static char *szVersion="4.00.00";
   static char *szCopyright="\nCopyright (C) 1998-1999 Active Silicon Ltd, Uxbridge.";
   static char *szFnName = "S24ansq";
   
   struct Info stiLocal;         /* Snapper / Imaging variables */
   struct Event *pseEventRoot;   /* Event queue message list */
   ui32 dwBufferNumber;          /* Current Buffer ID */
   Terr teStatus;                /* Snapper status flag */
   Terr teImageStatus;           /* Image status flag */


   /* Initialise port for debug output */
   DBG_Open( szFnName );


   /* Header message */
   DBG_WriteString( 10, "\n%s", szCopyright );
   DBG_WriteString( 10, "\nSnapper SNP24 area-scan repeated single image capture utility, version: %s", szVersion);


   /* Turn on the library error reporting */
   GUI_EnableErrorHandler( TRUE );


   /* Initialise the GUI display info structure */
   if ( ASL_is_err( GUI_DisplayInitInfo( &stiLocal.sGuiInfo, szFnName ) ) )
   {
      DBG_WriteString( 10 | XPL_DBG_PROMPT, "\n%s: Failed to configure GUI info structure", szFnName);
      TidyAndExit( &stiLocal );
   }


   /* SnapInfo structure init.
    */
   if( ASL_is_err ( GUI_SnapInit( &stiLocal.sSnapInfo ) ) )
   {
      DBG_WriteString( 10 | XPL_DBG_PROMPT, "\n%s: Failed to initialise SnapInfo structure", szFnName);
      TidyAndExit( &stiLocal );
   }

   /* Parse command-line options */
   if( ASL_is_err( GUI_SetCmdParams( &stiLocal, (ui16)argc, argv) ) )
   {
      DBG_WriteString( 10 | XPL_DBG_PROMPT, "\n%s: Failed to parse command line args", szFnName);
      TidyAndExit( &stiLocal );
   }


   /* TMG init.
    * Snapper absolutely requires the TMG imaging library just to store 
    * image data. Whether or not you use it for further processing is up to you. 
    */
   if( ASL_is_err ( GUI_TmgInit( &stiLocal.sGuiInfo ) ) )
   {
      DBG_WriteString( 10 | XPL_DBG_PROMPT, "\n%s: Failed to initialise TMG image structures", szFnName);
      TidyAndExit( &stiLocal );
   }


   /* Init display. 
    * Window sizing parameters from the command line are automatically processed.
    */
   if( ASL_is_err( GUI_DisplayInit( &stiLocal.sGuiInfo ) ) )
   {
      DBG_WriteString( 10 | XPL_DBG_PROMPT, "\n%s: Failed to initialise GUI display", szFnName);
      TidyAndExit( &stiLocal );
   }


   if ( ASL_is_err( stiLocal.sSnapInfo.hBase = BASE_create( (ui16) stiLocal.sSnapInfo.dwBoardID ) ) )
   {
      DBG_WriteString( 10 | XPL_DBG_PROMPT, "\n%s: Failed to configure Baseboard and Snapper", szFnName);
      TidyAndExit( &stiLocal );
   }

   if ( ( stiLocal.sSnapInfo.hSnapper = BASE_get_parameter( stiLocal.sSnapInfo.hBase, BASE_MODULE_HANDLE )) == MODULE_NULL_HANDLE )
   {
      DBG_WriteString( 10 | XPL_DBG_PROMPT, "\n%s: Failed to initialize Snapper - probably not fitted", szFnName, 0);
      TidyAndExit( &stiLocal );
   }

   /* Test to check that the correct Snapper personality module is fitted. This
      program drives a SNP24 so we only want a PCI/ISA SNP24 or a PMC SNP24. 
   */
   /* Test to check that the correct Snapper personality module is fitted. This
    * program drives a DIG16 so we only want a PCI/ISA DIG16 or a PMC DIG16. 
    */
   stiLocal.sSnapInfo.wModuleID = (ui16) ASL_get_ID( BASE_get_parameter( stiLocal.sSnapInfo.hBase, BASE_MODULE_FAMILY ) );
   if ( stiLocal.sSnapInfo.wModuleID != SNP24_ID )
   {
      DBG_WriteString( 10 | XPL_DBG_PROMPT, "\n%s: Incorrect Snapper module fitted (should be SNP24 family member)", szFnName);
      TidyAndExit( &stiLocal );
   }

   /* Specific camera-type initialisation.
    * Internally calls set_format with SNP24_FORMAT_INIT arg, 
    * so if the board is a SNAP-8, snapper data format is set to Y8_ON_RED 
    * with a TMG output format of TMG_Y8 else 
    * if a SNAP-24, snapper data format is set to RGB with 
    * TMG output format of RGBX32.
    */
   if ( ASL_is_err ( SNP24_initialize( stiLocal.sSnapInfo.hSnapper, gdwCameraType ) ) )
   {
      DBG_WriteString( 10 | XPL_DBG_PROMPT, "\n%s: Failed to initialise Snapper-24", szFnName );
      TidyAndExit( &stiLocal );
   }
   DBG_WriteString( 10, "\n%s: Snapper-24 Initialised OK", szFnName );
   
   /* Apply user specified options */
   SNP24_set_sync( stiLocal.sSnapInfo.hSnapper, gdwSyncSource );
   SNP24_set_capture( stiLocal.sSnapInfo.hSnapper, gdwHwSubSample );


   /* Set the camera to display mapping format, using the hardware datamapper
    * feature.  For display update speed we will attempt to match the camera
    * output format to the display format.  However this will mean that the
    * saved image will not be the full colour depth for non true colour displays.
    * If SNP24_set_format returns an error, it means that there is not a hardware
    * mapping available to convert the camera format to the display format.  In
    * this case, each video image will need to be converted prior to display.
    */
   stiLocal.sGuiInfo.wDisplayFormat = ASL_get_ret( TMG_display_get_parameter( stiLocal.sGuiInfo.hDisplay, TMG_PIXEL_FORMAT ) );
   GUI_EnableErrorHandler( FALSE );
   if (ASL_is_err( SNP24_set_format( stiLocal.sSnapInfo.hSnapper, gdwCameraFormat, stiLocal.sGuiInfo.wDisplayFormat ) ) )
      stiLocal.sGuiInfo.fConvertForDisplay = TRUE;
   GUI_EnableErrorHandler( TRUE );


   /* Store the image sizes and Lock the memory down to save time. */
   if (ASL_is_err( SNP24_capture( stiLocal.sSnapInfo.hSnapper, SNP24_START_AND_WAIT ) ) )
   {
      DBG_WriteString( 10 | XPL_DBG_PROMPT, "\n%s: Initial capture failed", szFnName );
      TidyAndExit( &stiLocal );
   }
   for( dwBufferNumber = 0; dwBufferNumber < NUM_VID_IMAGES; dwBufferNumber++ )
   {
      SNP24_set_image( stiLocal.sSnapInfo.hSnapper, GUI_TmgGetImage( &stiLocal.sGuiInfo, dwBufferNumber ) );
      SNP24_read_video_data( stiLocal.sSnapInfo.hSnapper, GUI_TmgGetImage( &stiLocal.sGuiInfo, dwBufferNumber ), TMG_RUN );
      TMG_image_set_flags( GUI_TmgGetImage( &stiLocal.sGuiInfo, dwBufferNumber ), TMG_LOCKED, TRUE);
   }
   
   /* Resize the window to be the full image
    */
   if( ASL_is_err( GUI_DisplayReSize( &stiLocal.sGuiInfo ) ) )
   {
      DBG_WriteString( 10 | XPL_DBG_PROMPT, "\n%s: Failed to ReSize the GUI display", szFnName);
      TidyAndExit( &stiLocal );
   }


#if defined _SINGLE_IMAGE   
/************************/

   DBG_WriteString( 10, "\n%s: Entering single image acquisition", szFnName );
   
   /* Start timing */
   ASL_Thrd_Gettime( &stiLocal.sSnapInfo.dStart );

   teStatus = SNP24_capture( stiLocal.sSnapInfo.hSnapper, SNP24_START_AND_WAIT);
   if( teStatus != ASL_OK )
   {
      DBG_WriteString( 10 | XPL_DBG_PROMPT, "\n%s: Capture_to_image failed", szFnName );
      TidyAndExit( &stiLocal );
   }
   else
   {
      stiLocal.sSnapInfo.dwFrameCount++;
      stiLocal.sSnapInfo.dwDisplayCount++;
   }

   _UNUSED_VAR( pseEventRoot );
   _UNUSED_VAR( teImageStatus );


#elif defined _SYNC_LOOP
/***********************/

   DBG_WriteString( 10, "\n%s: Entering synchronous imaging loop", szFnName );
      
   /* Start timing */
   ASL_Thrd_Gettime( &stiLocal.sSnapInfo.dStart );

   /* Acquire images */
   while ( !ASL_is_err(teStatus) && !SNAP_FramesFinished( stiLocal.sSnapInfo, gdwFramesRequired, stiLocal.sSnapInfo.dwFrameCount ) )
   {
      /* Start the next capture */
      teStatus = SNP24_capture( stiLocal.sSnapInfo.hSnapper, SNP24_START_AND_WAIT);
      teStatus = SNP24_read_video_data( stiLocal.sSnapInfo.hSnapper, GUI_TmgGetImage( &stiLocal.sGuiInfo, stiLocal.sSnapInfo.dwDisplayBufferNumber ), TMG_RUN );
      
      /* Having acquired the next image, display it */
      GUI_DisplayImage( &stiLocal.sGuiInfo, stiLocal.sSnapInfo.dwDisplayBufferNumber );

      stiLocal.sSnapInfo.dwDisplayBufferNumber = ( stiLocal.sSnapInfo.dwDisplayBufferNumber + 1) %  NUM_VID_IMAGES;
      stiLocal.sSnapInfo.dwFrameCount++;
      stiLocal.sSnapInfo.dwDisplayCount++;

      /* Handle user input */
      if( !ASL_is_err( teStatus ) )
         teStatus = GUI_CheckMessageQueue( &stiLocal.sGuiInfo, &pseEventRoot );
      if( !ASL_is_err( teStatus ) )
       /* Handle list of events returned */
         teStatus =  ConfigSnapperDynamic( &stiLocal.sGuiInfo, pseEventRoot );
   }
   _UNUSED_VAR( teImageStatus );

   
#elif defined _ASYNC_LOOP
/************************/

   DBG_WriteString( 10, "\n%s: Entering asynchronous imaging loop", szFnName );
   teImageStatus = ASL_OK;

   /* Acquire an initial image to display, whilst the next image is being acquired */
   teStatus = SNP24_capture( stiLocal.sSnapInfo.hSnapper, SNP24_START_AND_WAIT);
   if( teStatus != ASL_OK )
   {
      DBG_WriteString( 10, "\n%s: Capture_to_image failed", szFnName );
      TidyAndExit( &stiLocal );
   }

   ASL_Thrd_Gettime( &stiLocal.sSnapInfo.dStart );
   
   /* Acquire images */
   while ( !ASL_is_err( teStatus ) && !SNAP_FramesFinished( stiLocal.sSnapInfo, gdwFramesRequired, stiLocal.sSnapInfo.dwFrameCount ) )
   {
      /* Start acquiring the next frame */
      stiLocal.sSnapInfo.dwFrameBufferNumber = ( stiLocal.sSnapInfo.dwFrameBufferNumber + 1 ) % NUM_VID_IMAGES;
      teStatus = SNP24_capture( stiLocal.sSnapInfo.hSnapper, SNP24_START_AND_RETURN );
  
      /* While the Snapper is busy acquiring the next image, display the current image */
      GUI_DisplayImage( &stiLocal.sGuiInfo, stiLocal.sSnapInfo.dwDisplayBufferNumber );

      stiLocal.sSnapInfo.dwDisplayBufferNumber = ( stiLocal.sSnapInfo.dwDisplayBufferNumber + 1) %  NUM_VID_IMAGES;
      stiLocal.sSnapInfo.dwFrameCount++;
      stiLocal.sSnapInfo.dwDisplayCount++;

      /* Handle user input */
      teStatus = GUI_CheckMessageQueue( &stiLocal.sGuiInfo, &pseEventRoot );
      if( !ASL_is_err( teStatus ) )
         /* Handle list of events returned */
         teStatus = ConfigSnapperDynamic( &stiLocal.sGuiInfo, pseEventRoot );
      
      /* Get around to checking and cleaning up. If the loop is fast enough, we get here and wait, hopefully briefly.
       * If its not, then again the speed of acquisition is limited by the display/image processing call.
       */
      while ( ! SNP24_is_capture_complete( stiLocal.sSnapInfo.hSnapper ) ) 
      {}
      teImageStatus = SNP24_read_video_data( stiLocal.sSnapInfo.hSnapper, GUI_TmgGetImage( &stiLocal.sGuiInfo, stiLocal.sSnapInfo.dwFrameBufferNumber ), TMG_RUN  );
      
      if ( ASL_is_err( teImageStatus ) )
         teStatus = ASL_ERROR;
   }
   
#elif defined _CALLBACKS
/************************/

#if defined _WINNT || defined _SOLARIS2 || defined _VXWORKS || defined _LYNXOS
   DBG_WriteString( 10, "\n%s: Entering asynchronous imaging loop using callbacks", szFnName );
   
   /* Create Semaphore */
   if( ASL_is_err( ASL_Sem_Create( &stiLocal.sSemInfo, 0, 10 ) ) )
   {
      DBG_WriteString( 10, "\n%s: Sem_Create failed", szFnName );
      TidyAndExit( &stiLocal );
   }

   /* Enable interrupts & register our callback function */
   SNAP_CallbackSetActive( stiLocal.sSnapInfo, FALSE );
   SNAP_CallbackSetStatus( stiLocal.sSnapInfo, ASL_OK );
   SNP24_set_callback(   stiLocal.sSnapInfo.hSnapper, SNP24_CALLBACK_SET, MyCallback, &stiLocal ); 
   SNP24_set_interrupts( stiLocal.sSnapInfo.hSnapper, SNP24_INT_DATA_READY, TRUE);
   
   ASL_Thrd_Gettime( &stiLocal.sSnapInfo.dStart );
   
   /* Begin capture */
   teStatus = SNP24_capture( stiLocal.sSnapInfo.hSnapper, SNP24_START_AND_RETURN);
   if( teStatus != ASL_OK )
   {
      DBG_WriteString( 10, "\n%s: Capture_to_image failed", szFnName );
      TidyAndExit( &stiLocal );
   }
   else
   {
      /* No need to clean up with get_capture_status after the first asynch acquisition
       * because we use the callback function to do this immediately it gets notified 
       * an image has been acquired.
       */
      DBG_WriteString( 8, "\n%s: Initial image acquired OK", szFnName );
   }

   /* The use of callbacks has an extra advantage over the first two methods, by completely 
    * disconnecting the image processing loop from the acquisition loop, the acquisition can
    * proceed at full speed, even if the processing lags behind. The next image it processes
    * ( in this implementation ) will be the most recent image acquired. This does however mean
    * that some images, although acquired, will not be processed and so are lost. Get a faster 
    * display/processing routine if this is the case.
    */
   while ( !ASL_is_err( teStatus ) && !SNAP_FramesFinished( stiLocal.sSnapInfo, gdwFramesRequired, stiLocal.sSnapInfo.dwFrameCount ) )
   {
      /* Wait for a buffer to be filled by the callback. */
      teStatus = ASL_Sem_WaitWithTimeout( &stiLocal.sSemInfo, gdwSemWait );
      if( ASL_is_err( teStatus ) )
      {
         DBG_WriteString( 10, "\n%s: Failed to wait on callback capture %d", szFnName , stiLocal.sSnapInfo.dwFrameCount );
      }

      /* Check for Callback failure status.. */
      else if( ASL_is_err ( SNAP_CallbackGetStatus( stiLocal.sSnapInfo ) ) )
      {
         teStatus = SNAP_CallbackGetStatus( stiLocal.sSnapInfo );
      }

      else
      {
         /* Replace with image processing code as required */
         GUI_DisplayImage( &stiLocal.sGuiInfo,  stiLocal.sSnapInfo.dwDisplayBufferNumber );

         stiLocal.sSnapInfo.dwDisplayBufferNumber = ( stiLocal.sSnapInfo.dwDisplayBufferNumber + 1 ) % NUM_VID_IMAGES;
         stiLocal.sSnapInfo.dwDisplayCount++;   
      }

      /* Handle user input */
      if( !ASL_is_err( teStatus ) )
         if( ( teStatus = GUI_CheckMessageQueue( &stiLocal.sGuiInfo, &pseEventRoot ) ) == ASL_OK )
            /* Handle list of events returned */
            teStatus = ConfigSnapperDynamic( &stiLocal.sGuiInfo, pseEventRoot );
   
   }
   
   /* Stop interrupts & un-register callbacks */
   SNP24_set_interrupts( stiLocal.sSnapInfo.hSnapper, SNP24_INT_INIT, FALSE );
   SNP24_set_callback(   stiLocal.sSnapInfo.hSnapper, SNP24_CALLBACK_INIT, NULL, NULL); 

   /* Ensure that the Callback is not executing before tidying up.  This is only significant
    * if there is a system modal pop-up error message still active which has been generated
    * by the Callback routine.
    */
   while ( SNAP_CallbackGetActive( stiLocal.sSnapInfo ) == TRUE )
   {}

   /* Destroy semaphore */
   ASL_Sem_Destroy( &stiLocal.sSemInfo );

   _UNUSED_VAR( teImageStatus );

#else
#if ! defined __MWERKS__
#error "CallBacks are only Implemented on WindowsNT, Solaris, VxWorks and LynxOS"
#endif
#endif

#else/* No loop acquisition */
#if ! defined __MWERKS__
#error "No Loop Type Defined"
#endif
#endif

   /* Get timing */
   ASL_Thrd_Gettime( &stiLocal.sSnapInfo.dFinish );
   
   /* Write image to file. Filetype determined by definition at top of file.
    * Insert file conversion routine here to guarantee writeability of TIFF images
    */
   if( stiLocal.sSnapInfo.dwFrameCount > 0 )
      GUI_TmgWriteFile( &stiLocal.sGuiInfo, szFnName, gdwFileFormat, (stiLocal.sSnapInfo.dwFrameCount - 1) % NUM_VID_IMAGES );
   else
      DBG_WriteString( 10, "\n%s: no images to write to file", szFnName );
   
   /* Image acquisition rate calculation */
   GUI_DisplayRates( &stiLocal );

   DBG_WriteString( 10 | XPL_DBG_PROMPT, "\n%s: Program completed OK", szFnName ); 
   TidyAndExit( &stiLocal );
   return 0;
} /* End Main */


/*
 * Interrupt callback function. Any image processing is
 * done in the foreground process - the ISR here is used to signal back
 * to the foreground. This routine is called by a high priority driver thread
 * and so has very low latency.
 */
void EXPORT_FN MyCallback(Thandle hSnapper, ui32 dwIntSrc, void *pMyData)
{
   struct Info* psInfo = (struct Info*) pMyData;
   ui32 dwNextBufferNumber;

   SNAP_CallbackSetActive( psInfo->sSnapInfo, TRUE );
   
   DBG_WriteString( 6, "Callback: IntSrc = %08lx, frame %d, buffer %d\n", dwIntSrc, psInfo->sSnapInfo.dwFrameCount, psInfo->sSnapInfo.dwFrameBufferNumber );   
   
   /* Have we finished yet? */
   if ( SNAP_FramesFinished( psInfo->sSnapInfo, gdwFramesRequired, psInfo->sSnapInfo.dwFrameCount ) )
   {
      /* Clean up last (possibly only) acquisition. */
      SNP24_read_video_data( psInfo->sSnapInfo.hSnapper, GUI_TmgGetImage( &psInfo->sGuiInfo, psInfo->sSnapInfo.dwFrameBufferNumber ), TMG_RUN );
      
      /* Signal main process */
      ASL_Sem_Signal( &psInfo->sSemInfo, 1 );
   }

   /* No...there's still more data to acquire */
   else
   {
      /* Keep track of the number of frames acquired */
      psInfo->sSnapInfo.dwFrameCount++;

      /* Clean up previous acquisition. There should be no penalty from using START_AND_WAIT
       * because the interrupt calling this function tells us that the acquisition is complete.
       */
      if (ASL_is_ok( SNP24_read_video_data( psInfo->sSnapInfo.hSnapper, GUI_TmgGetImage( &psInfo->sGuiInfo, psInfo->sSnapInfo.dwFrameBufferNumber ), TMG_RUN ) ) )
      {

         /* Avoid acquiring into the image buffer currently being used by the display routine 
          * by checking which buffer it is using. If we are about to lap its position in the circular
          * buffer stack, acquire into the last used buffer & DONT signal, otherwise our
          * signal queue gets infinitely deep because the display routine cannot keep up.
          */
         dwNextBufferNumber = ( psInfo->sSnapInfo.dwFrameBufferNumber + 1) % NUM_VID_IMAGES;
         if ( dwNextBufferNumber != psInfo->sSnapInfo.dwDisplayBufferNumber )
         {
            psInfo->sSnapInfo.dwFrameBufferNumber = dwNextBufferNumber;
            ASL_Sem_Signal( &psInfo->sSemInfo, 1 );
         }

         /* Begin next read */
         SNP24_capture( psInfo->sSnapInfo.hSnapper, SNP24_START_AND_RETURN );
      }
      else
      {
         SNAP_CallbackSetStatus( psInfo->sSnapInfo, ASL_ERROR );
         ASL_Sem_Signal( &psInfo->sSemInfo, 1 );
      }
   }

   SNAP_CallbackSetActive( psInfo->sSnapInfo, FALSE );
}


/* Add messages as required. Check message translation function in Gui.c for
 * the current list, expand there as required also.
 */
Terr ConfigSnapperDynamic( struct GuiInfo* psGuiInfo, struct Event *pseEventRoot )
{
   static char *szFnName = "ConfigSnapperDynamic";
   struct Event* pseEventPtr = pseEventRoot;
   struct Event* pseLastEventPtr;
   Terr teStatus = ASL_OK;

   DBG_WriteString( 1, "\n%s: Entering..", szFnName); 
   while ( pseEventPtr != NULL )
   {
      DBG_WriteString( 1, "\n%s: dwCmd = [0x%x]", szFnName, pseEventPtr->dwCmd);
      switch( pseEventPtr->dwCmd )
      {
         case 'Q': 
         case 'q':teStatus = ASL_ERROR;
            break;
         default :
            break;
      }
      pseLastEventPtr = pseEventPtr;
      pseEventPtr = pseEventPtr->pseNextEvent;
      free( pseLastEventPtr );
   }

   _UNUSED_VAR( psGuiInfo );

   return teStatus;
}


/* Tidy up all the resources and then exit
 */
void TidyAndExit( struct Info *psInfo )
{
   BASE_destroy( psInfo->sSnapInfo.hBase );
   GUI_TmgClose( &psInfo->sGuiInfo );
   GUI_DisplayClose( &psInfo->sGuiInfo );
   DBG_Close();
   exit(0);
}
