/****************************************************************************
 *
 * ACTIVE SILICON LIMITED
 *
 * File name   : comp.c
 * Function    : JPEG compression utility.
 * Project     : TMG
 * Authors     : Colin Pearce
 * Systems     : ANSI C, Various.
 * Version     : See release history below.
 * Release Date: See release history below.
 *
 * Copyright (c) 1993-2002 Active Silicon Ltd.
 ****************************************************************************
 * 
 * File History
 * ------------
 * v1.0, 03-Sep-99, HCP.
 *       Copied from old Snapper directory and tidied up.
 *
 * v1.1, 17-Apr-02, HCP.
 *       Minor tidy ups, and generation of MSVC 6 makefile.
 *
 *
 * Comments:
 * --------
 * This program shows how to use the TMG library to JPEG compress a TIFF
 * file and save it as a JPEG/JFIF file.
 *
 * Typical Benchmarks using Symantec C v6.1 (raw data throughput in Mb/sec):
 * Grayscale: 486-66: 0.21  P90: 0.42  P133: 0.65
 * Colour:    486-66: 0.21  P90: 0.43  P133: 0.65
 *
 * And on a P450: 3.57Mb/s (compression rate without file I/O).
 *
 **************************************************************************
 */
#include <asl_inc.h>

int _SnapperDebugLevel = 2; /* for DEBUG */

int main(int argc, char** argv)
{
   Thandle hInImage, hImage1, hJpegImage;
   i32 nQfactor = 32;                 /* Default */
   clock_t dwStartTime, dwEndTime;    /* For timing information  */
   clock_t dwStartComp, dwCompTime;
   ui32 dwLinesPerStrip = 8;
   float   fTotalData;


   printf("\nSoftware JPEG Compression Utility - v1.1");
   printf("\nActive Silicon Ltd, UK.\n");


   if ( argc != 3 )
   {
      printf("\nIncorrect command line parameters\n\n");
      printf("Usage: COMP <input_filename> <output_filename>\n");
      exit(0);
   }


   /* Create image structures
    */
   hInImage = TMG_image_create();
   hImage1  = TMG_image_create();
   hJpegImage = TMG_JPEG_image_create();

   TMG_image_set_infilename(hInImage, argv[1]);
   TMG_image_set_outfilename(hInImage, argv[2]);

   TMG_JPEG_set_Quality_factor(hJpegImage, (ui16)nQfactor);


   /* Compress the image
    */
   dwStartTime = clock();

  /* Read the whole image into memory (work out height 
   * automatically on read.
   */
   TMG_image_set_parameter(hInImage, TMG_HEIGHT, TMG_AUTO_HEIGHT);
   TMG_image_read(hInImage, TMG_NULL, TMG_RUN);
#if 0
   /* Convert to RGB24 or Y8 */
   TMG_image_convert(hInImage, hImage1, TMG_Y8_OR_RGB24, 0, TMG_RUN);
   TMG_image_move(hImage1, hInImage);
#endif
   dwStartComp = clock();

   TMG_image_set_flags(hInImage, TMG_LOCKED, TRUE); /* saves re-malloc'ing data internally */
   TMG_JPEG_compress(hInImage, hJpegImage, TMG_RUN);

   dwCompTime = clock() - dwStartComp;

   TMG_image_write(hJpegImage, TMG_NULL, TMG_JPEG, TMG_RUN);


   dwEndTime = clock() - dwStartTime;

   printf("Done\n");

   printf("\nSpeed Statistics");
   printf("\n----------------\n");
   printf("Total time (including file I/O).....: %1.2f secs\n", (float) (dwEndTime - dwStartTime)/ (float) CLOCKS_PER_SEC);

   fTotalData = (float) ( ( 
         TMG_image_get_parameter(hInImage, TMG_BYTES_PER_LINE) *
         TMG_image_get_parameter(hInImage, TMG_HEIGHT) ) +
         TMG_image_get_parameter(hJpegImage, TMG_JPEG_NUM_BYTES_DATA) ) /
         (float) 1000000.0;

   printf("Throughput (including file I/O).....: %1.2f Mb/sec\n\n", fTotalData / ((float) (dwEndTime - dwStartTime)/ (float) CLOCKS_PER_SEC) );
   printf("Compression time (memory to memory).........: %1.2f secs\n", (float) dwCompTime / (float) CLOCKS_PER_SEC);
   printf("Compression throughput (memory to memory)...: %1.2f Mb/sec\n\n", fTotalData / ((float) dwCompTime / (float) CLOCKS_PER_SEC) );


   /*
    * Free the memory used.
    */
   TMG_image_destroy(TMG_ALL_HANDLES);

   return 0;
}
