/****************************************************************************
 *
 * ACTIVE SILICON LIMITED
 *
 * File name   : decomp.c
 * Function    : JPEG decompression utility.
 * Project     : TMG
 * Authors     : Colin Pearce
 * Systems     : ANSI C, Various.
 * Version     : See release history below.
 * Release Date: See release history below.
 *
 * Copyright (c) 1993-2002 Active Silicon Ltd.
 ****************************************************************************
 * 
 * File History
 * ------------
 * v1.0, 03-Sep-99, HCP
 *       Copied from old Snapper directory and tidied up.
 *
 * v1.1, 17-Apr-02, HCP.
 *       Minor tidy ups, and generation of MSVC 6 makefile. 
 * 
 *
 * Comments:
 * --------
 * This program shows how to use the TMG library to decompress a JPEG
 * file and save it as a TIFF file.
 *
 * Typical Benchmarks using Symantec C v6.1 (raw data throughput in Mb/sec):
 * Grayscale: 486-66: 0.22  P90: 0.51  P133: 0.77
 * Colour:    486-66: 0.27  P90: 0.62  P133: 1.01
 *
 */
#include <asl_inc.h>

int _SnapperDebugLevel = 2; /* for DEBUG */

int main(int argc, char** argv)

{
   Thandle hJpegImage, hOutImage;
   clock_t dwStartTime, dwEndTime;           /* for timing information */
   clock_t dwStartDecomp, dwDecompTime;
   float fTotalData;


   printf("\nSoftware JPEG Decompression Utility - v1.1");
   printf("\nActive Silicon Ltd, UK.\n");


   if ( argc != 3 )
   {
      printf("\nIncorrect command line parameters\n\n");
      printf("Usage : DECOMP <input_filename> <output_filename>\n");
      exit(0);
   }

   printf("\nProcessing image - memory to memory...");

   dwStartTime = clock();

   /*
    * Create image structures
    */
   hJpegImage = TMG_JPEG_image_create();
   hOutImage  = TMG_image_create();
   
   TMG_image_set_infilename(hJpegImage, argv[1]);
   TMG_image_set_outfilename(hJpegImage, argv[2]);

   /* Read = read the whole file into memory.
    */   
   if ( TMG_JPEG_file_read(hJpegImage) != TMG_OK )
   {
      printf("DECOMP: Failed to read in file %s\n", argv[1]);
      exit(0);
   }

  /* Decompress the image.
   */
   dwStartDecomp = clock();
   TMG_JPEG_decompress(hJpegImage, hOutImage, TMG_RUN);
   dwDecompTime = clock() - dwStartDecomp;

  /* Save the whole image.
   */
   TMG_image_set_parameter(hOutImage, TMG_LINES_THIS_STRIP, (ui16) TMG_image_get_parameter(hOutImage, TMG_HEIGHT));
   TMG_write_TIFF_file(hOutImage, TMG_RUN);

   dwEndTime = clock() - dwStartTime;

   printf("\n\nSpeed Statistics");
   printf("\n----------------\n");
   printf("Total time (including file I/O)....: %1.2f secs\n", (float) (dwEndTime - dwStartTime)/ (float) CLOCKS_PER_SEC);

   fTotalData = (float) ( (
         TMG_image_get_parameter(hOutImage, TMG_BYTES_PER_LINE) *
         TMG_image_get_parameter(hOutImage, TMG_HEIGHT) ) +
         TMG_image_get_parameter(hJpegImage, TMG_JPEG_NUM_BYTES_DATA) ) /
         (float) 1000000.0;

   printf("Throughput (including file I/O)....: %1.2f Mb/sec\n\n", fTotalData / ((float) (dwEndTime - dwStartTime)/ (float) CLOCKS_PER_SEC) );
   printf("Decompression time (memory to memory).........: %1.2f secs\n", (float) dwDecompTime / (float) CLOCKS_PER_SEC);
   printf("Decompression throughput (memory to memory)...: %1.2f Mb/sec\n\n", fTotalData / ((float) dwDecompTime / (float) CLOCKS_PER_SEC) );

   TMG_image_destroy(TMG_ALL_HANDLES);

   return 0;
}

