/**************************************************************************
 *                ACTIVE SILICON LIMITED / ACTIVE IMAGING LIMITED         *
 *                                                                        *
 *  File name   : asl_mac.h                                               *
 *  Function    : Type definitions specifically for Snapper MAC driver.   *
 *  Project     : Snapper                                                 *
 *  Author      : Chris Read, Active Silicon Ltd                          *
 *  Systems     : ANSI C                                                  *
 *  Version     : 3.10.48                                                 *
 *  Release Date: 18-Aug-98                                               *
 *                                                                        *
 *  Copyright (c) 1993-98 Active Silicon Ltd, Active Imaging Ltd.         *
 **************************************************************************
 * Comments:
 * --------
 * This is the Snapper Mac device driver include file for MacOS8.1 and 
 * above.
 *
 * NOTE: Check duplicated definitions are correct (at end of this file).
 *
 * File History
 * ------------
 * 18-Aug-98, CLR, 3.10 Rev.46 First "finished" Mac release - beta
 *  2-Oct-98, CLR/MBB, 3.10 Rev.47 First Mac release
 */

/* The driver interrogates the system to find the maximum amount of RAM available.
 * If this interrogation fails, then the max DMA size is set to a pre-defined value.
 */
#ifndef _SNAPPER_DEFAULT_DMA_SIZE
#define _SNAPPER_DEFAULT_DMA_SIZE (16*1024*1024)
#endif
#ifndef _SNAPPER_DEFAULT_MAX_DMA_SIZE
#define _SNAPPER_DEFAULT_MAX_DMA_SIZE (1024*1024*1024)
#endif
#ifndef _SNAPPER_DEFAULT_MIN_DMA_SIZE
#define _SNAPPER_DEFAULT_MIN_DMA_SIZE _SNAPPER_DEFAULT_DMA_SIZE
#endif


#if defined _MACINTOSH_KERNEL
/* os_sys.h has not been included - hence must define these for the other includes */
typedef char             i8;
typedef unsigned char   ui8;
typedef short           i16;
typedef unsigned short ui16;
typedef long            i32;
typedef unsigned long  ui32;
#endif

/* Driver internal reurn codes - these are only used by the uppoer level Mac
 * framework.
 * Note that real O/S errors are negative numbers.
 */

enum SnapperOSErr {
   kIoBusyStatus = 1,     /* The driver was busy doing omething else - faled */
   kIoQueuedStatus,       /* The driver has deferred completion until later */
   kIoSeqeuenceError      /* The call has been made out of sequence - an Error */
};

/*
 * Snapper Include Files
 * ---------------------
 */
#include <shared.h>
#include <snp_rev.h>

/* The device naming conventions.
 * Note that the Snapper does not have an IEEE1275 open firmware ROM fitted,
 * and thus appears in the registry under the PCI bus<vendor id>,<device id>
 * It is renamed to something more generic on device installation 
 */
#define DRV_MAC_DEVICE_NAME      ".SnapperDriver"
#define DRV_IEEE1275_DEVICE_NAME "pci5341,40" 
#define DRV_PASCAL_MAC_DEVICE_NAME      "\p.SnapperDriver"
#define DRV_PASCAL_IEEE1275_DEVICE_NAME "\ppci5341,40" 

/* It is not possible to do dynamic allocation from inside the driver (except for
 * load/unload and open/close) Therefore must preallocate a DMA table in memory.
 * DMA transfers longer than this will be split into partial transfers. These
 * partial transfers will be remapped by driver specific software interrupts, the
 * timing of which is entirely arbitary (despite what the documentation says! ).
 * This may be a problem for the DIG16 with its small bufferring capability.
 * Changing this value requires that both burstio.c and int.c be recompiled, and
 * the driver re-linked.
 */
#define DRV_MAX_DMA_LENGTH  (64*1024*1024)  /* (Was 8Mb) - Max DMA length - in bytes          */
#define DRV_MAX_PIO_LENGTH  (64*1024)       /* Max programmed I/O length in bytes */

/* The page size is not constant across all Mac platforms. It is either 4096 or
 * 8192 bytes (at present). Thus the size of the DMA table must be determined at
 * load time.
 */


/* Snapper I/O control codes
 * This is what is copied into the csCode entry of a cntrlParam structure.
 * Note that only 16 bits are available - hence cannot use an enum,
 * which would be 32 bits in native PPC
 */
#define   ASL_MAC_IOCTL(n)                         ( (short)(n) )
#define   IOCTL_SNAPPER_OPEN_DEVICE                ASL_MAC_IOCTL(0)
#define   IOCTL_SNAPPER_WRITE_UI8                  ASL_MAC_IOCTL(1)
#define   IOCTL_SNAPPER_READ_UI8                   ASL_MAC_IOCTL(2)
#define   IOCTL_SNAPPER_WRITE_UI16                 ASL_MAC_IOCTL(3)
#define   IOCTL_SNAPPER_READ_UI16                  ASL_MAC_IOCTL(4)
#define   IOCTL_SNAPPER_WRITE_UI32                 ASL_MAC_IOCTL(5)
#define   IOCTL_SNAPPER_READ_UI32                  ASL_MAC_IOCTL(6)
#define   IOCTL_SNAPPER_DYLAN_WRITE_UI32           ASL_MAC_IOCTL(7)
#define   IOCTL_SNAPPER_DYLAN_READ_UI32            ASL_MAC_IOCTL(8)
#define   IOCTL_SNAPPER_FPGA_SEND_INFO             ASL_MAC_IOCTL(9)
#define   IOCTL_SNAPPER_FPGA_SEND_DATA             ASL_MAC_IOCTL(10)
#define   IOCTL_SNAPPER_FPGA_RESET                 ASL_MAC_IOCTL(11)
#define   IOCTL_SNAPPER_FPGA_GET_CONFIG_STATUS     ASL_MAC_IOCTL(12)
#define   IOCTL_SNAPPER_BURST_IN_UI16              ASL_MAC_IOCTL(13)
#define   IOCTL_SNAPPER_BURST_IN_UI32              ASL_MAC_IOCTL(14)
#define   IOCTL_SNAPPER_BURST_IN_UI32_ASYNC        ASL_MAC_IOCTL(15)
#define   IOCTL_SNAPPER_READ_COMPLETE              ASL_MAC_IOCTL(16)
#define   IOCTL_SNAPPER_BURST_IN_UI16_SPECIAL      ASL_MAC_IOCTL(17)
#define   IOCTL_SNAPPER_BURST_OUT_UI16             ASL_MAC_IOCTL(18)
#define   IOCTL_SNAPPER_TABLE_LOAD                 ASL_MAC_IOCTL(19)
#define   IOCTL_SNAPPER_PAGE_WRITE_UI16            ASL_MAC_IOCTL(20)
#define   IOCTL_SNAPPER_BURST_OUT_UI16_SPECIAL     ASL_MAC_IOCTL(21)
#define   IOCTL_SNAPPER_GET_ERROR_STRING           ASL_MAC_IOCTL(22)
#define   IOCTL_SNAPPER_ABORT_OPERATION            ASL_MAC_IOCTL(23)
#define   IOCTL_SNAPPER_TIMEOUT_SET                ASL_MAC_IOCTL(24)
#define   IOCTL_SNAPPER_SERIAL_TX                  ASL_MAC_IOCTL(25)
#define   IOCTL_SNAPPER_SERIAL_RX                  ASL_MAC_IOCTL(26)
#define   IOCTL_SNAPPER_SERIAL_SEND_INFO           ASL_MAC_IOCTL(27)
#define   IOCTL_SNAPPER_SERIAL_GET_INFO            ASL_MAC_IOCTL(28)
#define   IOCTL_SNAPPER_INTERRUPT_SET              ASL_MAC_IOCTL(29)
#define   IOCTL_SNAPPER_INTERRUPT_STATUS           ASL_MAC_IOCTL(30)
#define   IOCTL_SNAPPER_GET_ERROR_CODE             ASL_MAC_IOCTL(31)
#define   IOCTL_SNAPPER_GET_DEBUG                  ASL_MAC_IOCTL(32)
#define   IOCTL_SNAPPER_SET_DEBUG                  ASL_MAC_IOCTL(33)
#define   IOCTL_SNAPPER_GET_MAX_DMA_LENGTH         ASL_MAC_IOCTL(34)

/* The Macintosh driver interface defines an area of 11 short integers for
 * parameter passing between an application and it's driver. This driver
 * uses this space for simple I/O requests (e.g. single read/write operations)
 * It uses the async I/O sharing model for the more complex operations
 */

 
#if PRAGMA_STRUCT_ALIGN
#pragma options align=mac68k
#endif

union SnapperIOParams {
    short MacWords[11];
    struct SnapperRW {
       ui32 dwAddress;
       ui32 dwData;
    } SnapperRW;
    struct SnapperSet {
       ui32 dwReference;
       ui32 dwValue;
    } SnapperSet;
    struct SnapperBurst{
       ui32 dwPortOffset;
       ui32 dwByteCount;
    } SnapperBurst;
    struct SnapperInterruptStatus{
       ui32 dwIntSrcFromDriver;
       ui32 dwBytesTransferred;
       IOCommandID ioCommand;          /* Only used by the driver */
    } SnapperInterruptStatus;
    struct SnapperReadComplete{
       ui32 dwMode;
       ui32 dwStatus;
       IOCommandID ioCommand;          /* Only used by the driver */
    } SnapperReadComplete;
    struct SnapperAbort{
       ui32 dwOperation;
    } SnapperAbort;
    struct SnapperInterruptSet{
       ui32 dwControlAddress;
       ui32 dwStatusAddress;
       ui32 dwData;
    } SnapperInterruptSet;
    struct SnapperTableLoad {
       ui32 hw_addr_offset;
       ui32 mode;
       ui32 count;
       ui32 ConfigAlgorithm;
    } SnapperTableLoad;
    struct SnapperCommsInfo {
       ui32 dwByteCount;
       ui32 dwTimeout;
    } SnapperCommsInfo;
    struct SnapperSerialInfoCommon {
       ui32 serial_rx_count;
       ui32 parity_error_count;
       ui32 overflow_error_count;
       ui8 serial_reset_comms;
       ui8 serial_xon_xoff_enable;
       ui8 serial_xon_char;
       ui8 serial_xoff_char;
       ui8 serial_parity_ignore;
       ui8 serial_xon_xoff_status;
   } SnapperSerialInfoCommon;

};



#if defined _MACINTOSH_KERNEL

/*
 * System Include Files
 * --------------------
 */
#include <Devices.h>
#include <DriverServices.h>

typedef Boolean DRV_BOOLEAN;
#define DRV_TRUE true
#define DRV_FALSE false

#if !defined (MAXLONG)
#define MAXLONG ( (ui32)(~0) )
#endif

typedef enum {
    STATUS_SUCCESS,
    STATUS_UNSUCCESSFUL,
    STATUS_NOT_IMPLEMENTED,
    STATUS_UNSUPPORTED,
    STATUS_INTERNAL_ERROR,
    STATUS_INVALID_DEVICE_REQUEST,
    STATUS_INVALID_BUFFER_SIZE,
    STATUS_ADAPTER_HARDWARE_ERROR,
    STATUS_INVALID_PARAMETER
} DRV_STATUS;

/* When installing an interrupt, it is first required to save the curently installed
 * handler etc. information. This must be replaced if the current devive is superseded.
 */
struct InterruptDescriptor
{
  InterruptSetMember	SetMember;            /* The identifier for this set */
  void*					pOldContext;          /* The context of this set */
  InterruptHandler		pFnOldHandler;        /* The installed handler */
  InterruptEnabler		pFnEnable;            /* Called to enable the interupt */
  InterruptDisabler		pFnDisable;           /* Called to disable the interrupt */
};

struct ScatterGather
{
   ui32 dwPhysicalAddress;                    /* Address to load in DMA engine */
   ui32 dwLength;
};

/*
 * A structure to hold device information when the driver first interrogates
 * the system. This structure is stored in the local device information structure
 * and is the parameter passed to SnapperAttachDevice()
 *
struct tDeviceInfo(ForBrief)
 */
struct tDeviceInfo {
   ui32 dwIDrev;            /* Standard Snapper IDrev (Bus Interface Board). */
   ui32 dwBusNum;           /* PCI bus number (0-255) */
   ui32 dwDeviceNumber;     /* For PCI device addressing. A funcion of the
                             * location of the Snapper card */
   ui32 dwFunctionNumber;   /* For PCI device addressing. A function of the
                             * Snapper card (0-7) */
   enum{                    /* Only PCI will be supported in this release */
      DRV_BUS_TYPE_UNDEFINED,
      DRV_BUS_TYPE_ISA,
      DRV_BUS_TYPE_PCI
   } dwBusType;
   ui32 BusAddress;         /* I/O address of board, as written to config space */
   ui32 PhysicalAddress;     /* I/O address of baord as seen from the processor */
   void *VirtualAddress;    /* Corresponding Virtual Address */
   ui32 nIRQLevel;          /* As seen by the processor */
   ui32 nIRQVector;         /* As seen by the processor */
};


/*
 * Information local to each device object
 * ---------------------------------------
 * This information is stored in the device extensions part of the device
 * object.
 *
struct tLocalDeviceInfo(ForBrief)
 */
struct tLocalDeviceInfo {

   struct tLocalDeviceInfo *pNext;          /* pointer to next entry in the list */
   struct tDeviceInfo *pDeviceInfo;         /* Information passed to attach */
   LogicalAddress BaseAddress;              /* As mapped in PPC kernel address space */
   SInt32   OpenCount;                      /* Count of tasks with driver open */
   ui32 Eeprom[BASE_PCI40_EEPROM_LENGTH];   /* Shadow of PCI NVRAM. */
   ui32 dwMagicNum;                         /* Used for assert type sanity checks. */
#define DRV_MAGIC_NUMBER  0xDEADBEEF
   /* FPGA information - stored between IoControl calls */
   struct Tfpga_info sFpgaInfo;             /* Local copy of FPGA programming information. */
   /* Timers. Separate timers required for serial receive and transmit.
    * Note that the DMA is timed out using the 1 sec driver heartbeat timer. This is a 
    * legacy from the NT driver, which uses the IoTimer supplied to all drivers to 
    * form the basis of the timeout logic. This 1 sec timer is implemented in the VxWorks
    * code using the watchdog functions (wdCreate etc), which run directly from the system
    * tick ISR.
    * The inbuilt timer routines use POSIX timers hooked into the SIGALRM signal
    * handler. The structures here may need to be interrogated by the use
    * installed functions if required
    */
   TimerID TimerSerialTx;                   /* Timer ID for the serial transmit data */
   TimerID TimerSerialRx;                   /* Timer ID for the serial receive data */
   TimerID TimerDriver;                     /* Timer ID for 1 second heartbeat */
   ui32 TimeoutRx;                          /* Value for receive timeout. */
   ui32 TimeoutTx;                          /* Value for transmit timeout. */
   ui32 TimoutDriver;                       /* 1 sec heartbeat used by DMA routines etc */
   i32 nTimeRemaining;                      /* Used by I/O timer - in seconds. */
   ui32 dwTimeoutBeforeStart;               /* Timeout for first receiving data. */
   ui32 dwTimeoutDuring;                    /* Timeout during DMA (in milliseconds). */
   /* Interrupt control
    */
   int  InterruptLockKey;                   /* Key returned by intLock for use in intUnlock */
   volatile ui32 dwINTCSR;                  /* AMCC Interrupt Control and Status register. */
   ui32 dwIRQVector;                        /* To pass back to Snapper library. */
   struct TInterruptInfo sInterruptInfo;    /* Address/data to write to Snapper interrupt register. */
   ui32 dwSnapperISR;                       /* Snapper interrupt and status register. */
   ui32 dwMaxDmaLength;                     /* The maximum DMA transfer size */
   ui32 dwOperation;                        /* This records the error that occurred during DMA. */
#define DRV_OPERATION_IDLE           0
#define DRV_DMA_UNDER_WAY            1
   /* Error codes - for internal driver messaging. Note the lower 16 bits are
    * left free - this may be used by the driver for additional information -
    * for example the page number at which the DMA timed out etc.
    */
   ui32 dwErrorCodeDMA;                    /* Type of operation in progress - reading or exiting etc. */
#define DRV_ERR_READ_TIMEOUT  0x10000      /* During read (DMA). */
#define DRV_ERR_READ_ABORTED  0x30000      /* User aborted operation (not really an error). */
#define DRV_ERR_READ_ERR      0x20000      /* AMCC problem */
#define DRV_ERR_TARGET_ABORT  0x20001
#define DRV_ERR_MASTER_ABORT  0x20002
   volatile ui32 dwPreviousErrorStatus;    /* Records the status of functions executed  */
#define DRV_ERR_WE_HAVE_A_PROBLEM   0xDEAD /* Or soon will be. */
   /* Serial Comms Stuff: Must be allocated early !
    */
   struct TSerialBuffer *pSerialBuffer;   /* Pointer to caller's structure. */
   struct TSerialInfo_Common *pSIC;       /* Pointer to driver's copy of common structure. */
   struct TSerialInfo_Common *pSICLib;    /* Pointer to user's copy of common structure. */
   struct TSerialInfo_Driver *pSID;       /* Pointer to driver specific structure. */
   ui32   dwNumCharsRequired;             /* How many more characters are rquired to fulfil the serial_rx command */
   ui32 dwErrorCodeComms;                 /* For comms timeouts. */
#define DRV_ERR_TIMEOUT_TX  0x40000       /* Transmit timeout. */
#define DRV_ERR_TIMEOUT_RX  0x50000       /* Receive timeout. */
#define DRV_ERR_OTHER_TX    0x60000       /* Internal error with transmit. */
#define DRV_ERR_OTHER_RX    0x70000       /* Internal error with receive. */
   /* For Dylan reads/writes under synchronized execution.
    */
   struct Ttable_info TableInfo;          /* For synchronized Dylan accesses. */
   ui32 dwRegNum;                         /* Used for Dylan reads and writes. */
   ui32 *pdwData;                         /* Used for Dylan reads. */
   ui32 dwData;                           /* Used for Dylan writes. */
   ui32 dwCommand;                        /* Used for Dylan block writes. */
   ui8  *pbData;                          /* Used for Dylan block writes. */
   /* set interttupt suport */
   ui32 dwDriverIntSrc;                   /* Cause of interrupt */
   ui32 dwTransferSizeBytes;              /* information size returned by interrupt status */
   /* Error Reporting - from driver back to library:
    */
   char szErrorString[384];               /* Used for returning error strings to the user. */
   ui32 dwIoctlCode;                      /* A copy to send back with the error string. */
   /* MacOS specific stuff goes here ! */
   DriverRefNum	refNum;                   /* The unique ID of this driver */
   RegEntryID RegEntry;                   /* The registry key for this device */
   ui32 PageSize;                         /* the length of an MMU page, in bytes */
   ui32 PageOffsetMask;                   /* Offset = (Address & PageOffsetMask) */
   ParamBlockRec *pIntStatus;             /* Command interrupt status info */
   ParamBlockRec *pAsyncPio;              /* Required to complete an asynchronous request */
   ParamBlockRec *pAsyncDma;              /* required to complete an asynchronous DMA operation */
   ParamBlockRec *pReadComplete;          /* required to complete a ReadComplete operation */
   ParamBlockRec *pTxCommand;
   ParamBlockRec *pRxCommand;
   ParamBlockRec *pTxBufferCommand;
   ParamBlockRec *pRxBufferCommand;
   IOCommandID PioCommand;                /* O/S identifier of pending command */
   IOCommandID DmaCommand;                /* O/S identifier of Async main DMA command */
   IOCommandID IntCommand;                /* O/S identifier of pending interrupt status command */
   IOCommandID CompleteCommand;           /* O/S identifier of ReadComplete command */
   IOCommandID TxCommand;
   IOCommandID RxCommand;
   IOCommandID TxBufferCommand;           /* Command to complete to release the transmit buffer */
   IOCommandID RxBufferCommand;           /* Command to complete to release the trceive buffer */
   IOPreparationTable PioTable;           /* I/O preparation table for application communication */
   IOPreparationTable DmaTable;           /* I/O Preparation table for main DMA transfer */
   LogicalMappingTablePtr pVirtual;       /* Allocated pointer to PIO address list */
   PhysicalMappingTablePtr pPhysical;     /* Allocated pointer for DMA address list */
   ui32 nVirtualEntries;                  /* The number of entries in allocated pVitual */
   ui32 nPhysicalEntries;                 /* The number of entries in allocated pPhysical */
   struct ScatterGather *pCurrentDma;     /* The DMA in progress */
   struct ScatterGather *pBusDmaRoot;     /* The start og the scatter gather list */
   /* Note - The address space passed is that of the calling thread. This may not be the
    * same as that of the calling process, as I/O is often actually performed by a 
    * device manager thread, independenty of a application process thread.
    */
   AddressSpaceID PioAddressSpace;        /* Address space of calling thread */
   AddressSpaceID DmaAddressSpace;        /* Address space of calling thread */
   ui32 DmaIndex;                         /* The next index into pPhysical */
   ui32 DmaBytesTransferred;              /* The number of bytes transferred so far */
   ui32 DmaBytesToTransfer;               /* The number of bytes to transfer */
   struct InterruptDescriptor SnapperInterrupt;/* Functions etc. called for this board */
   void *pPioBuffer;                      /* Used as a temporary copy location */
   enum {                                 /* Used to signal if the Pio is R/W */
      SnapperPioRead,
      SnapperPioWrite
   } SnapperPioRequest;
   Boolean TxBufferMapped;
   Boolean RxBufferMapped;
   char*   pRxBuffer;
   char*   pTxBuffer;
   SInt32 ErrorCode;                      /* The last error code */
   enum {                                 /* Current driver status */
      SnapperNotReady,                    /* Noy yet initialised */
      SnapperIdle,                        /* Nothing much is happening */
      SnapperMappedRead,                  /* A read PIO buffer has been mapped */
      SnapperMappedWrite,                 /* A write PIO buffer has been mapped */
      SnapperWaitForDmaRead,              /* burst in called, witing for async read */
      SnapperDmaInProgress,               /* Awaiting the end of Dma operation */
      SnapperError,                       /* Driver is in an unrecoverable state */
      SnapperExitting                     /* Driver being unloaded */
   } SnapperStatus;
   Boolean  DmaAbortInProgress;           /* Used to synchronise Secondary interrupt routines and ISR */
};

/*
 * Malloc macros
 * -------------
MALLOC_PHYS_MEMORY(ForBrief)
FREE_PHYS_MEMORY(ForBrief)
 */
/* The default allocators usd here allocated non-paged, but not necessarily contiguous
 * memory. In addition, the memory is zeroed upon allocation.
 */
#define  MALLOC_PHYS_MEMORY(n)  (PoolAllocateResident((n), true))
#define  FREE_PHYS_MEMORY(n)   (PoolDeallocate((n)))

/* Macro to prevent unused parameter message */
#define UNUSED_PARAMETER(a) ((a) = (a))

/*
 * Status and Error Macros
 * -----------------------
 * _DRV_Roe     - Tests result and returns result if it is an error.
 * _DRV_Rfoe    - Tests result and returns FALSE if it is an error.
 * _DRV_ErrRet  - Generates a useful error return string and puts it in the 
 *                LDI structure so the Snapper library may read it.
 * _DRV_SaveErr - Just puts a copy of the error string into the LDI structure
 *                and then sets Status to the error status passed in.
 *
_DRV_Error_Macros(ForBrief)
 */
#define DRV_SUCCESS(Success) ( (Success) == STATUS_SUCCESS )

#define _DRV_Roe(Function) \
      { DRV_STATUS   _Status; \
        _Status = Function; \
        if (!DRV_SUCCESS(_Status)) return(_Status); \
}

#define _DRV_Rfoe(Function) \
      { DRV_STATUS   _Status; \
        _Status = Function; \
        if (!DRV_SUCCESS(_Status)) return(DRV_FALSE); \
}

#define _DRV_Sfoe(Function) \
      { DRV_STATUS   _Status; \
        _Status = Function; \
        if (!DRV_SUCCESS(_Status)) pLDI->dwPreviousErrorStatus = _Status; else pLDI->dwPreviousErrorStatus = 0; \
}

#if 0
/* sprintf not available */
#define _DRV_ErrRet(ErrRet, szString) { \
   sprintf(pLDI->szErrorString, "Snapper Driver: IOCTL %x (%x):\r   %s - in '%s'", \
       pLDI->dwIoctlCode, ((pLDI->dwIoctlCode >> 2) & 0xfff), \
       szString, szFnName); \
   return( ErrRet ); \
}
#else
#define _DRV_ErrRet(ErrRet, szString) { \
   return (ErrRet); \
}
#endif
#if 0
/* sprintf is not available */
#define _DRV_SaveErr(ErrRet, szString) { \
   sprintf(pLDI->szErrorString, "Snapper Driver: IOCTL %x (%x):\r   %s - in '%s'", \
       pLDI->dwIoctlCode, ((pLDI->dwIoctlCode >> 2) & 0xfff), \
       szString, szFnName); \
   Status = ErrRet; \
}
#else
#define _DRV_SaveErr(ErrRet, szString) { \
   Status = ErrRet; \
}
#endif

extern void _DRV_Write_ui8( struct tLocalDeviceInfo *, ui32, ui8 );
extern void _DRV_Write_ui16( struct tLocalDeviceInfo *, ui32, ui16 );
extern void _DRV_Write_ui32( struct tLocalDeviceInfo *, ui32, ui32 );
extern ui8 _DRV_Read_ui8( struct tLocalDeviceInfo *, ui32 );
extern ui16 _DRV_Read_ui16( struct tLocalDeviceInfo *, ui32 );
extern ui32 _DRV_Read_ui32( struct tLocalDeviceInfo *, ui32 );

/* Parallel Port - used as signalling output for timing debugging.
_DRV_DEBUG_ParallelPortOut(ForBrief)
 */
#if defined (_DEBUG_PARALLEL_PORT)
#if defined _X86_
/* Assume PC architecture */
#define _DRV_DEBUG_ParallelPortOut(pLDI, dwBitPos, dwPolarity) \
  { if (dwPolarity == 1) bDebugPortShadow |= (1 << dwBitPos); \
    else bDebugPortShadow &= ~(1 << dwBitPos); \
    _outp( 0x378, bDebugPortShadow ); }
#else
/* Non PC architecture - rely on installed callback */
#define _DRV_DEBUG_ParallelPortOut(pLDI, dwBitPos, dwPolarity) \
  { if (dwPolarity == 1) bDebugPortShadow |= (1 << dwBitPos); \
    else bDebugPortShadow &= ~(1 << dwBitPos); \
    if ( pLDI->SetDebugIndicator != NULL ) pLDI->SetDebugIndicator(bDebugPortShadow); }
#endif
#else
/* _DEBUG_PARALLEL_PORT not defined */
#define _DRV_DEBUG_ParallelPortOut(pLDI, dwBitPos, dwPolarity)
#endif


/*
 * PROTOTYPES
 * ----------
 *
Prototypes(ForBrief)
 *
 * init.c
 * ------
 */

/*
 * snapper.c
 * ---------
 */
DRV_STATUS   DRV_DeviceClose(struct tLocalDeviceInfo *pLDI);
DRV_STATUS   DRV_DeviceOpen(struct tLocalDeviceInfo *pLDI, struct TBoardInfo *pBoardInfo);
DRV_STATUS   DRV_Fpga_GetConfigStatus(struct tLocalDeviceInfo *pLDI, ui32 *pdwConfigStatus);
DRV_STATUS   DRV_Fpga_ISAConfigure(struct tLocalDeviceInfo *pLDI);
DRV_STATUS   DRV_Fpga_PCIConfigure(struct tLocalDeviceInfo *pLDI);
DRV_STATUS   DRV_Fpga_PCIConfigure_1bit(struct tLocalDeviceInfo *pLDI);
DRV_STATUS   DRV_Fpga_PCIConfigure_32bit(struct tLocalDeviceInfo *pLDI);
DRV_STATUS   DRV_Fpga_Program(struct tLocalDeviceInfo *pLDI);
DRV_STATUS   DRV_Fpga_Reset(struct tLocalDeviceInfo *pLDI);
DRV_STATUS   DRV_Fpga_SendInfo(struct tLocalDeviceInfo *pLDI, struct Tfpga_info *pFpga);
DRV_STATUS   DRV_PCI_DylanIsBusy(struct tLocalDeviceInfo *pLDI, ui32 *pdwDylanStatus);
DRV_STATUS   DRV_PCI_DylanRead_ui32_NonSync(struct tLocalDeviceInfo *pLDI, ui32 dwRegNum, ui32 *pdwData);
DRV_STATUS   DRV_PCI_DylanRead_ui32_Sync(struct tLocalDeviceInfo *pLDI, ui32 dwRegNum, ui32 *pdwData);
DRV_BOOLEAN  DRV_PCI_DylanRead_ui32_KeSync(struct tLocalDeviceInfo *pLDI);
DRV_STATUS   DRV_PCI_DylanWrite_ui32_NonSync(struct tLocalDeviceInfo *pLDI, ui32 dwRegNum, ui32 dwData);
DRV_STATUS   DRV_PCI_DylanWrite_ui32_Sync(struct tLocalDeviceInfo *pLDI, ui32 dwRegNum, ui32 dwData);
DRV_BOOLEAN  DRV_PCI_DylanWrite_ui32_KeSync(struct tLocalDeviceInfo *pLDI);
DRV_STATUS   DRV_PCI_DylanWriteBlock_ui32_NonSync(struct tLocalDeviceInfo *pLDI, ui32 dwCommand, ui8 *pbData);
DRV_STATUS   DRV_PCI_DylanWriteBlock_ui32_Sync(struct tLocalDeviceInfo *pLDI, ui32 dwCommand, ui8 *pbData);
DRV_BOOLEAN  DRV_PCI_DylanWriteBlock_ui32_KeSync(struct tLocalDeviceInfo *pLDI);
DRV_STATUS   DRV_PCI_DylanWaitForNotBusy(struct tLocalDeviceInfo *pLDI, ui32 dwTimeout);
DRV_STATUS   DRV_PCI_ShadowEEPROM(struct tLocalDeviceInfo *pLDI, struct TBoardInfo *pBoardInfo);
DRV_STATUS   DRV_Serial_CheckForCommsTimeout(struct tLocalDeviceInfo *pLDI);
DRV_STATUS   DRV_Serial_ResetComms(struct tLocalDeviceInfo *pLDI);
DRV_BOOLEAN  DRV_Serial_GetInfo(struct tLocalDeviceInfo *pLDI);
DRV_BOOLEAN  DRV_Serial_SendInfo(struct tLocalDeviceInfo *pLDI);
DRV_STATUS   DRV_Serial_RxBuffer(struct tLocalDeviceInfo *pLDI);
DRV_BOOLEAN  DRV_Serial_Rx_CriticalIncrement(struct tLocalDeviceInfo *pLDI);
DRV_STATUS   DRV_Serial_TxBuffer(struct tLocalDeviceInfo *pLDI);
DRV_BOOLEAN  DRV_TableLoad(struct tLocalDeviceInfo *pLDI);
DRV_STATUS   DRV_Timeout_Set(struct tLocalDeviceInfo *pLDI, struct TTimeout *pTimeout);

/*
 * nt.c
 * ----
 */
DRV_STATUS   DRV_BurstIn_ui32(struct tLocalDeviceInfo *pLDI, void* PortAddress, ui32 *pdwData, ui32 dwByteCount);
DRV_STATUS   DRV_ReadComplete(struct tLocalDeviceInfo *pLDI, ui32 dwMode, ui32 *pdwResult);
DRV_STATUS   DRV_BurstInAsync_ui32(struct tLocalDeviceInfo *pLDI, void* PortAddress, ui32 *pdwData, ui32 dwByteCount);
DRV_STATUS   DRV_BurstIn_ui16(struct tLocalDeviceInfo *pLDI, void* PortAddress, ui16 *pData, ui32 dwByteCount);
DRV_STATUS   DRV_BurstIn_ui16_Special(struct tLocalDeviceInfo *pLDI, void* PortAddress, ui16 *pData, ui32 dwPixelCount, ui32 dwMapping);
DRV_STATUS   DRV_BurstOut_ui16(struct tLocalDeviceInfo *pLDI,void* PortAddress, ui16 *pwData, ui32 dwByteCount);
DRV_STATUS   DRV_BurstOut_ui16_Special(struct tLocalDeviceInfo *pLDI, void *PortAddress, ui16 *pwData, ui32 dwPixelCount, ui32 dwMapping);
DRV_BOOLEAN  DRV_DMA_Abort(struct tLocalDeviceInfo *pLDI);
DRV_STATUS   DRV_DMA_Init(struct tLocalDeviceInfo *pLDI);
DRV_STATUS   DRV_DMA_Release(struct tLocalDeviceInfo *pLDI);
DRV_BOOLEAN  DRV_DMA_Start(struct tLocalDeviceInfo *pLDI, struct ScatterGather *pThisDma);
DRV_BOOLEAN DRV_IoTimerSetBefore(struct tLocalDeviceInfo *pLDI);
DRV_BOOLEAN DRV_IoTimerSetDuring(struct tLocalDeviceInfo *pLDI);
void DRV_RaiseIrql(struct tLocalDeviceInfo *pLDI);
void DRV_RestoreIrql(struct tLocalDeviceInfo *pLDI);
DRV_STATUS   DRV_TimeDelay(ui32 dwMicroSecs);
DRV_STATUS   DRV_TimeGetStamp_ms(ui32 *pdwTimeStamp);
DRV_STATUS   DRV_TimeOut(ui32 dwTimeOut, ui32 *pfResult);
/* Not strictly true - void * is really struct tLocalDeviceInfo*, but there is a definition order
 * problem in the header files, hence the opaque pointer type.
 */
OSStatus DRV_Timeout_SerialTx(void *, void *);
OSStatus DRV_Timeout_SerialRx(void *, void *);
/* 
 * int.c
 * -----
 */
long         DRV_Interrupt_Handler(struct InterruptSetMember, void*, ui32);
OSStatus         DRV_IoTimer(void* Arg1, void* Arg2);
DRV_BOOLEAN  DRV_IoTimerProcess(struct tLocalDeviceInfo *pLDI);
DRV_STATUS   DRV_Interrupt_ClearAndMaskAll(struct tLocalDeviceInfo *pLDI);
DRV_STATUS   DRV_Interrupt_ClearDown(struct tLocalDeviceInfo *pLDI, ui32 *pdwIntType);
DRV_STATUS   DRV_Interrupt_Process(struct tLocalDeviceInfo *pLDI, ui32 dwIntType);
DRV_STATUS   DRV_Interrupt_ProcessDylan(struct tLocalDeviceInfo *pLDI);
DRV_STATUS   DRV_Interrupt_ProcessSnapper(struct tLocalDeviceInfo *pLDI);
DRV_STATUS   DRV_Interrupt_ReadSnapper(struct tLocalDeviceInfo *pLDI, DRV_BOOLEAN *pfSnapperInterrupt);
DRV_STATUS   DRV_Interrupt_ReadDylan(struct tLocalDeviceInfo *pLDI, DRV_BOOLEAN *pfDylanInterrupt);
DRV_STATUS   DRV_Interrupt_ReadStatus(struct tLocalDeviceInfo *pLDI, DRV_BOOLEAN *pfIRQ_Pending);
DRV_BOOLEAN  DRV_Interrupt_Rx_CriticalIncrement(struct tLocalDeviceInfo *pLDI);
DRV_BOOLEAN  DRV_Interrupt_Rx_CriticalDecrement(struct tLocalDeviceInfo *pLDI);
void DRV_Interrupt_Rx_Complete(struct tLocalDeviceInfo *, OSErr);
void DRV_Interrupt_Rx_Copy(struct tLocalDeviceInfo *);
/* the completion fnctions are called as secondary interrupt handlers */
OSStatus DRV_Interrupt_Complete_DMA(void*, void*);
OSStatus DRV_Interrupt_Complete_SerialRx(void*, void*);
OSStatus DRV_Interrupt_Complete_SerialTx(void*, void*);
OSStatus DRV_Interrupt_Complete_Snapper(void*, void*);
OSStatus   DRV_Interrupt_ProcessDylanRx(void*, void*);
OSStatus   DRV_Interrupt_ProcessDylanTx(void*, void*);
OSStatus   DRV_Interrupt_ProcessDylanTimer(void*, void*);
DRV_BOOLEAN  DRV_Interrupt_SerialEnable(struct tLocalDeviceInfo *pLDI);
DRV_BOOLEAN  DRV_Interrupt_Set(struct tLocalDeviceInfo *pLDI);

/* Functions in drv_cmd.c */
OSErr DriverInitializeCmd(AddressSpaceID, DriverInitInfoPtr);
OSErr DriverFinalizeCmd(DriverFinalInfoPtr);
OSErr DriverSupersededCmd(DriverSupersededInfoPtr, Boolean);
OSErr DriverReplaceCmd(AddressSpaceID, DriverReplaceInfoPtr, Boolean);
OSErr DriverOpenCmd(AddressSpaceID, ParmBlkPtr);
OSErr DriverCloseCmd(ParmBlkPtr);
OSErr DriverControlCmd( AddressSpaceID, IOCommandID, IOCommandKind, ParmBlkPtr);
OSErr DriverStatusCmd( AddressSpaceID, IOCommandID, IOCommandKind, ParmBlkPtr );
OSErr DriverReadCmd( AddressSpaceID, IOCommandID, IOCommandKind, ParmBlkPtr );
OSErr DriverWriteCmd( AddressSpaceID, IOCommandID, IOCommandKind, ParmBlkPtr );
OSErr DriverKillIOCmd(ParmBlkPtr);

/* Functions in drv_mem.c */
void SnapperMemSet(void*, ui8, size_t);
void SnapperCopyPagesToBuffer(LogicalAddress *, void *, Size);
void SnapperCopyBufferToPages(void*, LogicalAddress *, Size);
OSStatus SnapperMapInPio(void*, void*);
OSStatus SnapperMapInDma(void*, void*);
OSStatus SnapperMapOutDma(void*, void*);
OSStatus SnapperMapOutPio(void*, void*);
ui32 SnapperComputePages(struct tLocalDeviceInfo*, ParamBlockRec*);
void SnapperGenerateScatterGather(struct tLocalDeviceInfo *);
OSStatus SnapperStartDma(void* , void* );
ui32 SnapperGetPhysicalRamSize(void);

/* Secondary interrupt handlers */
OSStatus SnapperStartDma(void*, void*);
OSStatus SnapperQueueReadComplete(void*, void*);
OSStatus SnapperQueueInterruptStatus(void*, void*);
OSStatus SnapperAbortOperation(void*, void*);

int DrvSetTimer( struct tLocalDeviceInfo*, TimerID *, ui32, OSStatus (*)(void*, void*) );
int DrvCancelTimer( TimerID *);

/* Functions in drv_reg.c */
OSErr GetDeviceLogicalAddress(RegEntryIDPtr,PCIRegisterNumber,PCIAddressSpaceFlags, LogicalAddress*, ByteCount*);
void DisposeThisProperty(RegPropertyValue* );
OSErr GetThisProperty(RegEntryIDPtr,RegPropertyNamePtr,RegPropertyValue**,RegPropertyValueSize*);

/* Functions in drv_io.c */
OSStatus DrvSynchronousExecutionInterrupt( void*, void*);
DRV_BOOLEAN DrvSynchronizeExecution(struct tLocalDeviceInfo *, DRV_BOOLEAN (*)(struct tLocalDeviceInfo*));

DRV_STATUS DRV_Init_LDI(struct tLocalDeviceInfo*);
#if !defined DIG16_BIT_INT_MAIN_TRANSFER_EN
#define DIG16_BIT_INT_MAIN_TRANSFER_EN   ((ui32) 0x00010000)
#endif

#if !defined MODULE_READ_VPORT_UI32
#define MODULE_READ_VPORT_UI32          ((ui8) 5)
#endif

#endif /* _MACINTOSH_KERNEL */

