/**************************************************************************
 *                ACTIVE SILICON LIMITED / ACTIVE IMAGING LIMITED         *
 *                                                                        *
 *  File name   : asl_vx.h                                                *
 *  Function    : Type definitions for Snapper VxWorks driver             *
 *  Project     : Snapper                                                 *
 *  Author      : Chris Read, Active Silicon Ltd                          *
 *  Systems     : ANSI C                                                  *
 *  Version     : 3.10.28                                                 *
 *  Release Date: 22-Oct-97                                               *
 *                                                                        *
 *  Copyright (c) 1993-97 Active Silicon Ltd, Active Imaging Ltd.         *
 **************************************************************************
 * Comments:
 * --------
 * This is the Snapper VxWorks driver include file
 *
 * File History
 * ------------
 * 22-Oct-97, CLR, 3.10 Initial version
 * 13-Jul-99, CLR, 4.00 version - added dependency on snp_rev.h
 *                              - added scatter/gather capabilities
 *
 */

/*
 * Driver Revision Level
 * ---------------------
 * This revision level is sent back to the Snapper library as a check that 
 * compatible libraries and driver are being used.
 */
#define SNAPPER_DRIVER_REV_LEVEL  (SNP_REV_LEVEL)


/*
 * Driver Build Options
 * --------------------
 * _DEBUG_PARALLEL_PORT - When defined, allows the macro 
 *                        _DRV_DEBUG_ParallelPortOut to be used to drive
 *                        pins in the parallel port high/low etc for 
 *                        timing/speed measurements.
 *                        This is only supportable on certain hardware platforms
 *                        and is enabled by the user filling in the appropriate
 *                        stub functions
 *                        
 *
 *
 */
/* Note that unlike other Snapper drivers, the use of DMA and interrupts
 * is controlled on a per device basis, using the bUseDma and bUseInterrupt 
 * flags.
 */

 /*
 *#define _DEBUG_PARALLEL_PORT
 *#define _DYLAN_AS_W95
 */
typedef ui32 DRV_BOOLEAN;
#define DRV_TRUE (1==1)
#define DRV_FALSE ( !(DRV_TRUE) )

#if !defined (MAXLONG)
#define MAXLONG ( (ui32)(~0) )
#endif

typedef enum {
    STATUS_SUCCESS,
    STATUS_UNSUCCESSFUL,
    STATUS_NOT_IMPLEMENTED,
    STATUS_UNSUPPORTED,
    STATUS_INTERNAL_ERROR,
    STATUS_INVALID_DEVICE_REQUEST,
    STATUS_INVALID_BUFFER_SIZE,
    STATUS_ADAPTER_HARDWARE_ERROR,
    STATUS_INVALID_PARAMETER
} DRV_STATUS;
/*
 * System Include Files
 * --------------------
 */
#include <taskLib.h>
#include <vmLib.h>
#include <cacheLib.h>
#include <intLib.h>
#include <semLib.h>
#include <sysLib.h>
#include <vwModNum.h>
#include <errnoLib.h>
#include <sigLib.h>
#include <wdLib.h>
#include <timers.h>
/*
 * General #defines
 * ----------------
 */
#define DRV_MAX_NUM_PCI_BUSES 256   /* Maximum number of PCI buses          */

/* AlenLists are Address/Length lists.
 * Each entry describes a section of memory as its starting address
 * and extent. Each list is of variable length, as address translation
 * routines may need to sub-divide large memory sections
 */
typedef struct AlenList{
    struct AlenList *pNext;
    union {
        void   *VirtualAddress;
        ui32   PhysicalAddress;
    } Address;
    size_t Length;
    ui32   Flags;
} AlenList;

struct tVxDriverTimer{
   timer_t Timer_id;               /* The POSIX timer ID used by the default (builtin) routines */
   ui32    Timeout;                /* The timeout period in msec */
   void    (*pFnTimeout)(void *);  /* The callback function in timeout */
   void    *pInfo;                 /* User pointer. The value passed in pFnCreateTimer is the
                                    * argument which MUST be passed to pFnTimeout when the timer expires.
                                    */
};
/*
 * A structure to hold device information when the driver first interrogates
 * the system. This structure is stored in the local device information structure
 * and is the parameter passed to SnapperAttachDevice()
 *
struct tDeviceInfo(ForBrief)
 */
struct tDeviceInfo {
   ui32 dwIDrev;            /* Standard Snapper IDrev (Bus Interface Board). */
   ui32 dwBusNum;           /* PCI bus number (0-255) */
   ui32 dwDeviceNumber;     /* For PCI device addressing. A funcion of the
                             * location of the Snapper card */
   ui32 dwFunctionNumber;   /* For PCI device addressing. A function of the
                             * Snapper card (0-7) */
   enum{                    /* Only PCI will be supported in this release */
      DRV_BUS_TYPE_UNDEFINED,
      DRV_BUS_TYPE_ISA,
      DRV_BUS_TYPE_PCI
   } dwBusType;
   ui32 bDmaEnable;         /* Non-zero value indicates this board can be a DMA bus master */
   ui32 bInterruptEnable;   /* Non-zero value indicates this board can generate interrupts */
   ui32 BusAddress;         /* I/O address of board, as written to config space */
   ui32 PhysicalAddress;     /* I/O address of baord as seen from the processor */
   void *VirtualAddress;    /* Corresponding Virtual Address */
   ui32 nIRQLevel;          /* As seen by the processor */
   ui32 nIRQVector;         /* As seen by the processor */
   /* The following set of user supplied functions are for the reading/writing of
    * individual registers from the PPC to the PowerPC. If no special user function
    * is rewuired, then the function pointer should be left set to NULL. This will
    * cause the default function compiled into the library to be used, This function
    * performs no address translation (assumed to be performed by the bus bridge chip)
    * and full 16/32 bit byte swapping. In addition to this, the default functions
    * issue a PPC EIEIO instruction to guarantee write ordering.
    * In normal operation, no reference is required to the tDeviceInfo structure,
    * as the Address passed is the Address as seen by the processor. However, it
    * is envisaged that in some systems knowledge of the bus number may be required
    * in order to deduce the operations required.
    */
   void (*pFnWritePortUlong)( struct tDeviceInfo*, void*, ui32);
   void (*pFnWritePortUshort)( struct tDeviceInfo*, void*, ui16);
   void (*pFnWritePortUbyte)( struct tDeviceInfo*, void*, ui8);
   ui32 (*pFnReadPortUlong)( struct tDeviceInfo*, void*);
   ui16 (*pFnReadPortUshort)( struct tDeviceInfo*, void*);
   ui8  (*pFnReadPortUbyte)( struct tDeviceInfo*, void*);
   /* And a similar set of functions for reading/writeing data to/from the AMCC chip.
    * Note that the video data is essentially a video byte stream, and therefore has no
    * inherent endian attribute. (16 bit data is automatically swapped by the AMCC chip)
    * Therefore, with big-endian processor, the data accesses are normally not swapped,
    * whilst the register accesses (above) are
    */
   void (*pFnWriteDataUlong)( struct tDeviceInfo*, void*, ui32);
   void (*pFnWriteDataUshort)( struct tDeviceInfo*, void*, ui16);
   ui32 (*pFnReadDataUlong)( struct tDeviceInfo*, void*);
   ui16 (*pFnReadDataUshort)( struct tDeviceInfo*, void*);
   /* The following set of user supplied routines are for address translation.
    * This may be a very complex process on systems with multiple PCI buses
    * interconnected by PCI-PCI bridges, and perhaps multiple processors
    * with various shared mamory architectures.
    * However, most VxWorks systems with basic MMU support and a single 
    * processor have a simple linear mapping which may be derived from the
    * sysPhysMemDesc list in sysLib.c of the board support package. A set of
    * sample routines is provided for this case.
    * If it is known that this list gives a 1:1 correspondence of virtual and
    * physical addresses, then AlenListVirtToPhys is normally a null
    * function, and both AlenListPhysToBus and AlenListVirtToBus simply
    * apply the appropriate address offsets generated in the CPU/PCI bus
    * interface chip.
    * All routines return the number of elements used in the target list
    */
   int  (*pFnAlenListVirtToPhys)(AlenList *, AlenList *);
                            /* Converts a list of processor virtual memory addresses to
                             * physical memory addresses, valid on the calling processor
                             * A NULL value indicates that no conversion is required
                             */
   int  (*pFnAlenListPhysToBus)(AlenList *, AlenList *, struct tDeviceInfo *);
                            /* Converts a list of physical addresses in the memory
                             * space of the calling processor to physical addresses
                             * as seen from the PCI bus specified in tDeviceInfo
                             * A NULL value indicates that no conversion is required
                             */                
   /* The following set of routines are required to maintain cache coherency between
    * the processor, system memory and the Snapper card. Note that there may be multiple
    * levels of caching, both in the processor hardware and write buffers associated
    * with the particular bus interface controller
    */
   int  (*pFnAlenListCacheFlush)(AlenList *);
                            /* Causes the cache to be flushed on the calling processor
                             * for the list of virtual addresses specified. A NULL value indicates
                             * that no cache synchronisation operations are necessary
                             */
   int  (*pFnAlenListCacheInvalidate)(AlenList *);
                            /* Causes the cache to be invalidated on the calling processor
                             * for the list of virtual addresses specified. A NULL value indicates
                             * that no cache synchronisation operations are necessary
                             */
   /* On some systems, it is required to issue special commands to interrupt or bus bridge
    * controllers at the beginning and end of interrupt service routines. Such systems
    * are the exception rather than the rule. Normally, these members should be NULL pointers,
    * indicating that the default routines (which do nothing) should be used.
    */
   void (*pFnPreInterruptHook)(struct tDeviceInfo *);
                            /* Performs any hardware processing required before any Snapper
                             * interrupt processing is performed
                             * For multiple Snapper boards sharing the same interrupt, this
                             * routine will be called once for each interrupting device
                             * Note: This routine is always called at interrupt context
                             * A NULL value indicates that no processing is required
                             */
   void (*pFnPostInterruptHook)(struct tDeviceInfo *);
                            /* Performs any hardware processing required after any Snapper
                             * interrupt processing has been performed, and before the
                             * Snapper interrupt service routine returns to the kernel
                             * For multiple Snapper boards sharing the same interrupt, this
                             * routine will be called once for each interrupting device
                             * interrupt dispatching routines.
                             * Note: This routine is always called at interrupt context
                             * A NULL value indicates that no processing is required
                             */
/* The default timer routines use the SIGALRM signal handler. If this is requied by
 * the application code, then the application must be responsible for running the
 * timers associated with the serial communications.
 * Note that if SIGALRM is not to be used, then these functions must be overrun
 * for every board attached to the system.
 */
   DRV_BOOLEAN (*pFnCreateTimer)(struct tVxDriverTimer *);
                             /* Creates the specified timer, but does not start it. The Timeout
                              * field of the struct tVxDriverTimer is not guaranteed to be
                              * valid at this point.
                              * The pInfo element of struct tVxDriverTimer points to the
                              * struct tLocalDeviceInfo when this routine is called. If the user
                              * routine needs the pInfo for some other use, then the pLDI
                              * must be saved, as it is a required parameter of all the driver
                              * timeout routines.
                              * Returns DRV_TRUE on success.
                              */
   DRV_BOOLEAN (*pFnDeleteTimer)(struct tVxDriverTimer *);
                             /* Deletes the specified timer from the system and frees any resources
                              * associated with it. Care must be taken, as it is not guaranteed that
                              * the specified timer has been cancelled, thus the situation of a pending
                              * allback must be properly handled.
                              * Returns DRV_TRUE on success.
                              */
   DRV_BOOLEAN (*pFnStartTimer)(struct tVxDriverTimer *);
                             /* Starts the specified timer, to expire in a period of Timeout milliseconds.
                              * If the specified timeout period cannot be generated due to the granularity
                              * of the clock, then the period should always be rounded UP
                              * Returns DRV_TRUE on success.
                              */
   DRV_BOOLEAN (*pFnCancelTimer)(struct tVxDriverTimer *);
                             /* Stops the specified timer. If the timer has already expired, then
                              * this routine should do nothing (i.e. return 0)
                              * Returns DRV_TRUE on success.
                              * Note: This routine may be called at interrupt context
                              */
   void (*ToggleDebugIndicator)(ui8);
                             /* For performance and debugging analysis, it can be useful to set
                              * additional hardware signals from the Snapper driver (e.g. toggling
                              * lines on a parallel port) This field is for ASL use only, and
                              * should always be set to NULL
                              * Note: This routine may be called at interrupt context
                              */
};


/*
 * Information local to each device object
 * ---------------------------------------
 * This information is stored in the device extensions part of the device
 * object.
 *
struct tLocalDeviceInfo(ForBrief)
 */
struct tLocalDeviceInfo {

   struct tLocalDeviceInfo *pNext;          /* pointer to next entry in the list */
   struct tDeviceInfo *pDeviceInfo;         /* Information passed to attach */
   ui32   dwUseCount;                       /* Count of tasks with driver open */
   ui32 Eeprom[BASE_PCI40_EEPROM_LENGTH];   /* Shadow of PCI NVRAM. */
   ui32 dwMagicNum;                         /* Used for assert type sanity checks. */
#define DRV_MAGIC_NUMBER  0xDEADBEEF
   /* FPGA information - stored between IoControl calls */
   struct Tfpga_info sFpgaInfo;             /* Local copy of FPGA programming information. */
   /* Timers. Separate timers required for serial receive and transmit.
    * Note that the DMA is timed out using the 1 sec driver heartbeat timer. This is a 
    * legacy from the NT driver, which uses the IoTimer supplied to all drivers to 
    * form the basis of the timeout logic. This 1 sec timer is implemented in the VxWorks
    * code using the watchdog functions (wdCreate etc), which run directly from the system
    * tick ISR.
    * The inbuilt timer routines use POSIX timers hooked into the SIGALRM signal
    * handler. The structures here may need to be interrogated by the use
    * installed functions if required
    */
   struct tVxDriverTimer TimerSerialTx;     /* Timer information for the serial transmit data */
   struct tVxDriverTimer TimerSerialRx;     /* Timer information for the serial receive data */
   ui32 TimeoutRx;                          /* Value for receive timeout. */
   ui32 TimeoutTx;                          /* Value for transmit timeout. */
   WDOG_ID TimerDriver;                     /* 1 sec heartbeat used by DMA routines etc */
   volatile i32 nTimeRemaining;             /* Used by I/O timer - in seconds. */
   volatile ui32 dwTimeoutBeforeStart;      /* Timeout for first receiving data. */
   volatile ui32 dwTimeoutDuring;           /* Timeout during DMA (in milliseconds). */
   /* Interrupt control
    */
   int  InterruptLockKey;                   /* Key returned by intLock for use in intUnlock */
   volatile ui32 dwINTCSR;                  /* AMCC Interrupt Control and Status register. */
   ui32 dwIRQVector;                        /* To pass back to Snapper library. */
   volatile struct TInterruptInfo sInterruptInfo;    /* Address/data to write to Snapper interrupt register. */
   volatile ui32 dwSnapperISR;                       /* Snapper interrupt and status register. */
   volatile ui32 dwOperation;               /* This records the error that occurred during DMA. */
#define DRV_OPERATION_IDLE           0
#define DRV_DMA_UNDER_WAY            1
   /* Error codes - for internal driver messaging. Note the lower 16 bits are
    * left free - this may be used by the driver for additional information -
    * for example the page number at which the DMA timed out etc.
    */
   volatile ui32 dwErrorCodeDMA;           /* Type of operation in progress - reading or exiting etc. */
#define DRV_ERR_READ_TIMEOUT  0x10000      /* During read (DMA). */
#define DRV_ERR_READ_ABORTED  0x30000      /* User aborted operation (not really an error). */
   volatile ui32 dwPreviousErrorStatus;    /* Records the status of functions executed  */
#define DRV_ERR_WE_HAVE_A_PROBLEM   0xDEAD /* Or soon will be. */
   /* Semaphores:
    * Note that a VxWorks semaphore is an allocated kernel resource, the SEM_ID
    * being merely a handle to that resource. Thus semaphores cannot be simply
    * recreated at each open (unlike the NT driver), otherwise kernel memory
    * leaks will occur.
    */
   SEM_ID SemaDMA;                         /* To signal DMA completion. */
   SEM_ID SemaTx;                          /* To signal serial comms - TX. */
   SEM_ID SemaRx;                          /* To signal serial comms - RX. */
   SEM_ID SemaInt;                         /* Snapper interrupt semaphore */
   /* Serial Comms Stuff:
    */
   volatile struct TSerialBuffer *pSerialBuffer;   /* Pointer to caller's structure. */
   volatile struct TSerialInfo_Common *pSIC;       /* Pointer to driver's copy of common structure. */
   volatile struct TSerialInfo_Common *pSICLib;    /* Pointer to user's copy of common structure. */
   volatile struct TSerialInfo_Driver *pSID;       /* Pointer to driver specific structure. */
   volatile ui32 dwErrorCodeComms;                 /* For comms timeouts. */
#define DRV_ERR_TIMEOUT_TX  0x40000       /* Transmit timeout. */
#define DRV_ERR_TIMEOUT_RX  0x50000       /* Receive timeout. */
#define DRV_ERR_OTHER_TX    0x60000       /* Internal error with transmit. */
#define DRV_ERR_OTHER_RX    0x70000       /* Internal error with receive. */
   /* For Dylan reads/writes under synchronized execution.
    */
   volatile struct Ttable_info *pTableInfo;        /* For synchronized Dylan accesses. */
   volatile ui32 dwRegNum;                         /* Used for Dylan reads and writes. */
   volatile ui32 *pdwData;                         /* Used for Dylan reads. */
   volatile ui32 dwData;                           /* Used for Dylan writes. */
   volatile ui32 dwCommand;                        /* Used for Dylan block writes. */
   volatile ui8  *pbData;                          /* Used for Dylan block writes. */
   /* set interttupt suport */
   volatile ui32 dwDriverIntSrc;                   /* Cause of interrupt */
   volatile ui32 dwTransferSizeBytes;              /* information size returned by interrupt status */
   /* Error Reporting - from driver back to library:
    */
   char szErrorString[384];               /* Used for returning error strings to the user. */
   ui32 dwIoctlCode;                      /* A copy to send back with the error string. */
   /* DMA address conversion etc
    */
   AlenList* pVirtualDmaRoot;             /* Current list of processor addresses being DMA'd to */
   AlenList* pPhysicalDmaRoot;            /* Current list of memory addresses being DMA'd to */
   AlenList* pBusDmaRoot;                 /* Current lstt of bus addresses being DMA'd to */
   AlenList* pUserDmaRoot;                /* For user supplied scatter/gather list */
   AlenList* pTempDmaRoot;                /* Used as temporary */
   volatile AlenList* pCurrentDma;                 /* The actual entry in pBusRoot programmed in the AMCC chip */
};


/*
 * Malloc macros
 * -------------
MALLOC_PHYS_MEMORY(ForBrief)
FREE_PHYS_MEMORY(ForBrief)
 */
/* VxWorks does not perform paging, therefore the standard memory
 * allocator is OK here
 */
#define  MALLOC_PHYS_MEMORY(n)  _ASL_Malloc((n))
#define  FREEE_PHYS_MEMORY(n)   _ASL_Free((n))


/*
 * Status and Error Macros
 * -----------------------
 * _DRV_Roe     - Tests result and returns result if it is an error.
 * _DRV_Rfoe    - Tests result and returns FALSE if it is an error.
 * _DRV_ErrRet  - Generates a useful error return string and puts it in the 
 *                LDI structure so the Snapper library may read it.
 * _DRV_SaveErr - Just puts a copy of the error string into the LDI structure
 *                and then sets Status to the error status passed in.
 *
_DRV_Error_Macros(ForBrief)
 */
#define DRV_SUCCESS(Success) ( (Success) == STATUS_SUCCESS )

#define _DRV_Roe(Function) \
      { DRV_STATUS   _Status; \
        _Status = Function; \
        if (!DRV_SUCCESS(_Status)) return(_Status); \
}

#define _DRV_Rfoe(Function) \
      { DRV_STATUS   _Status; \
        _Status = Function; \
        if (!DRV_SUCCESS(_Status)) return(FALSE); \
}

#define _DRV_Sfoe(Function) \
      { DRV_STATUS   _Status; \
        _Status = Function; \
        if (!DRV_SUCCESS(_Status)) pLDI->dwPreviousErrorStatus = _Status; else pLDI->dwPreviousErrorStatus = 0; \
}


#define _DRV_ErrRet(ErrRet, szString) { \
   sprintf(pLDI->szErrorString, "Snapper Driver: IOCTL %x (%x):\r   %s - in '%s'", \
       pLDI->dwIoctlCode, ((pLDI->dwIoctlCode >> 2) & 0xfff), \
       szString, szFnName); \
   return( ErrRet ); \
}

#define _DRV_SaveErr(ErrRet, szString) { \
   sprintf(pLDI->szErrorString, "Snapper Driver: IOCTL %x (%x):\r   %s - in '%s'", \
       pLDI->dwIoctlCode, ((pLDI->dwIoctlCode >> 2) & 0xfff), \
       szString, szFnName); \
   Status = ErrRet; \
}

extern void _DRV_Write_ui8( struct tLocalDeviceInfo *, ui32, ui8 );
extern void _DRV_Write_ui16( struct tLocalDeviceInfo *, ui32, ui16 );
extern void _DRV_Write_ui32( struct tLocalDeviceInfo *, ui32, ui32 );
extern ui8 _DRV_Read_ui8( struct tLocalDeviceInfo *, ui32 );
extern ui16 _DRV_Read_ui16( struct tLocalDeviceInfo *, ui32 );
extern ui32 _DRV_Read_ui32( struct tLocalDeviceInfo *, ui32 );

/* Parallel Port - used as signalling output for timing debugging.
_DRV_DEBUG_ParallelPortOut(ForBrief)
 */
#if defined (_DEBUG_PARALLEL_PORT)
#if defined _X86_
/* Assume PC architecture */
#define _DRV_DEBUG_ParallelPortOut(pLDI, dwBitPos, dwPolarity) \
  { if (dwPolarity == 1) bDebugPortShadow |= (1 << dwBitPos); \
    else bDebugPortShadow &= ~(1 << dwBitPos); \
    _outp( 0x378, bDebugPortShadow ); }
#else
/* Non PC architecture - rely on installed callback */
#define _DRV_DEBUG_ParallelPortOut(pLDI, dwBitPos, dwPolarity) \
  { if (dwPolarity == 1) bDebugPortShadow |= (1 << dwBitPos); \
    else bDebugPortShadow &= ~(1 << dwBitPos); \
    if ( pLDI->SetDebugIndicator != NULL ) pLDI->SetDebugIndicator(bDebugPortShadow); }
#endif
#else
/* _DEBUG_PARALLEL_PORT not defined */
#define _DRV_DEBUG_ParallelPortOut(pLDI, dwBitPos, dwPolarity)
#endif


/*
 * PROTOTYPES
 * ----------
 *
Prototypes(ForBrief)
 *
 * init.c
 * ------
 */
DRV_STATUS   LCL_Init_LDI(struct tLocalDeviceInfo *pLDI);
DRV_STATUS   DRV_Init_LDI(struct tLocalDeviceInfo *pLDI);

/* 
 * alenlist.c
 */
void DRV_AlenList_Destroy(struct AlenList *pElement);
struct AlenList* DRV_AlenList_New_Element(struct AlenList *pElement);
struct AlenList* DRV_AlenList_Next_Element(struct AlenList *pElement);
ui32 DRV_AlenList_Count(struct AlenList *pElement);
/*
 * snapper.c
 * ---------
 */
DRV_STATUS   DRV_DeviceClose(struct tLocalDeviceInfo *pLDI);
DRV_STATUS   DRV_DeviceOpen(struct tLocalDeviceInfo *pLDI, struct TBoardInfo *pBoardInfo);
DRV_STATUS   DRV_Fpga_GetConfigStatus(struct tLocalDeviceInfo *pLDI, ui32 *pdwConfigStatus);
DRV_STATUS   DRV_Fpga_ISAConfigure(struct tLocalDeviceInfo *pLDI);
DRV_STATUS   DRV_Fpga_PCIConfigure(struct tLocalDeviceInfo *pLDI);
DRV_STATUS   DRV_Fpga_PCIConfigure_1bit(struct tLocalDeviceInfo *pLDI);
DRV_STATUS   DRV_Fpga_PCIConfigure_32bit(struct tLocalDeviceInfo *pLDI);
DRV_STATUS   DRV_Fpga_Program(struct tLocalDeviceInfo *pLDI);
DRV_STATUS   DRV_Fpga_Reset(struct tLocalDeviceInfo *pLDI);
DRV_STATUS   DRV_Fpga_SendInfo(struct tLocalDeviceInfo *pLDI, struct Tfpga_info *pFpga);
DRV_STATUS   DRV_PCI_DylanIsBusy(struct tLocalDeviceInfo *pLDI, ui32 *pdwDylanStatus);
DRV_STATUS   DRV_PCI_DylanRead_ui32_NonSync(struct tLocalDeviceInfo *pLDI, ui32 dwRegNum, ui32 *pdwData);
DRV_STATUS   DRV_PCI_DylanRead_ui32_Sync(struct tLocalDeviceInfo *pLDI, ui32 dwRegNum, ui32 *pdwData);
DRV_BOOLEAN  DRV_PCI_DylanRead_ui32_KeSync(struct tLocalDeviceInfo *pLDI);
DRV_STATUS   DRV_PCI_DylanWrite_ui32_NonSync(struct tLocalDeviceInfo *pLDI, ui32 dwRegNum, ui32 dwData);
DRV_STATUS   DRV_PCI_DylanWrite_ui32_Sync(struct tLocalDeviceInfo *pLDI, ui32 dwRegNum, ui32 dwData);
DRV_BOOLEAN  DRV_PCI_DylanWrite_ui32_KeSync(struct tLocalDeviceInfo *pLDI);
DRV_STATUS   DRV_PCI_DylanWriteBlock_ui32_NonSync(struct tLocalDeviceInfo *pLDI, ui32 dwCommand, ui8 *pbData);
DRV_STATUS   DRV_PCI_DylanWriteBlock_ui32_Sync(struct tLocalDeviceInfo *pLDI, ui32 dwCommand, ui8 *pbData);
DRV_BOOLEAN  DRV_PCI_DylanWriteBlock_ui32_KeSync(struct tLocalDeviceInfo *pLDI);
DRV_STATUS   DRV_PCI_DylanWaitForNotBusy(struct tLocalDeviceInfo *pLDI, ui32 dwTimeout);
DRV_STATUS   DRV_PCI_ShadowEEPROM(struct tLocalDeviceInfo *pLDI, struct TBoardInfo *pBoardInfo);
DRV_STATUS   DRV_Serial_CheckForCommsTimeout(struct tLocalDeviceInfo *pLDI);
DRV_STATUS   DRV_Serial_ResetComms(struct tLocalDeviceInfo *pLDI);
DRV_BOOLEAN  DRV_Serial_GetInfo(struct tLocalDeviceInfo *pLDI);
DRV_BOOLEAN  DRV_Serial_SendInfo(struct tLocalDeviceInfo *pLDI);
DRV_STATUS   DRV_Serial_RxBuffer(struct tLocalDeviceInfo *pLDI);
DRV_BOOLEAN  DRV_Serial_Rx_CriticalIncrement(struct tLocalDeviceInfo *pLDI);
DRV_STATUS   DRV_Serial_TxBuffer(struct tLocalDeviceInfo *pLDI);
DRV_BOOLEAN  DRV_TableLoad(struct tLocalDeviceInfo *pLDI);
DRV_STATUS   DRV_Timeout_Set(struct tLocalDeviceInfo *pLDI, struct TTimeout *pTimeout);

/*
 * nt.c
 * ----
 */
DRV_STATUS   DRV_BurstIn_ui32(struct tLocalDeviceInfo *pLDI, void* PortAddress, ui32 *pdwData, ui32 dwByteCount);
DRV_STATUS   DRV_ReadComplete(struct tLocalDeviceInfo *pLDI, ui32 dwMode, ui32 *pdwResult);
DRV_STATUS   DRV_BurstInAsync_ui32(struct tLocalDeviceInfo *pLDI, void* PortAddress, ui32 *pdwData, ui32 dwByteCount);
DRV_STATUS   DRV_BurstIn_ui16(struct tLocalDeviceInfo *pLDI, void* PortAddress, ui16 *pData, ui32 dwByteCount);
DRV_STATUS   DRV_BurstIn_ui16_Special(struct tLocalDeviceInfo *pLDI, void* PortAddress, ui16 *pData, ui32 dwPixelCount, ui32 dwMapping);
DRV_STATUS   DRV_BurstOut_ui16(struct tLocalDeviceInfo *pLDI,void* PortAddress, ui16 *pwData, ui32 dwByteCount);
DRV_STATUS   DRV_BurstOut_ui16_Special(struct tLocalDeviceInfo *pLDI, void *PortAddress, ui16 *pwData, ui32 dwPixelCount, ui32 dwMapping);
DRV_BOOLEAN  DRV_DMA_Abort(struct tLocalDeviceInfo *pLDI);
DRV_STATUS   DRV_DMA_Init(struct tLocalDeviceInfo *pLDI);
DRV_STATUS   DRV_DMA_Release(struct tLocalDeviceInfo *pLDI);
DRV_BOOLEAN  DRV_DMA_Start(struct tLocalDeviceInfo *pLDI);
DRV_BOOLEAN DRV_IoTimerSetBefore(struct tLocalDeviceInfo *pLDI);
DRV_BOOLEAN DRV_IoTimerSetDuring(struct tLocalDeviceInfo *pLDI);
void DRV_RaiseIrql(struct tLocalDeviceInfo *pLDI);
void DRV_RestoreIrql(struct tLocalDeviceInfo *pLDI);
DRV_STATUS   DRV_TimeDelay(ui32 dwMicroSecs);
DRV_STATUS   DRV_TimeGetStamp_ms(ui32 *pdwTimeStamp);
DRV_STATUS   DRV_TimeOut(ui32 dwTimeOut, ui32 *pfResult);
/* Not strictly true - void * is really struct tLocalDeviceInfo*, but there is a definition order
 * problem in the header files, hence the opaque pointer type.
 */
void DRV_Timeout_SerialTx(void *);
void DRV_Timeout_SerialRx(void *);
/* 
 * int.c
 * -----
 */
void         DRV_Interrupt_Handler(struct tLocalDeviceInfo *pLDI);
void         DRV_IoTimer(int Arg);
DRV_BOOLEAN  DRV_IoTimerProcess(struct tLocalDeviceInfo *pLDI);
DRV_STATUS   DRV_Interrupt_ClearAndMaskAll(struct tLocalDeviceInfo *pLDI);
DRV_STATUS   DRV_Interrupt_ClearDown(struct tLocalDeviceInfo *pLDI, ui32 *pdwIntType);
void     DRV_Interrupt_Off(struct tLocalDeviceInfo *);
void     DRV_Interrupt_On(struct tLocalDeviceInfo *);
DRV_STATUS   DRV_Interrupt_Process(struct tLocalDeviceInfo *pLDI, ui32 dwIntType);
DRV_STATUS   DRV_Interrupt_ProcessDylan(struct tLocalDeviceInfo *pLDI);
DRV_STATUS   DRV_Interrupt_ProcessDylanRx(struct tLocalDeviceInfo *pLDI, ui32 dwDylanSerialDataAndStatus);
DRV_STATUS   DRV_Interrupt_ProcessDylanTx(struct tLocalDeviceInfo *pLDI);
DRV_STATUS   DRV_Interrupt_ProcessDylanTimer(struct tLocalDeviceInfo *pLDI);
DRV_STATUS   DRV_Interrupt_ProcessSnapper(struct tLocalDeviceInfo *pLDI);
DRV_STATUS   DRV_Interrupt_ReadSnapper(struct tLocalDeviceInfo *pLDI, DRV_BOOLEAN *pfSnapperInterrupt);
DRV_STATUS   DRV_Interrupt_ReadDylan(struct tLocalDeviceInfo *pLDI, DRV_BOOLEAN *pfDylanInterrupt);
DRV_STATUS   DRV_Interrupt_ReadStatus(struct tLocalDeviceInfo *pLDI, DRV_BOOLEAN *pfIRQ_Pending);
DRV_BOOLEAN  DRV_Interrupt_Rx_CriticalIncrement(struct tLocalDeviceInfo *pLDI);
void DRV_Interrupt_Complete_DMA(struct tLocalDeviceInfo*);
void DRV_Interrupt_Complete_SerialRx(struct tLocalDeviceInfo*);
void DRV_Interrupt_Complete_SerialTx(struct tLocalDeviceInfo*);
void DRV_Interrupt_Complete_Snapper(struct tLocalDeviceInfo *pLDI);
DRV_BOOLEAN  DRV_Interrupt_SerialEnable(struct tLocalDeviceInfo *pLDI);
DRV_BOOLEAN  DRV_Interrupt_Set(struct tLocalDeviceInfo *pLDI);

DRV_BOOLEAN  DrvSynchronizeExecution(struct tLocalDeviceInfo*, DRV_BOOLEAN (*)(struct tLocalDeviceInfo*));
DRV_BOOLEAN  DrvSignalSemaphore(SEM_ID*);
DRV_BOOLEAN  DrvInitializeSemaphore(SEM_ID*, ui32, ui32);
DRV_BOOLEAN  DrvWaitForSemaphore(SEM_ID*,ui32);
DRV_BOOLEAN  DrvDeleteSemaphore(SEM_ID *pSemaphore);
void DrvTimerSignalInterrupt(int , siginfo_t *, void *);
DRV_BOOLEAN  DrvStartTimer(struct tVxDriverTimer*);
DRV_BOOLEAN  DrvCancelTimer(struct tVxDriverTimer*);
DRV_BOOLEAN  DrvCreateTimer(struct tVxDriverTimer*);
DRV_BOOLEAN  DrvDeleteTimer(struct tVxDriverTimer*);

/* From vx_sg.c */
DRV_STATUS DRV_Physmem_Unlock(struct tLocalDeviceInfo *pLDI, struct TBurstTransfer *pBurstInfo, ui32 *pSGTable);
DRV_STATUS DRV_Physmem_Lock(struct tLocalDeviceInfo *pLDI, struct TBurstTransfer *pBurstInfo, ui32 *pSGTable);
DRV_STATUS DRV_Copy_SGTable(struct tLocalDeviceInfo *pLDI, ui32 *pSGTable);

/* Snapper I/O control codes
 * These are used to give code compatibility between the standard
 * driver environment (e.g. WindowsNT) and VxWorks. However, in
 * VxWorks, there is not an separate driver.
 * An enum is used in preference to #defines, as these are symbolically 
 * visible at run time in the debugger
 */
enum SnapperIoControlCode {
   IOCTL_SNAPPER_OPEN_DEVICE,
   IOCTL_SNAPPER_WRITE_UI8,
   IOCTL_SNAPPER_READ_UI8,
   IOCTL_SNAPPER_WRITE_UI16,
   IOCTL_SNAPPER_READ_UI16,
   IOCTL_SNAPPER_WRITE_UI32,
   IOCTL_SNAPPER_READ_UI32,
   IOCTL_SNAPPER_DYLAN_WRITE_UI32,
   IOCTL_SNAPPER_DYLAN_READ_UI32,
   IOCTL_SNAPPER_FPGA_SEND_INFO,
   IOCTL_SNAPPER_FPGA_SEND_DATA,
   IOCTL_SNAPPER_FPGA_RESET,
   IOCTL_SNAPPER_FPGA_GET_CONFIG_STATUS,
   IOCTL_SNAPPER_BURST_IN_UI16,
   IOCTL_SNAPPER_BURST_IN_UI32,
   IOCTL_SNAPPER_BURST_IN_UI32_ASYNC,
   IOCTL_SNAPPER_READ_COMPLETE,
   IOCTL_SNAPPER_BURST_IN_UI16_SPECIAL,
   IOCTL_SNAPPER_BURST_OUT_UI16,
   IOCTL_SNAPPER_TABLE_LOAD,
   IOCTL_SNAPPER_PAGE_WRITE_UI16,
   IOCTL_SNAPPER_BURST_OUT_UI16_SPECIAL,
   IOCTL_SNAPPER_GET_ERROR_STRING,
   IOCTL_SNAPPER_ABORT_OPERATION,
   IOCTL_SNAPPER_TIMEOUT_SET,
   IOCTL_SNAPPER_SERIAL_TX,
   IOCTL_SNAPPER_SERIAL_RX,
   IOCTL_SNAPPER_SERIAL_SEND_INFO,
   IOCTL_SNAPPER_SERIAL_GET_INFO,
   IOCTL_SNAPPER_INTERRUPT_SET,
   IOCTL_SNAPPER_INTERRUPT_STATUS,
   IOCTL_SNAPPER_SGTABLE_SEND_INFO,
   IOCTL_SNAPPER_PHYSMEM_LOCK,
   IOCTL_SNAPPER_PHYSMEM_UNLOCK
};

DRV_BOOLEAN  SnapperIoControl(void*,  /* Device handle (actually struct tLocalDeviceInfo*) */
                              enum SnapperIoControlCode, /* Code for requested operation */
                              void*,  /* Pointer to input parameters */
                              size_t, /* Size of input parameters structure */
                              void*,  /* Pointer rto output parameters */
                              size_t, /* Size of output parameters structure */
                              ui32*,  /* Number of bytes written to output structure */
                              void*); /* Not used */


struct tLocalDeviceInfo *SnapperAttach(struct tDeviceInfo*);
void *Snapper_Open(int);
DRV_BOOLEAN Snapper_Close(void*);
void DrvSetDefaultIoFuncs(struct tDeviceInfo *pDI);



