/**************************************************************************
 *                ACTIVE SILICON LIMITED / ACTIVE IMAGING LIMITED         *
 *                                                                        *
 *  File name   : base.h                                                  *
 *  Function    : Definitions for Baseboard Library functions             *
 *  Project     : Snappers                                                *
 *  Author      : Colin Pearce, Active Silicon Ltd                        *
 *  Systems     : ANSI C                                                  *
 *  Written     : October 1993                                            *
 *  Updated     : 15-Jan-96                                               *
 *  Version     : 3.10                                                    *
 *                                                                        *
 *  Copyright (c) 1993-96 Active Silicon, Active Imaging Ltd.             *
 **************************************************************************
 *  Comments:
 */

#ifndef _base_h
#define _base_h


/* maximum number of Baseboard instances */
#define BASE_NUM_HANDLES  16
#define BASE_ALL_HANDLES  1000    /* unique identifier */


/* Error Codes
 * -----------
 * The error return code consists of 4 bytes.  The lower 2 contain any
 * parameter information from the function.  The next byte contains an
 * error code, and the highest byte contains the board ID of the
 * device signalling the error.  This technique allows the SNAPPER SW to
 * return an error although the original call may have been made
 * indirectly through a baseboard call.
 */
#define ISA_ID    ((ui8) 0x20)
#define PCI_ID    ((ui8) 0x40)
#define SBUS_ID   ((ui8) 0x50)

#define MODULE_NULL_HANDLE        ((Terr) 0)
#define BASE_NULL_HANDLE          ((Terr) 0)

/*
 * Handle type definitions (see also shared.h)
 */
typedef Thandle   Tmodule_handle;

/* #defines to be used with BASE_get_parameter */
/* NOTE :
 * BASE_MODULE_FAMILY returns the Snapper Family ID (but not REV).  This
 * has been added at value 7 such that existing applications which call
 * BASE_MODULE_IDREV will not need to be recompiled for use with the
 * SNP-PCI-xxx boards.
 * BASE_MODULE_IDREV has been renamed as BASE_MODULE_IDREV_VALUE to make it
 * consistent across the Snapper libraries and also to force the user to
 * check their applications for the new return values.  (In most cases the
 * new BASE_MODULE_FAMILY call will be more appropriate).
 */
#define BASE_DMA_FAILURE_COUNT    1
#define BASE_DMA_SUCCESS_COUNT    2
#define BASE_BASEBOARD_TYPE       3
#define BASE_IDREV_VALUE          4
#define BASE_LIBRARY_REV_LEVEL    5
#define BASE_MODULE_HANDLE        6
#define BASE_MODULE_FAMILY_VALUE  7
#define BASE_MODULE_FAMILY        BASE_MODULE_FAMILY_VALUE /* For Compatibility */
#define BASE_IRQ                  8
#define BASE_MODULE_IDREV_VALUE   9
#define BASE_ID_VALUE            10
#define BASE_REV_VALUE           11

/* #defines to to be with BASE_serial_set/get_parameter */
#define BASE_SERIAL_INIT             0
#define BASE_SERIAL_BAUDRATE         1
#define BASE_SERIAL_DATA_BITS        2
#define BASE_SERIAL_STOP_BITS        3
#define BASE_SERIAL_XON_XOFF_ENABLE  4
#define BASE_SERIAL_XON_XOFF_STATUS  5   /* get only, INIT resets it */
#define BASE_SERIAL_XON_CHAR         6
#define BASE_SERIAL_XOFF_CHAR        7
#define BASE_SERIAL_PARITY_MODE      8
#define BASE_SERIAL_PARITY_IGNORE    9
#define BASE_SERIAL_RX_COUNT         10  /* get only, INIT resets it */
#define BASE_SERIAL_PARITY_ERRORS    11  /* get only, INIT resets it */
#define BASE_SERIAL_OVERFLOW_ERRORS  12  /* get only, INIT resets it */

/* #defines to to be with BASE_set_ptr */
#define BASE_SG_TABLE           0

/* Miscellaneous #defines related to serial comms */
#define XON                    17
#define XOFF                   19


/* The Tproperty structure contains baseboard properties after they have
 * been parsed.  This allows quick access to property values, and values
 * to be manipulated if necessary.
 *
struct_Tproperty(ForBrief)
 */
#define BASE_NUM_PROPERTIES          20
#define BASE_PROPERTY_LENGTH         20
#define BASE_PROPERTY_VALUE_LENGTH   20
struct Tproperty
{
   char property[BASE_PROPERTY_LENGTH];
   char value[BASE_PROPERTY_VALUE_LENGTH];
};


/* Baseboard Structure
 * -------------------
struct_Tbase(ForBrief)
 */
struct Tbase {
   ui32 dwBackReference;   /* Copy ofthe handle number that references
                            * this structure instance. */
   ui32 dwLibraryRevLevel;
   ui32 dwIRQ;              /* By interrogation of driver in OS_Open */ 
   ui8  IDrev;
   ui16 board_type;
   
#define BASE_UNDEFINED_BOARD 0
#define BASE_ISA_BIB  1
#define BASE_ISA_JPG  2
#define BASE_PCI_BIB  3
#define BASE_PCI_JPG  4
#define BASE_SBUS_BIB 5
   char base_date_string[RBT_ASL_DATE_LENGTH];   /* Date read from FPGA file   */ 
   char mapper_date_string[RBT_ASL_DATE_LENGTH]; /* Date read from FPGA file   */ 
   char mapper_name_string[RBT_ASL_NAME_LENGTH]; /* Name of current Datamapper */ 
   ui16 mapper_input_format;                     /* Datamapper input format    */
   ui16 mapper_output_format;                    /* Datamapper output format (these   */
                                                 /* are the TMG internal image types) */

  /*
   * Serial Comms (not available on ISA)
   * -----------------------------------
   * These elements are used by the Snapper library only and are
   * not required by the driver. In other words these parameters
   * are set up by the user calling BASE_serial_set_parameter, which
   * in turn maps to standard OS_writes to the hardware.
   * These comms elements could perhaps be a separate structure called
   * struct "TSerialInfo_Lib", but reference by the base structure.
   */
   ui32 serial_baud_rate;   /* up to 125k typically */
   ui8  serial_data_bits;   /* 7 or 8 */
   ui8  serial_stop_bits;   /* 1 or 2 */
   ui8  serial_parity_mode;
#define BASE_SERIAL_NONE   0
#define BASE_SERIAL_ODD    1
#define BASE_SERIAL_EVEN   2
#define BASE_SERIAL_MARK   3
#define BASE_SERIAL_SPACE  4

/* pointer to structure of common serial information - see shared.h */
struct TSerialInfo_Common *pSIC;

#ifndef _FOR_WIN95  /* Windows 16 bit for Windows 95 uses the 95 VxD */
#if defined _DOS || defined _WIN31   /* not "for Windows 95" */
/*
 * In a "real" operating system, the structure "TSerialInfo_Driver" 
 * would contain the following members and be private to the driver.
 * For DOS and Windows they are conveniently put in this base structure.
 */
#ifndef BASE_COMMS_BUFFER_SIZE
#ifdef _DOS16
#define BASE_COMMS_BUFFER_SIZE  16
#else
#define BASE_COMMS_BUFFER_SIZE  1024
#endif
#endif

   ui16 serial_rx_buffer[BASE_COMMS_BUFFER_SIZE];
   ui32 serial_rx_rd_index;
   ui32 serial_rx_wr_index;
   volatile ui8  serial_tx_ready;        /* TRUE if ready for another char, FALSE if not */
   volatile ui32 serial_errors_in_buffer;

#endif  /* for DOS/Windows 3.1 only */
#endif  /* for _FOR_WIN95 */

#if defined _DOS || defined _WIN31   /* Also 16 bit windows: "for Windows 95" */

/* Similarly, these variables should be local to the driver, but for 
 * DOS/Win31 we'll do it here. And the mmtask stuff needs them running
 * 16 bit under Windows 95.
 */
   /* timeouts in milliseconds */
   ui32 dwTimeoutBeforeStart;  /* timeout for first receiving data    */
   ui32 dwTimeoutDuring;       /* timeout for each DMA read operation */
#endif


/* Set Datamapper command definitions */

#define MAPPER_SET_FORMAT              ((ui16) 0x00)
#define MAPPER_SET_HBANK_COUNT         ((ui16) 0x01)
#define MAPPER_SET_PCI_SNAPCLK         ((ui16) 0x02)
#define MAPPER_SET_TEST_FPGA           ((ui16) 0x03)
#define MAPPER_RESTORE                 ((ui16) 0x04)
#define MAPPER_FLUSH                   ((ui16) 0x05)
#define MAPPER_SYNCHRONOUS_INTERFACE   ((ui16) 0x06)
#define MAPPER_ASYNCHRONOUS_INTERFACE  ((ui16) 0x07)
#define MAPPER_TOGGLE_VBANK            ((ui16) 0x08)
#define MAPPER_INTEGRATED_INTERFACE    ((ui16) 0x09)

/* Get Datamapper command definitions */
#define MAPPER_GET_INPUT_FORMAT        ((ui16) 0x00)
#define MAPPER_GET_OUTPUT_FORMAT       ((ui16) 0x01)
#define MAPPER_GET_HBANK_COUNT         ((ui16) 0x02)
#define MAPPER_GET_RATIO_NUMERATOR     ((ui16) 0x03)
#define MAPPER_GET_RATIO_DENOMINATOR   ((ui16) 0x04)

/* Set Datamapper HBANK_DISABLE definition */
#define MAPPER_HBANK_DISABLE        ((ui16) 0x00)
                                       
   ui32 mapper_hbank;                  /* Datamapper HBANK counter value */
   ui32 mapper_ratio_numerator;        /* Datamapper pixels/read numerator */
   ui32 mapper_ratio_denominator;      /* Datamapper pixels/read denominator */
   ui32 mapper_snapper_sync_interface; /* Set to TRUE for a synchronous Snapper interface */
   ui32 mapper_serial_ctrl;            /* Datamapper serial control string value */
   ui32 base_addr;
   ui16 wRequestedAddr;                /* Requested "base address" e.g. 0x300 (ISA) or 8001/auto etc */

   Tboolean mode_changed;              /* set to TRUE if a reconfig is required */

   /* Driver software mappings - required for Crunch-ISA */
   ui16 burst_read_mode;  /* for possible values see "shared.h" */
   ui16 burst_write_mode;

   struct Tbase_registers *Pbase_reg;  /* pointer to ANY baseboard register structure */
   Tmodule_handle Hmodule;             /* handle to Snapper etc */
   ui8 module_IDrev;                   /* ID & rev for module   */
   ui8 dummy_for_packing;              /* keep alignment for MSVC */
   
   struct Tcrunch *Pcrunch;            /* pointer to Crunch related stuff */
   struct Tcrunch *Pnew1;              /* pointer to new HW function */
   struct Tcrunch *Pnew2;              /* pointer to new HW function */
   struct Tcrunch *Pnew3;              /* pointer to new HW function */

#if defined _WIN95 || defined _WINNT
   HANDLE hDevice;               /* Handle to Win95/NT device driver */
#define BASE_WIN32_DEVICE_NAMELENGTH 64
   char szDeviceName[BASE_WIN32_DEVICE_NAMELENGTH];
#if defined _WINNT
   HANDLE hAsyncDevice;          /* Handle opened with FILE_FLAG_OVERLAPPED */
   OVERLAPPED sOverlapped;       /* Overlapped structure used by burst_in_ui32_async */
#endif
#elif _WIN31                     /* OK for native Win31 and 16 bit under 95 */
   ui16 hDevice;                 /* Compatible with MSVC and SC (for pci_asm) */
   ui32 dwVxDEntryAddr;          /* VxD entry point for use under Win95 */
#elif _LYNXOS
   ui32 hDevice;                 /* Actually a file handle */
#elif defined _OS2 
   int descriptor;               /* Device driver descriptor */
#elif defined _SOLARIS || defined _LINUX
   int descriptor;               /* Device driver descriptor */
#if !defined _SOLARIS2_USE_AIO
   /* New code section to get round aio_read functions huge setup latency */
   pthread_t AsyncThread;                     /* The thread which performs the I/O */
   pthread_attr_t AsyncThreadAttr;
   pthread_mutex_t AsyncThreadMutex;
   pthread_mutexattr_t AsyncThreadMutexAttr;
   pthread_cond_t AsyncThreadCond;
   pthread_condattr_t AsyncThreadCondAttr;
   pthread_cond_t AsyncThreadDoneCond;
   pthread_condattr_t AsyncThreadDoneCondAttr;
   ui32 AsyncAddrOffset;
   IM_UI32 *pAsyncData;
   ui32 AsyncByteCount;
   Tboolean bAsyncThreadOperationFinished;
   Terr AsyncThreadReturn;
   Tboolean bAsyncThreadExit;
   Thandle hBase;
#else
   struct aiocb aioControl;
#endif
#elif defined _MAC_LVR
   short refNum;                 /* Device driver reference */
#elif defined _VXWORKS
   void *hDevice;		            /* Actually a pointer to the tLocalDeviceInfo structure! */
#elif defined _MACINTOSH
   short refNum;                 /* Reference to open device descriptor */
#elif defined _MACOSX
   io_connect_t hDevice;         /* used as the handle int the device */
   pthread_t threadID;           /* Used for asynchronous callbacks */
#endif
   ui16 device_number;           /* Generic - 0/1/2/3 etc */

   ui8 bus_number;     /* for PCI */
   ui8 dev_func_num;   /* for PCI */

#ifndef _FOR_WIN95
#ifdef _WIN31  /* Variables for pre-locking DMA areas under Windows 3.1 */
   IM_UI8 *pData;           /* local copy of TMG image data pointer */
   ui32 NumBytes;           /* Size in bytes of strip */
   ui32 NumRegions;         /* Number of DMA regions  */
   struct TRegionListElement *pRegionList;  /* array of DMA regions */
#ifdef _OS_THREADS
#define NUM_DMA_REGIONS   ((ui32) 4096) /* 4096 x 4096 (DMA block) = 16Mb */
#else
#define NUM_DMA_REGIONS   ((ui32) 2048) /* 2048 x 4096 (DMA block) = 8Mb */
#endif  /* else part of _OS_THREADS */
#endif  /* for _WIN31 */
#endif  /* not for _FOR_WIN95 */
   ui32 DMASuccessCount;    /* for monitoring how well DMA is performing */
   ui32 DMAFailureCount;    /* under Windows */

   Tboolean property_table_valid;          /* Has property table been constructed yet ? */
   ui8 eeprom[BASE_PCIMAX_EEPROM_LENGTH];  /* To shadow PCI EEPROM */
   struct Tproperty property_table[BASE_NUM_PROPERTIES];  /* Array of property values */

  /* 
   * Interrupt callback to user API.
   */
#if defined _WIN95 || defined _WINNT 
   HANDLE hInterruptThread;    /* For Win32 threads */
                               /* NOTE: ui32 for _beginthread */
   ui32 dwInterruptThreadId;   /* For Win32 threads */
#elif defined _VXWORKS
   int hInterruptThread;
#elif defined _LYNXOS
   char szThreadName[32];
   pthread_t ptThreadID;             /* User callback processing thread */
   pthread_attr_t ptaThreadAttr;     /* User callback thread attributes */
   pthread_mutex_t ptmThreadBusySem; /* User callback mutex to flag when the handler is busy */
   pthread_mutexattr_t ptmThreadMutexAttr;  /* user callback mutex attributes block */

#elif defined _SOLARIS2 || defined _LINUX
   Tboolean pthreadCreated;
   pthread_t pthreadID;
   pthread_attr_t pthreadAttr;
   Thandle pthreadArg;

#elif defined _MAC_LVR
   ThreadID hInterruptThread;  /* NB MacOS threads are non-preemptive, as at MacOS 8.0 */
#elif defined _MACINTOSH
   /* Mac device I/O calls may ru at interrupt level, and cannot specifically block.
    * The only thing to do is to use an asynchronous I/O call, and then suspend the
    * calling thread manually. The I/O completion routine then wakes up the appropriate
    * thread. Hence, we need to know both thread ID's.
    */
   ThreadID hMainThreadID;
   ThreadID hInterruptThreadID;
   
#endif

#if defined _OS2 || defined __WATCOMC__
   void (EXPORT_FN *pFnCallback)(Thandle, ui32, void*);
#else
   void (*pFnCallback)(Thandle, ui32, void*);  /* Points to user "ISR" */
#endif

   void *pCallbackParameter;   /* Parameter to pass to callback function */

   ui32 *pSGTable;  /* Pointer to scatter/gather table (see shared.h) - used
                     * for DMAing direct to a device */
};  /* End base structure */


/* Datamapper FPGA Structure (map/pak definition, pixel/read ratio
 * & serial control string)
 */
struct Tdm_fpga 
{
   ui32  mappak;           /* Lowest byte gives packing function (see mappers.xls)
                            * Next byte is mapping function      (see mappers.xls)
                            * Next byte is unpacking function    (see mappers.xls)
                            * Packing, mapping or unpacking value of 0xFF
                            * indicates not supported.
                            * Upper two bits of upper byte is ISA pixel bus control
                            * bits:  0x00 = not supported  0x40 = 16 bit I/F
                            *        0x80 = 24 bit I/F     0xC0 = (8 bit, upper byte)
                            */
   ui32  ctrl;             /* Not used at present (no datamappers support programmable formats) */
   ui8   ctrl_bits;        /* Number of bits in datamapper serial control shift register */
   ui8   hbank_pos;        /* Position of HBANK count value in serial control shift register */
};


/* Baseboard Registers */
/* ------------------- */
struct Tbase_registers {
/*
 * ISA baseboard definitions:
 */
   ui16 isa_control_reg1;
#define ISA_SW_RESET    0x01   /* resets board and CL550 */
#define ISA_COMPRESS    0x02   /* set 1 for compress 0 for decompress */
#define ISA_HALF_RATE   0x04
#define ISA_SEL_DECOMP  0x38   /* All high - use in decompression */
#define ISA_SEL_ISA     0x30   /* active low - bit 3 */
#define ISA_SEL_VID     0x28   /* active low - bit 4 */
#define ISA_SEL_EXT     0x18   /* active low - bit 5 */
#define ISA_TEST        0x40   /* puts CL550 into test mode      */
#define ISA_MHZ30       0x80   /* 1 to run at 30MHz, 0 for 10MHz */

#define ISA_X_A0        0x0100  /* external FIFO address lines */
#define ISA_X_A1        0x0200
#define ISA_M_A0        0x0400  /* module (piggy back) address lines */
#define ISA_M_A1        0x0800
#define ISA_M_A2        0x1000
#define ISA_MODULE_INDEX  0x1C00         

   ui16 isa_control_reg2;     /* FPGA internal control register */
#define ISA_VID_RAW    0x01
#define ISA_VRAW_IN    0x02
#define ISA_PBUS_24    0x04
#define ISA_AUTO_INC   0x08  /* auto-increments host bus index register  */
#define ISA_NO_XFIFO   0x40  /* runs software without external FIFO      */
#define ISA_HBUS_RUN   0x80  /* enables external FIFO to Codec transfers */
#define ISA_PBUS_RUN   0x20  /* enables Pixel Bus latch to CL550 transfers  */
#define ISA_FLUSH_FIFO 0x10  /* enables Host bus to flush the internal FIFO */

   ui16 isa_irq_mask_reg;

/*
 * PCI baseboard definitions
 */
   ui32 pci_control_reg;
#define PCI_BIT_CCLK       ((ui32) 0x00000040)   /* CCLK to Datamapper and Snapper FPGAs */
#define PCI_BIT_DIN        ((ui32) 0x00000080)   /* DIN to Datamapper and Snapper FPGAs */
#define PCI_BIT_DMCE       ((ui32) 0x00002000)   /* Datamapper chip enable */
#define PCI_BIT_LED        ((ui32) 0x00004000)   /* LED control (clear = on) */
#define PCI_BIT_MRS        ((ui32) 0x00010000)   /* Datamapper FPGA serial port MRS */
#define PCI_BIT_MCK        ((ui32) 0x00020000)   /* Datamapper FPGA serial port MCK */
#define PCI_BIT_MLD        ((ui32) 0x00040000)   /* Datamapper FPGA serial port MLD */
#define PCI_BIT_MRD        ((ui32) 0x00080000)   /* Datamapper FPGA serial port MRD */
#define PCI_BIT_MCS        ((ui32) 0x00100000)   /* Datamapper FPGA serial port MCS */
#define PCI_BIT_MSD        ((ui32) 0x00200000)   /* Datamapper FPGA serial port MSD */
#define PCI_BITS_DM_PROG   ((ui32) 0x00400010)   /* Datamapper FPGA DONE/PROGRAM, RST */
#define PCI_BITS_SN_PROG   ((ui32) 0x00800020)   /* Snapper FPGA DONE/PROGRAM, RST */
#define PCI_MASK_SERIAL    ((ui32) 0x003E0000)   /* Datamapper serial control mask */

/* Shadow of Dylan serial HW register */
   ui16 dylan_serial_reg;
#define PCI_SERIAL_HMASK_BAUD      ((ui16) 0x0ff00)
#define PCI_SERIAL_HMASK_X         PCI_SERIAL_BIT_X4
#define PCI_SERIAL_BIT_X4          ((ui16) 0x0080)
#define PCI_SERIAL_BIT_X12         ((ui16) 0x0000)
#define PCI_SERIAL_HMASK_N         PCI_SERIAL_BIT_N2
#define PCI_SERIAL_BIT_N2          ((ui16) 0x0040)
#define PCI_SERIAL_BIT_N1          ((ui16) 0x0000)
#define PCI_SERIAL_HMASK_STOP      PCI_SERIAL_BIT_STOP2
#define PCI_SERIAL_BIT_STOP2       ((ui16) 0x0020)
#define PCI_SERIAL_BIT_STOP1       ((ui16) 0x0000)
#define PCI_SERIAL_HMASK_DATA      PCI_SERIAL_BIT_DATA8
#define PCI_SERIAL_BIT_DATA8       ((ui16) 0x0010)
#define PCI_SERIAL_BIT_DATA7       ((ui16) 0x0000)
#define PCI_SERIAL_HMASK_SERRST    PCI_SERIAL_BIT_SERRST
#define PCI_SERIAL_BIT_SERRST      ((ui16) 0x0008)
#define PCI_SERIAL_HMASK_PARITY    ((ui16) 0x0007)
#define PCI_SERIAL_BITS_EVEN       ((ui16) 0x0000)
#define PCI_SERIAL_BITS_ODD        ((ui16) 0x0001)
#define PCI_SERIAL_BITS_MARK       ((ui16) 0x0002)
#define PCI_SERIAL_BITS_SPACE      ((ui16) 0x0003)
#define PCI_SERIAL_BITS_NONE       ((ui16) 0x0004)

/*
 * SBus baseboard definitions
 */
   ui32 sbus_reg_8051_control2;
};

/* General definitions
 * -------------------
 * See also shared.h
 */
#define BASE_MAPPER_FITTED        ((Terr) 1)
#define BASE_MAPPER_NOT_FITTED    ((Terr) 0)

#define BASE_DMA_ENABLED          ((Terr) 1)
#define BASE_DMA_NOT_ENABLED      ((Terr) 0)


/*
BASE_set_timer()
----------------
*/
#define BASE_TIMER_INIT             ((Tparam) 0x00000001L)
#define BASE_TIMER_MONOSTABLE       ((Tparam) 0x00000002L)
#define BASE_TIMER_ASTABLE          ((Tparam) 0x00000004L)
#define BASE_TIMER_START_AND_WAIT   ((Tparam) 0x00000008L)
#define BASE_TIMER_START_AND_RETURN ((Tparam) 0x00000010L)

#define BASE_TIMER_INVALID_PARAM    ((Tparam) 0xFFFFFFE0L)

/* These defines are used to implement the config file settings names
   written and read by SnapperTool and now by the Snapper library.
   They are defines to allow easy changes to be made - ie spelling mistake corrections etc.
   They are located here because some are generic to all Snappers.
   The individual name array declarations specific to Snappers are located in Snp16.h, DIG16.h and SNP24.h.
 */

/* dwFormat settings */
#define SNP_INI_IS_FILE           0 /* pvData is null-terminated filename */
#define SNP_INI_IS_BUFFER         1 /* pvData is pointer to memory buffer of null-terminated ascii strings*/
#define SNP_INI_IS_UNICODE_FILE   2
#define SNP_INI_IS_UNICODE_BUFFER 3

#define SNPINI_SNP_INI_FILE_NAME       "snap.ini"

/* Names of parameter section headers */
#define SNPINI_SNP_INI_MATCH_NAME      "[General Parameters]"
#define SNPINI_SNP16_INI_MATCH_NAME    "[SNAPPER-16 Parameters]"
#define SNPINI_SNP24_INI_MATCH_NAME    "[SNAPPER-24 Parameters]"
#define SNPINI_SNPD16_INI_MATCH_NAME   "[SNAPPER-DIG16 Parameters]"

/* Tokens for specifying which Snapper config file type to load */
#define SNPINI_SNP_INI         0
#define SNPINI_SNP16_INI       1
#define SNPINI_SNP24_INI       2
#define SNPINI_SNPD16_INI      3
#define SNPINI_SNP8_INI        4

/* Default LUT values */
#define SNPINI_DEFAULT_GAMMA_LEVEL      200
#define SNPINI_DEFAULT_CONTRAST_LEVEL   100
#define SNPINI_DEFAULT_BRIGHTNESS_LEVEL 100
#define SNPINI_DEFAULT_RED_INTENSITY    100
#define SNPINI_DEFAULT_GREEN_INTENSITY  100
#define SNPINI_DEFAULT_BLUE_INTENSITY   100

#define SNPINI_SNAPPER_TYPE_NAME            "Snapper Type"

/* These are generic across more than one Snapper type */
#define SNPINI_VIDEO_STANDARD_NAME          "Video Standard"
#define SNPINI_VIDEO_SOURCE_NAME            "Video Source"
#define SNPINI_INTEGRATION_MODE_NAME        "Integration Mode"
#define SNPINI_INTEGRATION_TIME_NAME        "Integration Time"
#define SNPINI_SUB_SAMPLING_MODE_NAME       "SubSample Mode"
#define SNPINI_CAPTURE_MODE_NAME            "Capture Mode"

/* LUT - related */
#define SNPINI_BRIGHTNESS_LEVEL_NAME        "Brightness Level"
#define SNPINI_CONTRAST_LEVEL_NAME          "Contrast Level"
#define SNPINI_GAMMA_LEVEL_NAME             "Gamma Level"
#define SNPINI_RED_INTENSITY_NAME           "Red Intensity"
#define SNPINI_GREEN_INTENSITY_NAME         "Green Intensity"
#define SNPINI_BLUE_INTENSITY_NAME          "Blue Intensity"
/* ROI - related */
/* Because of a botch in SnapperTool ,these have different names depending 
   on whether you are using a DIG16 or a SNP24/16 
   eg : SNP24/16
   #define ROI_XSTART_NAME              "Active Area X Start"
   DIG16:
   #define ROI_XSTART_NAME              "X Start"  etc...
*/
#define SNPINI_ROI_XSTART_NAME              "Active Area X Start"
#define SNPINI_ROI_YSTART_NAME              "Active Area Y Start"
#define SNPINI_ROI_XLENGTH_NAME             "Active Area X Length"
#define SNPINI_ROI_YLENGTH_NAME             "Active Area Y Length"
#define SNPINI_DIG16_ROI_XSTART_NAME        "X Start"
#define SNPINI_DIG16_ROI_YSTART_NAME        "Y Start"
#define SNPINI_DIG16_ROI_XLENGTH_NAME       "X Length"
#define SNPINI_DIG16_ROI_YLENGTH_NAME       "Y Length"

/* These SNPINI_for SNP16 */
#define SNPINI_PLL_MODE_NAME                "Phase Lock Loop"
#define SNPINI_EXTERNAL_TRIGGER_MODE_NAME   "External Trigger"

/* These for SNP24 */
#define SNPINI_TRIGGER_JUMPER_MODE_NAME     "Trigger Jumper Mode"
#define SNPINI_HSYNC_JUMPER_MODE_NAME       "HSync Jumper Mode"
#define SNPINI_VSYNC_JUMPER_MODE_NAME       "VSync Jumper Mode"
#define SNPINI_PCLK_JUMPER_MODE_NAME        "PClk Jumper Mode"
#define SNPINI_TRIGGER_MODE_NAME            "Trigger Mode"          /* Which edge of pclk, hsync, vsync etc*/
#define SNPINI_TRIGGER_SOURCE_NAME          "Trigger Source"        /* pclk, vsync, hsync, trig1 only, must be before TRIGGER_MODE_NAME  */
/* Due to a problem with SnapperTool writing the cfg file, these names are reversed */
#define SNPINI_SYNC_SOURCE_NAME             "Sync Mode"           /* Internal or external */
#define SNPINI_SYNC_MODE_NAME               "Sync Source"             
#define SNPINI_CLOCK_MODE_NAME              "Clock Mode"            /* PLL slow/fast lock , external pclk input polarity */
#define SNPINI_SYNC_OUTPUT_MODE_NAME        "Sync Output Mode"
#define SNPINI_COLOUR_MONO_INPUT_MODE_NAME  "Colour-Mono Input Mode"
#define SNPINI_INTERLACE_MODE_NAME          "Interlace Mode"
#define SNPINI_HSYNC_PHASE_MODE_NAME        "HSync Phase Mode"
#define SNPINI_LINE_FREQUENCY_NAME          "Line Frequency"        /* Sets horizontal line frequency. Use to calc pixels per line  */
#define SNPINI_PIXEL_FREQUENCY_NAME         "Pixel Frequency"       /* Sets individual pixel frequency. Use to calc pixels per line */
#define SNPINI_CUSTOM_LM1882_VALUES_NAME    "Custom lm1882 Value_"  
#define SNPINI_LM1882_CLOCKF_NAME           "lm1882 Clock Frequency" /* Selects which crystal to use as source for custom settings. */

/* These for DIG16 */
#define SNPINI_CLOCK_FREQ_MODE_NAME              "Clock Frequency Mode"
#define SNPINI_CAMERA_STANDARD_NAME              "Camera Standard"
#define SNPINI_STANDARD_CAMERA_NAME              "Standard Camera"
#define SNPINI_NON_STANDARD_CAMERA_NAME          "Non Standard Camera"
#define SNPINI_SQUARE_PULSE_MODE_NAME            "Square Pulse Mode"
#define SNPINI_SQUARE_PULSE_TIME_NAME            "Square Pulse Time"
#define SNPINI_OUTPUT_ENABLE_NAME                "Output Mode"
#define SNPINI_OUTPUT_A_MODE_NAME                "Output A Mode"
#define SNPINI_OUTPUT_B_MODE_NAME                "Output B Mode"
#define SNPINI_OUTPUT_C_MODE_NAME                "Output C Mode"
#define SNPINI_OUTPUT_D_MODE_NAME                "Output D Mode"
#define SNPINI_IO_LINE_A_MODE_NAME               "IO Line A Mode"
#define SNPINI_IO_LINE_B_MODE_NAME               "IO Line B Mode"
#define SNPINI_IO_LINE_C_MODE_NAME               "IO Line C Mode"
#define SNPINI_IO_LINE_D_MODE_NAME               "IO Line D Mode"
#define SNPINI_MSB_POSITION_NAME                 "MSB Position"
#define SNPINI_LSB_POSITION_NAME                 "LSB Position"
#define SNPINI_BIT_DEPTH_NAME                    "Bit Depth"
#define SNPINI_ALIGNMENT_MODE_NAME               "Alignment Mode"
#define SNPINI_CAPTURE_FIELD_MODE_NAME           "Capture Field Mode"
#define SNPINI_CLOCK_SOURCE_MODE_NAME            "Clock Source Mode"
#define SNPINI_CLOCK_FREQUENCY_MODE_NAME         "Clock Frequency Mode"
#define SNPINI_CONTINUOUS_CLOCK_FLAG_NAME        "Continous Clock Flag"/* sic */
#define SNPINI_TWO_TAP_FLAG_NAME                 "Two Tap Flag"

/* Linescan specific */
#define SNPINI_LINE_ACCEPTANCE_MODE_NAME         "Line Acceptance Mode"
#define SNPINI_PULSE_TO_CAMERA_MODE_NAME         "Pulse To Camera Mode"
#define SNPINI_PULSE_FROM_CAMERA_MODE_NAME       "Pulse From Camera Mode"
#define SNPINI_PULSE_WIDTH_NAME                  "Pulse Width"
#define SNPINI_WATERFALL_MODE_NAME               "WaterFall Mode"
#define SNPINI_LINES_PER_BANK_NAME               "Lines Per Bank"

/* Data stream specific */
#define SNPINI_DATA_ACQUISITION_MODE_NAME        "Data Acquisition Mode"
#define SNPINI_START_CAPTURE_MODE_NAME           "Start Capture Mode"

/* Comms - specific */
#define SNPINI_COMMS_STANDARD_NAME               "Comm Standard"
#define SNPINI_DATA_BITS_MODE_NAME               "Data Bits Mode"
#define SNPINI_STOP_BITS_MODE_NAME               "Stop Bits Mode"
#define SNPINI_PARITY_MODE_NAME                  "Parity Mode"
#define SNPINI_SPEED_MODE_NAME                   "Speed Mode"
#define SNPINI_COMM_XON_XOFF_NAME                "XON\\XOFF" /* double escape the slash */

/* These are not supported but still need to be recognised */
#define SNPINI_INITIAL_FILE_NAME                 "Initial File"
#define SNPINI_INITIAL_FILE PATH_NAME            "Initial File Path"
#define SNPINI_PRESNAP_FILE_NAME                 "Presnap File"
#define SNPINI_PRESNAP_FILE PATH_NAME            "Presnap File Path"
#define SNPINI_POSTSNAP_FILE_NAME                "Postsnap File"
#define SNPINI_POSTSNAP_FILE PATH_NAME           "Postsnap File Path"
#define SNPINI_PREGRAB_FILE_NAME                 "Pregrab File"
#define SNPINI_PREGRAB_FILE PATH_NAME            "Pregrab File Path"
#define SNPINI_POSTGRAB_FILE_NAME                "Postgrab File"
#define SNPINI_POSTGRAB_FILE PATH_NAME           "Postgrab File Path"

/* Mplus -specific - not supported as not custom camera but still need to be recognised */
#define SNPINI_MPLUS_I_EXPOSURE_TIME_NAME        "MPlus-i Exposure Time"
#define SNPINI_MPLUS_I_GAIN_NAME                 "MPlus-i Gain"
#define SNPINI_MPLUS_I_BLACK_LEVEL_NAME          "MPlus-i Black Level"
#define SNPINI_MPLUS_I_DEFECT_CONCEAL_MODE_NAME  "MPlus-i Defect Conceal Mode"
#define SNPINI_MPLUS_CONTROL_MODE_NAME           "MPlus Control Mode"
#define SNPINI_MPLUS_EXPOSURE_TIME_NAME          "MPlus Exposure Time"
#define SNPINI_MPLUS_SHUTTER_MODE_NAME           "MPlus Shutter Mode"
#define SNPINI_MPLUS_FRAME_RESET_NAME            "MPlus Frame Reset Mode"
#define SNPINI_MPLUS_I_SHUTTER_MODE_NAME         "MPlus-i Shutter Mode"
#define SNPINI_MPLUS_I_STROBE_MODE_NAME          "MPlus-i Strobe Mode"

/* Structure used for storing output of parsed input <source> in a doubly-linked list.*/
#if !defined MAX_NAME_STRING_LENGTH
#define MAX_NAME_STRING_LENGTH 40
#endif

struct InitTag{
   char   szParam  [MAX_NAME_STRING_LENGTH];
   char   szArg    [MAX_NAME_STRING_LENGTH];
   struct InitTag* pstNextTag;
   struct InitTag* pstPrevTag;
   struct InitTag* pstRootTag;
   ui32   dwDone;
   };



/* MACRO DEFINITIONS not for error handling (in alphabetical order)
 * ----------------------------------------------------------------
 */
#if defined _lint
struct Tbase * EXPORT_FN BASE_Dummy_get_Pbase(char *, Thandle);
#define BASE_get_Pbase(s,Ps,Hs)   Ps = BASE_Dummy_get_Pbase(s,Hs);

/*#elif _MALLOC_DEBUG*/
#elif defined _MALLOC_DEBUG
#define BASE_get_Pbase(s,Ps,Hs)   {if (((Hs) < 1) || ((Hs) >= BASE_NUM_HANDLES))\
                                      ASL_proe(ASLERR_BAD_HANDLE,"Handle out of range",s)\
                                   else if ((Ps = Pbase_array[Hs]) == NULL)\
                                      ASL_proe(ASLERR_BAD_HANDLE,"Null pointer",s)\
                                  }
#else
#define BASE_get_Pbase(s,Ps,Hs)   {if ((Ps = Pbase_array[Hs]) == (volatile void *) NULL) ASL_proe(ASLERR_BAD_HANDLE,"",s)}
#endif


/* MACRO DEFINITIONS for error handling (in alphabetical order)
 * ------------------------------------------------------------
 * Note: BASE_err_ret and BASE_ret_on_err cannot be used on the line
 * after an `else' - {} are needed even on single line if-else statements.
 */
#ifdef _DOS16  /* real mode DOS - we lose some the comments to save space */
#define BASE_err_ret(Hs,e,d,v,s)    {ASL_err_process(s,BASE_make_ret(Hs,e,v),"");return(BASE_make_ret(Hs,e,v));}
#else
#define BASE_err_ret(Hs,e,d,v,s)    {ASL_err_process(s,BASE_make_ret(Hs,e,v),d);return(BASE_make_ret(Hs,e,v));}
#define BASE_err_process(Hs,e,d,v,s)    {ASL_err_process(s,BASE_make_ret(Hs,e,v),d);}
#endif

#define BASE_make_ret(Hs,e,v)       (( Pbase_array[Hs]->IDrev << 24) | (e) | ((Terr) v & (Terr) 0xffff))

#endif /* _base_h */
