/**************************************************************************
 *                ACTIVE SILICON LIMITED / ACTIVE IMAGING LIMITED         *
 *                                                                        *
 *  File name   : shared.h                                                *
 *  Function    : Definitions for structures shared between library and   *
 *              : drivers and between drivers.                            *
 *  Project     : Snappers                                                *
 *  Author      : Colin Pearce, Active Silicon Ltd                        *
 *  Systems     : ANSI C                                                  *
 *  Version     : 3.10                                                    *
 *  Release Date: 09-Jul-97                                               *
 *                                                                        *
 *  Copyright (c) 1993-96 Active Silicon, Active Imaging Ltd.             *
 **************************************************************************
 * Comments:
 * --------
 * This file is is used by the Snapper libraries and the various drivers
 * for different operating systems.  The structures defined here are used
 * by the driver and the Snapper libraries (above the base<os>.c layer).
 * See the ioctl.h (e.g. w95ioctl.h) file for structures to pass data 
 * between base_<os>.c and the driver.
 *
 * File History
 * ------------
 * 16-Oct-97, HCP, wSerial_RxBuffer made volatile.
 *
 */

#ifndef _shared_h
#define _shared_h

#if defined _KERNEL || defined _MACOSX_KERNEL
#if !defined MODULE_INTERRUPT_SET
#define MODULE_INTERRUPT_SET ((ui8)24)
#endif
#endif

#if defined _MACINTOSH_KERNEL || defined _MACOSX_KERNEL || defined _LINUX_KERNEL
#if !defined RBT_PREAMBLE_LENGTH
#define RBT_PREAMBLE_LENGTH 4
#endif
#if !defined MODULE_INTERRUPT_SET
#define MODULE_INTERRUPT_SET ((ui8)24)
#endif
#endif


/*
 * #define codes used in the driver and the BASE library.
 * Note the casting is to ui32 and not Terr - generally a 
 * driver has its own specific (simple) return types.
 */
#define BASE_FPGA_STATUS_UNREAD   ((ui32) 2)
#define BASE_FPGA_CONFIGURED      ((ui32) 1)
#define BASE_FPGA_NOT_CONFIGURED  ((ui32) 0)

/*
 * Software mappings done in the driver (see also "struct Tbase")
 */
#define BASE_NO_MAPPING       0
#define BASE_RGBX32_TO_RGB24  1
#define BASE_RGB24_TO_RGBX32  2


/*
 * PCI EEPROM length for PCI-BIB ID 40
 */
#define BASE_PCI40_EEPROM_LENGTH   256
#define BASE_PCIMAX_EEPROM_LENGTH  256


/*
 * Library definitions for opening a Snapper device.  BASE_DEVICE is 
 * conveniently used by the driver to catch attempts at opening a 
 * board twice. (These definitions were moved from base.h.)
 */
#define BASE_NULL_ADDR            NULL
#define BASE_AUTO                 0       /* base address definition */
#define BASE_DEVICE               0x8000  /* library specific device number */

/*
 * Timeout types - used by library and driver - these generic types are 
 * generated from the Snapper library specific timeout types.  See 
 * base_<os>.c (OS_set_timeout) for more details.
 */
#define BASE_TIMEOUT_BEFORE  1
#define BASE_TIMEOUT_DURING  2

/*
 * Asynchronous read completion - options are either wait or poll for 
 * interrogation and complete or busy for a reply.
 */
#define BASE_POLL  0
#define BASE_WAIT  1

#define BASE_COMPLETE 0
#define BASE_BUSY     1
#define BASE_TIMEOUT  2
#define BASE_TARGET_ABORT 3
#define BASE_MASTER_ABORT 4

/*
 * FPGA Info Structure
 * -------------------
 * The members are all ui32s to reduce structure alignment hassles when 
 * interfacing to a driver.
 *
struct_Tfpga_info(ForBrief)
 */
struct Tfpga_info {
   ui32  fpga_type;         /* type ui32 for VxD alignment issues */
#define BASE_FPGA      0
#define MODULE_FPGA    1
#define MAPPER_FPGA    2
   ui8   FAR *fpga_data;
   ui32  fpga_bit_length;

   ui32  data_reg;
   ui32  data0_clock0;
   ui32  data0_clock1;
   ui32  data1_clock0;
   ui32  data1_clock1;

   ui32  reset_reg;
   ui32  reset_assert;
   ui32  reset_deassert;

   ui32  status_reg;
   ui32  dp_mask;

   ui32  ConfigAlgorithm;    /* pass information about how to configure */
#define BASE_USE_1_BIT   0
#define BASE_USE_32_BIT  1
};

/* In a "real" operating system, the following structure would exist in
 * the driver and libraries and OS_serial_send/get_info would be used
 * to pass information between the libraries and driver.  In DOS/Win31
 * only the library copy exists and is also used by the driver.
 * OS_serial_send/get_info is used by BASE_serial_set/get_parameter 
 * to pass this information between the libraries and the driver.
 * This structure is split into 2 parts. The first part is settable by
 * the libraries, and used by the driver. The second part is set by the
 * driver and read by the libraries.
 * They are all ui32s to reduce structure alignment hassles when interfacing
 * to a driver.
 *
struct TSerialInfo_Common(ForBrief)
 */
struct TSerialInfo_Common {       /* SIC */
   /* Library set, driver read: */
   ui32 serial_reset_comms;       /* TRUE tells the driver to reset its comms */
   volatile ui32 serial_xon_xoff_enable;   /* TRUE for XON/XOFF protocol, FALSE for none */
   volatile ui32 serial_xon_char;
   volatile ui32 serial_xoff_char;
   volatile ui32 serial_parity_ignore;

   /* Driver set, library read: */
   volatile ui32 serial_xon_xoff_status;   /* TRUE if XON received, FALSE for XOFF or n/a */
   volatile ui32 serial_rx_count;
   volatile ui32 parity_error_count;
   volatile ui32 overflow_error_count;
};

/*
struct_Ttable_in(ForBrief)
*/
struct Ttable_info
{
   ui32 hw_addr_offset;  /* address of table relative to base address */
   ui32 count;           /* number of elements */
   ui32 *Ptable_ui32;    /* actual table data - for 32 bit table */
   ui16 *Ptable_ui16;    /* actual table data - for 16 bit table */
   ui8  *Ptable_ui8;     /* actual table data - for  8 bit table */
   ui32 mode;
#define ASL_ADDR_INC    ((ui32) 0x01)
#define ASL_ADDR_FIXED  ((ui32) 0x02)
   ui32 ConfigAlgorithm;    /* pass information about how to configure */
#define BASE_USE_8_BIT_TABLE   0
#define BASE_USE_32_BIT_TABLE  1
};


/*
 * Driver structure for serial comms (in shared.h as it could be used
 * by DOS/Windows 3.1).
 *
struct TSerialInfo_Driver(ForBrief)
 */
struct TSerialInfo_Driver    /* SID */
{
#ifndef BASE_COMMS_BUFFER_SIZE
#define BASE_COMMS_BUFFER_SIZE  1024
#endif
   /* Transmit related */

   volatile ui32 dwTxCharCountToGo;
   volatile ui8  Serial_TxBuffer[BASE_COMMS_BUFFER_SIZE];
   volatile ui8 *pTxBuffer;
   volatile ui32 dwSerial_ErrorsInBuffer;

   volatile ui8 bSerialTxBlocked;  /* Added by martin */
   volatile ui8 bSerialTxActive;   /* Added by martin */

   /* Receive related */

   volatile ui16 wSerial_RxBuffer[BASE_COMMS_BUFFER_SIZE];
   ui32 dwSerial_RxReadIndex;
   ui32 dwSerial_RxWriteIndex;
};

/*
 * A structure for sending the interrupt information to the driver and also
 * for use by the driver to store the interrupt information.
 *
struct_TInterruptInfo(ForBrief)
 */
struct TInterruptInfo {
   ui32 dwIntControlAddr;  /* Address of interrupt control register */
   ui32 dwIntStatusAddr;   /* Address of interrupt status register  */
   ui32 dwData;            /* Data to write to interrupt control register */
};

/*
 * A structure to pass port address offset and data for simple driver
 * writes.
 *
union_URegData(ForBrief)
struct_TWriteBuffer(ForBrief)
 */
union URegData {               /* Data to be written               */
   ui32  dwData;
   ui16  wData;
   ui8   bData;
};

struct TWriteBuffer {
   ui32  dwAddrOffset;   /* Address offset from base address */
   union URegData uRegData;
};

/*
 * A structure to pass port address offset and byte count of amount of 
 * data to transfer etc.
 *
struct_TBurstTransfer(ForBrief)
 */
struct TBurstTransfer {
   ui32 dwAddrOffset;  /* port address offset         */
   ui32 dwByteCount;   /* number of bytes to transfer */
   ui32 dwMapping;     /* mapping required            */
   void *pData;        /* For outgoing data           */
};

/*
 * A structure to pass port address offset and byte count of amount of 
 * data to transfer etc.
 *
struct_TPhysMemLock(ForBrief)
 */
struct TPhysMemLock {
   void *pData;        /* For outgoing data           */
   ui32 *pSGTable;
   ui32 dwByteCount;   /* number of bytes to transfer */
};

/*
 * A structure to pass the information for a paged write.
 *
struct_TPageWrite(ForBrief)
 */
struct TPageWrite {
   ui32 dwPageRegAddr;
   ui32 dwPageRegData;
   ui32 dwPageOffset;
   ui32 dwData;
};

/*
 * A structure to pass the information about board type/address to and 
 * from the driver.
 * Note we make all structure elements 32 bit to eliminate potential 
 * problems with structure data alignment between library and drivers.
 *
TBoardInfo(ForBrief)
 */
struct TBoardInfo {

   /* Passed into driver: */

   ui32 dwBaseAddress;     /* 0, 300 or 8001 etc */
   ui32 dwBoardType;       /* ISA or PCI         */
   ui32 dwLibDeviceNum;    /* if base address = 8001, then this = 1 */

   /* Returned from driver: */

   /* Note: dwBaseAddress is returned from NT driver */
   ui32 dwInterruptLine;   /* Interrupt line number (can be interesting) */
   ui32 dwIDrev;           /* Baseboard IDrev      */
   ui32 *pdwEeprom;        /* To shadow PCI EEPROM - mallocs     */
                           /* BASE_PCIMAX_EEPROM_LENGTH*4 bytes  */
                           /* Used by Windows 95 */
   ui32 dwDriverRevLevel;
   ui32 dwEeprom[BASE_PCIMAX_EEPROM_LENGTH]; /* NT driver uses this - the "proper" way */
};

/*
 * A structure to pass timeout information to the driver.
 *
struct_TTimeout(ForBrief)
 */
struct TTimeout {
   ui32 dwTimeout;    /* timeout in milliseconds */
   ui32 dwType;       /* type of timeout - see shared.h */
};

/*
 * A structure for sending/receiving a buffer of characters to/from
 * the driver.
 *
struct_TSerialBuffer(ForBrief)
 */
struct TSerialBuffer {
   ui8  *pBuffer;
   ui32 *pdwCount;
   ui32 dwTimeoutTx;
   ui32 dwTimeoutRx;
};

/*
 * A structure to pass back information about what event/interrupt occurred
 * in the driver.
 *
struct_TCallbackEventInfo(Brief)
 */
struct TCallbackEventInfo {
   volatile ui32 dwIntSrcFromDriver;  /* Lower 16 bits for HW interrupts, upper for SW interrupts */
   volatile ui32 dwTransferSizeBytes; /* Size of DMA just completed if appropriate */
};

#define DRV_MAX_ERROR_STRING_LENGTH 512  /* NOTE: Same as MAX_ERROR_STRING_LENGTH in asl_gen.h!!! */

/*
 * A structure to read back an error code and string from the driver.
 *
struct_TErrorInfo(ForBrief)
 */
struct TErrorInfo {
   ui32 dwErrorCode;
   char szErrorString[DRV_MAX_ERROR_STRING_LENGTH];
};


/*
 * Scatter/Gather table information/macros for locking and freeing physical
 * memory. Note the Area For Transfer (AFT) entries - this allows a sub-area
 * within the total scatter/gather table to be handled. This is useful, in 
 * fact required, for Snapper-DIG16 main and flush DMAs direct to a physical 
 * device.
 * 
 * The SG table is of the form (all entries are ui32s):
 *
 *   pSGTable-> Allocated size of table (in bytes),
 *              Number of Entries,
 *              Total amount of memory represented by table (in bytes),
 *              Offset (in bytes - virtual) to Area For Transfer (AFT),
 *              Transfer Size of AFT.
 *              Address,
 *              Length,
 *              ...
 *
 * x = Pointer to SG table, y = entry number, z = value to set.
 * 
 * All sizes are in bytes.
 *
Scatter_Gather_Macros(ForBrief)
 */
#define SGTABLE_HEADER_SIZE  ((ui32) (5*4))  /* 32 bit words */
#define SGTABLE_BYTES_PER_ENTRY (2 * sizeof(ui32))

#define _SGTable_Get_AllocatedSize(x)     (*(((ui32*)(x))+0))
#define _SGTable_Get_NumEntries(x)        (*(((ui32*)(x))+1))
#define _SGTable_Get_MemorySize(x)        (*(((ui32*)(x))+2))
#define _SGTable_Get_StartOffset(x)       (*(((ui32*)(x))+3))
#define _SGTable_Get_TransferSize(x)      (*(((ui32*)(x))+4))

#define _SGTable_Get_PhysAddr(x,y)        (*(((ui32*)(x))+5+((y)<<1)))
#define _SGTable_Get_PhysLen(x,y)         (*(((ui32*)(x))+6+((y)<<1)))

#define _SGTable_Set_AllocatedSize(x,z)   (*(((ui32*)(x))+0) = z)
#define _SGTable_Set_NumEntries(x,z)      (*(((ui32*)(x))+1) = z)
#define _SGTable_Set_MemorySize(x,z)      (*(((ui32*)(x))+2) = z)
#define _SGTable_Set_StartOffset(x,z)     (*(((ui32*)(x))+3) = z)
#define _SGTable_Set_TransferSize(x,z)    (*(((ui32*)(x))+4) = z)

#define _SGTable_Set_PhysAddr(x,y,z)      (*(((ui32*)(x))+5+((y)<<1)) = z)
#define _SGTable_Set_PhysLen(x,y,z)       (*(((ui32*)(x))+6+((y)<<1)) = z)


/*
 * Dylan time - while waiting for busy to clear.
 */
#define DRV_DYLAN_TIMEOUT 500   /* in milliseconds. */

/*
 * Interrupt thread signalling bit. Passed from driver to base OS layer
 * to indicate that the interrupt thread should be aborted.
 */
#define BASE_INT_BIT_EXIT ((ui32) 0x80000000)


#endif /* _shared_h */

