/**************************************************************************
 *                ACTIVE SILICON LIMITED / ACTIVE IMAGING LIMITED         *
 *                                                                        *
 *  File name   : snp24.h                                                 *
 *  Function    : Header for SNP-24 and SNP-8 - #defines and types        *
 *  Project     : Snappers - DC09                                         *
 *  Author      : Chris Beynon, Martin Bone, Active Silicon Ltd           *
 *  Systems     : ANSI C                                                  *
 *  Written     : June 1994                                               *
 *  Updated     : 15-Jan-96                                               *
 *  Version     : 3.10                                                    *
 *                                                                        *
 *  Copyright (c) 1994-96 Active Silicon, Active Imaging Ltd.             *
 **************************************************************************
 *  Comments:
 */

#ifndef _SNP24_H_
#define _SNP24_H_

#ifndef _MALLOC_DEBUG
#define SNP24_COMPATIBILITY  /* defined for release */
#endif


/* TYPE DEFINITIONS
 * ----------------
 */


/* ARRAY SIZES, LIMITS ETC
 * -----------------------
 */
#define SNP24_NUM_HANDLES       16
#define SNP24_NUM_PAGES         16
#define SNP24_NUM_LCA_REGS      (8*SNP24_NUM_PAGES)
#define SNP24_NUM_ICS1522_REGS  7
#define SNP24_NUM_LM1882_REGS   19

#define SNP24_MIN_ROI_LENGTH       (i16) 8
#define SNP24_MAX_X_LENGTH         1024
#define SNP24_MAX_Y_LENGTH         1024
#define SNP24_MAX_LINESCAN_LENGTH  8096

/* Physical size 262144, but need to leave free space at end of bank */
#define SNP24_PIXELS_PER_BANK      262000



/* ERROR CODES
 * -----------
 * The error return code consists of 4 bytes.  The lower 2 contain any
 * parameter information from the function.  The next byte contains an
 * error code, and the highest byte contains the ID of the module
 * signalling the error.  This technique allows the software to
 * return an error although the original call may have been made
 * indirectly through another library call.
 */

#define SNP24_ID             ((ui8) 0xB0)      /* SNP24 HW ID */
#define SNP8_ID              ((ui8) 0xB8)      /* SNP8 HW ID  */
#define SNP24_PCI_ID         ((ui8) 0x90)      /* SNP PCI24 HW ID */
#define SNP8_PCI_ID          ((ui8) 0x98)      /* SNP PCI8  HW ID */
#define SNP24_PMC_ID         ((ui8) 0xE0)      /* PMC SNP24 HW ID */
#define SNP8_PMC_ID          ((ui8) 0xE8)      /* PMC SNP8  HW ID  */
#define SNP24_FAMILY_ID      SNP24_ID
#define SNP8_FAMILY_ID       SNP8_ID

#define PMC_SNP24_ID         SNP24_PMC_ID      /* For Compatibility */
#define PMC_SNP8_ID          SNP8_PMC_ID       /* For Compatibility */
#define SNP_PMC24_ID         SNP24_PMC_ID      /* For Compatibility */
#define SNP_PMC8_ID          SNP8_PMC_ID       /* For Compatibility */
#define SNP_PCI24_ID         SNP24_PCI_ID      /* For Compatibility */
#define SNP_PCI8_ID          SNP8_PCI_ID       /* For Compatibility */



/* REGISTER ADDRESSES
 * ------------------
 */
#define SNP24_PAGE_NONE      0x00
#define SNP24_PAGE_GEN       0x00
#define SNP24_PAGE_ROI       0x01
#define SNP24_PAGE_SYNC      0x02
#define SNP24_PAGE_GEN2      0x03
#define SNP24_PAGE_1882      0x09
#define SNP24_PAGE_252R      0x0C
#define SNP24_PAGE_252G      0x0D
#define SNP24_PAGE_252B      0x0E
#define SNP24_PAGE_252A      0x0F

#define SNP24_REG_BANK       ((SNP24_PAGE_NONE << 3) | 0x00)
#define SNP24_REG_ID         ((SNP24_PAGE_NONE << 3) | 0x00)
#define SNP24_REG_CAPTURE    ((SNP24_PAGE_NONE << 3) | 0x01)
#define SNP24_REG_STATUS     ((SNP24_PAGE_NONE << 3) | 0x01)
#define SNP24_REG_INT_CTRL   ((SNP24_PAGE_NONE << 3) | 0x02)
#define SNP24_REG_INT_STAT   ((SNP24_PAGE_NONE << 3) | 0x02)
#define SNP24_REG_PAGE       ((SNP24_PAGE_NONE << 3) | 0x03)
#define SNP24_REG_IO_STAT    ((SNP24_PAGE_NONE << 3) | 0x03)

#define SNP24_REG_READ_RST   ((SNP24_PAGE_GEN  << 3) | 0x04)
#define SNP24_REG_TRIG_CTRL  ((SNP24_PAGE_GEN  << 3) | 0x05)
#define SNP24_REG_TTL_422    ((SNP24_PAGE_GEN  << 3) | 0x06)
#define SNP24_REG_GENERAL    ((SNP24_PAGE_GEN  << 3) | 0x07)

#define SNP24_REG_AROI_X_START    ((SNP24_PAGE_ROI  << 3) | 0x04)
#define SNP24_REG_AROI_Y_START    ((SNP24_PAGE_ROI  << 3) | 0x05)
#define SNP24_REG_AROI_X_LENGTH   ((SNP24_PAGE_ROI  << 3) | 0x06)
#define SNP24_REG_AROI_Y_LENGTH   ((SNP24_PAGE_ROI  << 3) | 0x07)

#define SNP24_REG_SYNC_SRC   ((SNP24_PAGE_SYNC << 3) | 0x04)
#define SNP24_REG_SYNC_DIR   ((SNP24_PAGE_SYNC << 3) | 0x05)
#define SNP24_REG_SYNC_POL   ((SNP24_PAGE_SYNC << 3) | 0x06)
#define SNP24_REG_1522       ((SNP24_PAGE_SYNC << 3) | 0x07)

#define SNP24_REG_1882_ADDR  ((SNP24_PAGE_1882 << 3) | 0x04)
#define SNP24_REG_1882_DATAL ((SNP24_PAGE_1882 << 3) | 0x05)
#define SNP24_REG_1882_ADDRI ((SNP24_PAGE_1882 << 3) | 0x06)
#define SNP24_REG_1882_DATAH ((SNP24_PAGE_1882 << 3) | 0x07)

#define SNP24_REG_CLMP_START ((SNP24_PAGE_GEN2 << 3) | 0x04)
#define SNP24_REG_MONO_CTRL  ((SNP24_PAGE_GEN2 << 3) | 0x05)

#define SNP24_REG_252_ADDR   0x04
#define SNP24_REG_252_LUT    0x05
#define SNP24_REG_252_GEN    0x06


/* Linescan registers
 * ------------------
 */
#define SNP24_REG_LROI_START_L   ((SNP24_PAGE_ROI  << 3) | 0x04)
#define SNP24_REG_LROI_START_H   ((SNP24_PAGE_ROI  << 3) | 0x05)
#define SNP24_REG_LROI_LENGTH_L  ((SNP24_PAGE_ROI  << 3) | 0x06)
#define SNP24_REG_LROI_LENGTH_H  ((SNP24_PAGE_ROI  << 3) | 0x07)

#define SNP24_REG_LINESCAN_CTRL  ((SNP24_PAGE_SYNC << 3) | 0x05)

#define SNP24_REG_LINE_START_OUT ((SNP24_PAGE_GEN2 << 3) | 0x06)
#define SNP24_REG_LINES_PER_BANK ((SNP24_PAGE_GEN2 << 3) | 0x07)


/* SNP24_REGISTER_BITS
 * ===================
 *
 * Bank Register
 * -------------
 */
#define SNP24_HMASK_READ_BANK_SELECT  SNP24_BIT_READ_BANK_B
#define SNP24_BIT_READ_BANK_A         ((ui8) 0x00)
#define SNP24_BIT_READ_BANK_B         ((ui8) 0x01)

#define SNP24_HMASK_BANK_SW_HW        SNP24_BIT_BANK_HW
#define SNP24_BIT_BANK_SW             ((ui8) 0x00)
#define SNP24_BIT_BANK_HW             ((ui8) 0x02)

#define SNP24_HMASK_BANK_TOGGLE       SNP24_BIT_VBANK_TOGGLE
#define SNP24_BIT_HBANK_TOGGLE        ((ui8) 0x00)
#define SNP24_BIT_VBANK_TOGGLE        ((ui8) 0x04)


/* Capture Register
 * ----------------
 */
#define SNP24_BIT_CAPTURE           ((ui8) 0x01)

#define SNP24_HMASK_SEQUENCE_SINGLE SNP24_BIT_SEQUENCE_CAPTURE
#define SNP24_BIT_SINGLE_CAPTURE    ((ui8) 0x00)
#define SNP24_BIT_SEQUENCE_CAPTURE  ((ui8) 0x02)

#define SNP24_HMASK_FIELD_FRAME     SNP24_BIT_FIELD_MODE
#define SNP24_BIT_FRAME_MODE        ((ui8) 0x00)
#define SNP24_BIT_FIELD_MODE        ((ui8) 0x04)

#define SNP24_HMASK_WHICH_FIELD     SNP24_BIT_1ST_FIELD
#define SNP24_BIT_NEXT_FIELD        ((ui8) 0x00)
#define SNP24_BIT_1ST_FIELD         ((ui8) 0x08)

#define SNP24_HMASK_SUBSAMPLE_MODE  ((ui8) 0x60)
#define SNP24_BITS_SUB_X1           ((ui8) 0x00)
#define SNP24_BITS_SUB_X2           ((ui8) 0x20)
#define SNP24_BITS_SUB_X4           ((ui8) 0x40)
#define SNP24_BITS_SUB_X1_FIELD     ((ui8) 0x60)
#define SNP24_BITS_SUB_X8           ((ui8) 0x60)   /* Linescan */

#define SNP24_HMASK_TRIG_ENABLE     SNP24_BIT_TRIG_ENABLE
#define SNP24_BIT_TRIG_DISABLE      ((ui8) 0x00)
#define SNP24_BIT_TRIG_ENABLE       ((ui8) 0x80)


/* Interrupt Control & Status Register
 * -----------------------------------
 * Note: Any bits in the lower 16 bits represent hardware generated 
 * interrupts. Any bits in the upper 16 bits represent virtual interrupts
 * generated by the driver which appear like hardware interrupts to the user
 * but in fact are software generated interrupts.
 * Note: 0x8000000 as a software interrupt is reserved for abort/closing of 
 * interrupt thread (it is used in base_<os>.c and the driver).
 */
#define SNP24_HMASK_INT_VSYNC       SNP24_BIT_INT_VSYNC_EN
#define SNP24_BIT_INT_VSYNC_DIS     ((ui32) 0x0000)
#define SNP24_BIT_INT_VSYNC_EN      ((ui32) 0x0001)

#define SNP24_HMASK_INT_DATA_RDY    SNP24_BIT_INT_DATA_RDY_EN
#define SNP24_BIT_INT_DATA_RDY_DIS  ((ui32) 0x0000)
#define SNP24_BIT_INT_DATA_RDY_EN   ((ui32) 0x0002)

/* "Software" Interrupts: */

#define SNP24_HMASK_INT_TRANSFER   SNP24_BIT_INT_TRANSFER_EN
#define SNP24_BIT_INT_TRANSFER_DIS ((ui32) 0x00000000)
#define SNP24_BIT_INT_TRANSFER_EN  ((ui32) 0x00010000)

#define SNP24_HMASK_INT_RESERVED  SNP24_BIT_INT_RESERVED_EN
#define SNP24_BIT_INT_RESERVED_DIS  ((ui32) 0x00000000)
#define SNP24_BIT_INT_RESERVED_EN   ((ui32) 0x80000000)

#define SNP24_HMASK_HW_INTERRUPTS  ((ui32) 0x0000ffff)
#define SNP24_HMASK_SW_INTERRUPTS  ((ui32) 0xffff0000)

#define SNP24_HMASK_INTERRUPTS_IN_USE \
                               (SNP24_HMASK_INT_VSYNC    | \
                                SNP24_HMASK_INT_DATA_RDY)


/* Read Reset Register
 * -------------------
 */
#define SNP24_BIT_READ_RST          ((ui8) 0x01)


/* Trigger Control Register
 * ------------------------
 */
#define SNP24_HMASK_TRIG_OUT_IN     SNP24_BIT_TRIG_OUT
#define SNP24_BIT_TRIG_IN           ((ui8) 0x00)
#define SNP24_BIT_TRIG_OUT          ((ui8) 0x01)

#define SNP24_HMASK_TRIG_IN_POS_NEG SNP24_BIT_TRIG_IN_POS
#define SNP24_BIT_TRIG_IN_NEG       ((ui8) 0x00)
#define SNP24_BIT_TRIG_IN_POS       ((ui8) 0x02)

#define SNP24_HMASK_TRIG_DATA_OUT   SNP24_BIT_TRIG_DATA_OUT_1
#define SNP24_BIT_TRIG_DATA_OUT_0   ((ui8) 0x00)
#define SNP24_BIT_TRIG_DATA_OUT_1   ((ui8) 0x04)

#define SNP24_HMASK_TRIG_TIMER_EN   SNP24_BIT_TRIG_TIMER_EN
#define SNP24_BIT_TRIG_TIMER_DIS    ((ui8) 0x00)
#define SNP24_BIT_TRIG_TIMER_EN     ((ui8) 0x08)

#define SNP24_HMASK_TRIG_TIMER_POL  SNP24_BIT_TRIG_TIMER_POS
#define SNP24_BIT_TRIG_TIMER_NEG    ((ui8) 0x00)
#define SNP24_BIT_TRIG_TIMER_POS    ((ui8) 0x10)


/* TTL/422 Register
 * ----------------
 */
#define SNP24_HMASK_HSYNC_TTL_422   SNP24_BIT_HSYNC_TTL
#define SNP24_BIT_HSYNC_422         ((ui8) 0x00)
#define SNP24_BIT_HSYNC_TTL         ((ui8) 0x01)

#define SNP24_HMASK_VSYNC_TTL_422   SNP24_BIT_VSYNC_TTL
#define SNP24_BIT_VSYNC_422         ((ui8) 0x00)
#define SNP24_BIT_VSYNC_TTL         ((ui8) 0x02)

#define SNP24_HMASK_TRIG_TTL_422    SNP24_BIT_TRIG_TTL
#define SNP24_BIT_TRIG_422          ((ui8) 0x00)
#define SNP24_BIT_TRIG_TTL          ((ui8) 0x04)

#define SNP24_HMASK_PCLK_TTL_422    SNP24_BIT_PCLK_TTL
#define SNP24_BIT_PCLK_422          ((ui8) 0x00)
#define SNP24_BIT_PCLK_TTL          ((ui8) 0x08)


/* General Register
 * ----------------
 */
#define SNP24_HMASK_1882_RST        SNP24_BIT_1882_RUN
#define SNP24_BIT_1882_RST          ((ui8) 0x00)
#define SNP24_BIT_1882_RUN          ((ui8) 0x01)

#define SNP24_HMASK_TRIGSEL         ((ui8) 0x06)
#define SNP24_BITS_TRIGSEL_TRIG1    ((ui8) 0x00)
#define SNP24_BITS_TRIGSEL_TRIG2    ((ui8) 0x02)
#define SNP24_BITS_TRIGSEL_TRIG3    ((ui8) 0x04)
#define SNP24_BITS_TRIGSEL_TRIG4    ((ui8) 0x06)

#define SNP24_HMASK_CTRLOUT_EN      SNP24_BIT_CTRLOUT_EN
#define SNP24_BIT_CTRLOUT_DIS       ((ui8) 0x00)
#define SNP24_BIT_CTRLOUT_EN        ((ui8) 0x08)

#define SNP24_HMASK_CTRLOUT         ((ui8) 0x30)

#define SNP24_MAX_CTRLOUT           ((ui8) 0x03)
#define SNP24_SHIFT_CTRLOUT         4

#define SNP24_HMASK_Y8_ON_RED_EN    SNP24_BIT_Y8_ON_RED_EN
#define SNP24_BIT_Y8_ON_RED_DIS     ((ui8) 0x00)
#define SNP24_BIT_Y8_ON_RED_EN      ((ui8) 0x80)


/* Sync Source Register
 * --------------------
 */
#define SNP24_HMASK_CLK_EXT_PLL     SNP24_BIT_CLK_EXT
#define SNP24_BIT_CLK_PLL           ((ui8) 0x00)
#define SNP24_BIT_CLK_EXT           ((ui8) 0x01)

#define SNP24_HMASK_SYNC_INT_EXT    SNP24_BIT_SYNC_INT
#define SNP24_BIT_SYNC_EXT          ((ui8) 0x00)
#define SNP24_BIT_SYNC_INT          ((ui8) 0x02)

#define SNP24_HMASK_SYNC_EXT_SRC    ((ui8) 0x1C)
#define SNP24_BITS_SYNC_EXT_RED     ((ui8) 0x00)
#define SNP24_BITS_SYNC_EXT_GRN     ((ui8) 0x04)
#define SNP24_BITS_SYNC_EXT_BLU     ((ui8) 0x08)
#define SNP24_BITS_SYNC_EXT_CSYNC   ((ui8) 0x0C)
#define SNP24_BITS_SYNC_EXT_HVSYNC  ((ui8) 0x10)
#define SNP24_BITS_SYNC_EXT_C_ON_H  ((ui8) 0x14)

#define SNP24_HMASK_CCIR_EIA        SNP24_BIT_CCIR
#define SNP24_BIT_EIA               ((ui8) 0x00)
#define SNP24_BIT_CCIR              ((ui8) 0x20)

#define SNP24_HMASK_CLK_PHASE       ((ui8) 0xC0)
#define SNP24_BITS_CLK_PHASE0       ((ui8) 0x00)
#define SNP24_BITS_CLK_PHASE90      ((ui8) 0x40)
#define SNP24_BITS_CLK_PHASE180     ((ui8) 0x80)
#define SNP24_BITS_CLK_PHASE270     ((ui8) 0xC0)


/* Sync Direction Register
 * -----------------------
 */
#define SNP24_HMASK_HSYNC_OUT_IN    SNP24_BIT_HSYNC_OUT
#define SNP24_BIT_HSYNC_IN          ((ui8) 0x00)
#define SNP24_BIT_HSYNC_OUT         ((ui8) 0x01)

#define SNP24_HMASK_VSYNC_OUT_IN    SNP24_BIT_VSYNC_OUT
#define SNP24_BIT_VSYNC_IN          ((ui8) 0x00)
#define SNP24_BIT_VSYNC_OUT         ((ui8) 0x02)

#define SNP24_HMASK_CSYNC_OUT_IN    SNP24_BIT_CSYNC_OUT
#define SNP24_BIT_CSYNC_IN          ((ui8) 0x00)
#define SNP24_BIT_CSYNC_OUT         ((ui8) 0x04)

#define SNP24_HMASK_PCLK_OUT_IN     SNP24_BIT_PCLK_OUT
#define SNP24_BIT_PCLK_IN           ((ui8) 0x00)
#define SNP24_BIT_PCLK_OUT          ((ui8) 0x08)

#define SNP24_HMASK_SYNC_C_ON_H_OUT SNP24_BIT_SYNC_C_ON_H_OUT
#define SNP24_BIT_SYNC_H_ON_H_OUT   ((ui8) 0x00)
#define SNP24_BIT_SYNC_C_ON_H_OUT   ((ui8) 0x10)

#define SNP24_HMASK_SWAP_FIELDS     SNP24_BIT_FIELDS_SWAPPED
#define SNP24_BIT_FIELDS_STD        ((ui8) 0x00)
#define SNP24_BIT_FIELDS_SWAPPED    ((ui8) 0x20)


/* Line Scan Control Register
 * --------------------------
 */
#define SNP24_HMASK_LTRIG_POL       SNP24_BIT_LTRIG_POS
#define SNP24_BIT_LTRIG_NEG         ((ui8) 0x00)
#define SNP24_BIT_LTRIG_POS         ((ui8) 0x01)

#define SNP24_HMASK_LSTART_OUT_EN   SNP24_BIT_LSTART_OUT_EN
#define SNP24_BIT_LSTART_OUT_DIS    ((ui8) 0x00)
#define SNP24_BIT_LSTART_OUT_EN     ((ui8) 0x02)   /* Was 0x10, MBB 4-Aug-99 */

#define SNP24_HMASK_LTRIG_INT_SEL   SNP24_BIT_LTRIG_INT_HSYNC
#define SNP24_BIT_LTRIG_INT_TIMER   ((ui8) 0x00)
#define SNP24_BIT_LTRIG_INT_HSYNC   ((ui8) 0x04)

#define SNP24_HMASK_LTRIG_IN_SEL    SNP24_BIT_LTRIG_IN_EXT
#define SNP24_BIT_LTRIG_IN_INT      ((ui8) 0x00)
#define SNP24_BIT_LTRIG_IN_EXT      ((ui8) 0x10)   /* Was 0x08, MBB 10-Aug-99 */

#define SNP24_HMASK_LSTART_IN_SEL   SNP24_BIT_LSTART_IN_INT
#define SNP24_BIT_LSTART_IN_EXT     ((ui8) 0x00)
#define SNP24_BIT_LSTART_IN_INT     ((ui8) 0x20)

#define SNP24_HMASK_LINEACC_MODE    ((ui8) 0xC0)
#define SNP24_BITS_LINE_X1          ((ui8) 0x00)
#define SNP24_BITS_LINE_X2          ((ui8) 0x40)
#define SNP24_BITS_LINE_X4          ((ui8) 0x80)
#define SNP24_BITS_LINE_X8          ((ui8) 0xC0)


/* Sync Polarity Register
 * ----------------------
 */
#define SNP24_HMASK_HSYNC_IN_POL    SNP24_BIT_HSYNC_IN_POS
#define SNP24_BIT_HSYNC_IN_NEG      ((ui8) 0x00)
#define SNP24_BIT_HSYNC_IN_POS      ((ui8) 0x01)

#define SNP24_HMASK_LSTART_IN_POL   SNP24_BIT_LSTART_IN_POS
#define SNP24_BIT_LSTART_IN_NEG     ((ui8) 0x00)
#define SNP24_BIT_LSTART_IN_POS     ((ui8) 0x01)   /* Was 0x02 in LineScan Control Reg, MBB 4-Aug-99 */

#define SNP24_HMASK_VSYNC_IN_POL    SNP24_BIT_VSYNC_IN_POS
#define SNP24_BIT_VSYNC_IN_NEG      ((ui8) 0x00)
#define SNP24_BIT_VSYNC_IN_POS      ((ui8) 0x02)

#define SNP24_HMASK_CSYNC_IN_POL    SNP24_BIT_CSYNC_IN_POS
#define SNP24_BIT_CSYNC_IN_NEG      ((ui8) 0x00)
#define SNP24_BIT_CSYNC_IN_POS      ((ui8) 0x04)

#define SNP24_HMASK_PCLK_IN_POL     SNP24_BIT_PCLK_IN_POS
#define SNP24_BIT_PCLK_IN_NEG       ((ui8) 0x00)
#define SNP24_BIT_PCLK_IN_POS       ((ui8) 0x08)

#define SNP24_HMASK_HSYNC_OUT_POL   SNP24_BIT_HSYNC_OUT_POS
#define SNP24_BIT_HSYNC_OUT_NEG     ((ui8) 0x00)
#define SNP24_BIT_HSYNC_OUT_POS     ((ui8) 0x10)

#define SNP24_HMASK_LSTART_OUT_POL  SNP24_BIT_LSTART_OUT_POS
#define SNP24_BIT_LSTART_OUT_NEG    ((ui8) 0x00)
#define SNP24_BIT_LSTART_OUT_POS    ((ui8) 0x10)   /* Was 0x04 in LineScan Control Reg, MBB 4-Aug-99 */

#define SNP24_HMASK_VSYNC_OUT_POL   SNP24_BIT_VSYNC_OUT_POS
#define SNP24_BIT_VSYNC_OUT_NEG     ((ui8) 0x00)
#define SNP24_BIT_VSYNC_OUT_POS     ((ui8) 0x20)

#define SNP24_HMASK_CSYNC_OUT_POL   SNP24_BIT_CSYNC_OUT_POS
#define SNP24_BIT_CSYNC_OUT_NEG     ((ui8) 0x00)
#define SNP24_BIT_CSYNC_OUT_POS     ((ui8) 0x40)

#define SNP24_HMASK_PCLK_OUT_POL    SNP24_BIT_PCLK_OUT_POS
#define SNP24_BIT_PCLK_OUT_NEG      ((ui8) 0x00)
#define SNP24_BIT_PCLK_OUT_POS      ((ui8) 0x80)


/* ICS1522 Access Register
 * -----------------------
 */
#define SNP24_HMASK_SCLK            SNP24_BIT_SCLK
#define SNP24_BIT_SCLK              ((ui8) 0x01)

#define SNP24_HMASK_SELN            SNP24_BIT_SELN
#define SNP24_BIT_SELN              ((ui8) 0x02)

#define SNP24_HMASK_SDATA           SNP24_BIT_SDATA
#define SNP24_BIT_SDATA             ((ui8) 0x04)
#define SNP24_BITPOS_SDATA          ((ui8) 0x02)

#define SNP24_HMASK_SDATAWT         SNP24_BIT_SDATAWT
#define SNP24_BIT_SDATAWT           ((ui8) 0x08)


/* ICS1522 Registers
 * -----------------
 */
#define SNP24_ICS1522_HMASK_OP_SCALE   ((Tparam) 0x0030)
#define SNP24_ICS1522_BITS_OP_DIV1     ((Tparam) 0x0030)
#define SNP24_ICS1522_BITS_OP_DIV2     ((Tparam) 0x0020)
#define SNP24_ICS1522_BITS_OP_DIV4     ((Tparam) 0x0010)
#define SNP24_ICS1522_BITS_OP_DIV8     ((Tparam) 0x0000)

#define SNP24_ICS1522_HMASK_VCO_GAIN   ((ui16) 0x007)
#define SNP24_ICS1522_HMASK_PD_GAIN    ((ui16) 0x038)

#define SNP24_ICS1522_HMASK_FILTER     SNP24_ICS1522_BIT_FILTER_EXT
#define SNP24_ICS1522_BIT_FILTER_INT   ((ui16) 0x0000)
#define SNP24_ICS1522_BIT_FILTER_EXT   ((ui16) 0x0080)

#define SNP24_ICS1522_HMASK_AUXEN      SNP24_ICS1522_BIT_AUXEN
#define SNP24_ICS1522_BIT_AUXEN        ((ui16) 0x0100)

#define SNP24_ICS1522_HMASK_OMUX       ((ui16) 0x0078)
#define SNP24_ICS1522_BITS_OMUX1       ((ui16) 0x0008)
#define SNP24_ICS1522_BITS_OMUX2       ((ui16) 0x0018)
#define SNP24_ICS1522_BITS_OMUX3       ((ui16) 0x0028)
#define SNP24_ICS1522_BITS_OMUX4       ((ui16) 0x0048)

#define SNP24_ics1522_sdata(x)  Psnp24->LCA_reg[SNP24_REG_1522] = (ui8) ( (Psnp24->LCA_reg[SNP24_REG_1522] & ~SNP24_HMASK_SDATA) | ((x) << SNP24_BITPOS_SDATA) )
#define SNP24_ics1522_seln(x)   Psnp24->LCA_reg[SNP24_REG_1522] = (ui8) ( (Psnp24->LCA_reg[SNP24_REG_1522] & ~SNP24_HMASK_SELN) | (x) )
#define SNP24_ics1522_rw(x)     Psnp24->LCA_reg[SNP24_REG_1522] = (ui8) ( (Psnp24->LCA_reg[SNP24_REG_1522] & ~SNP24_HMASK_SDATAWT) | (x) )

#define SNP24_MAX_VCO_GAIN          ((ui16) 7)
#define SNP24_MAX_PD_GAIN           ((ui16) 7)
#define SNP24_MIN_PIX_PER_LINE      ((ui16) 64)
#define SNP24_MAX_PIX_PER_LINE      ((ui16) 2048)

/* Following frequencies in kHz */
/* #define SNP24_MIN_LSCAN_FREQ        ((ui16) 475) NOTE need to fix DIV4 problem before 475 allowed */
#define SNP24_MIN_LSCAN_FREQ        ((ui16) 2000)
#define SNP24_MAX_LSCAN_FREQ        ((ui16) 20000)


#define SNP24_SELN_ASSERT           0    
#define SNP24_SELN_DEASSERT         SNP24_HMASK_SELN

#define SNP24_SDATA_R               0    
#define SNP24_SDATA_W               SNP24_HMASK_SDATAWT

#define SNP24_ICS1522_REG0_CCIR     ((Tparam) 0x03AF)  /* 944 in decimal */
#define SNP24_ICS1522_REG0_EIA      ((Tparam) 0x030B)  /* 780 in decimal */  /* Was 30F */
#define SNP24_ICS1522_REG1_INIT     ((Tparam) 0x0003)
#define SNP24_ICS1522_REG2_INIT     ((Tparam) 0x0006)
#define SNP24_ICS1522_REG3_INIT     ((Tparam) 0x0000)
#define SNP24_ICS1522_REG4_FAST     ((Tparam) 0x07E5)
#define SNP24_ICS1522_REG4_SLOW     ((Tparam) 0x077D)
#define SNP24_ICS1522_REG5_INIT_ES  ((Tparam) 0x0561)
#define SNP24_ICS1522_REG5_INIT     ((Tparam) 0x05E1)
#define SNP24_ICS1522_REG6_INIT     ((Tparam) 0x04CF)


/* LM1882 Registers
 * ----------------
 */

/* Bt252 Registers
 * ---------------
 */
#define SNP24_BT252_BITS_LEV125     ((ui8) 0x0C)

#define SNP24_BT252_HMASK_SYNC_SEL  ((ui8) 0x30)
#define SNP24_BT252_BITS_SYNC_VID0  ((ui8) 0x00)
#define SNP24_BT252_BITS_SYNC_VID1  ((ui8) 0x10)
#define SNP24_BT252_BITS_SYNC_VID2  ((ui8) 0x20)
#define SNP24_BT252_BITS_SYNC_VID3  ((ui8) 0x30)

#define SNP24_BT252_HMASK_DIG_SEL   ((ui8) 0xC0)
#define SNP24_BT252_BITS_DIG_VID0   ((ui8) 0x00)
#define SNP24_BT252_BITS_DIG_VID1   ((ui8) 0x40)
#define SNP24_BT252_BITS_DIG_VID2   ((ui8) 0x80)
#define SNP24_BT252_BITS_DIG_VID3   ((ui8) 0xC0)


/* SNP24_REG_MONO_CTRL
 * -------------------
 */
#define SNP24_HMASK_CLAMP_RED_EN     SNP24_BIT_CLAMP_RED_DIS
#define SNP24_BIT_CLAMP_RED_EN       ((ui8) 0x00)
#define SNP24_BIT_CLAMP_RED_DIS      ((ui8) 0x01)

#define SNP24_HMASK_CLAMP_GRN_EN     SNP24_BIT_CLAMP_GRN_DIS
#define SNP24_BIT_CLAMP_GRN_EN       ((ui8) 0x00)
#define SNP24_BIT_CLAMP_GRN_DIS      ((ui8) 0x02)

#define SNP24_HMASK_CLAMP_BLU_EN     SNP24_BIT_CLAMP_BLU_DIS
#define SNP24_BIT_CLAMP_BLU_EN       ((ui8) 0x00)
#define SNP24_BIT_CLAMP_BLU_DIS      ((ui8) 0x04)

#define SNP24_HMASK_PRE_PACK_SEL     ((ui8) 0x18)
#define SNP24_BITS_PRE_PACK_NONE     ((ui8) 0x00)
#define SNP24_BITS_PRE_PACK_RED      ((ui8) 0x08)
#define SNP24_BITS_PRE_PACK_GRN      ((ui8) 0x10)
#define SNP24_BITS_PRE_PACK_BLU      ((ui8) 0x18)



/* Status Register
 * ---------------
 */
#define SNP24_STAT_CAPTURE          ((ui8) 0x01)
#define SNP24_STAT_TRIGGER          ((ui8) 0x02)
#define SNP24_STAT_PLL_LOCKED       ((ui8) 0x04)
#define SNP24_STAT_SEQ_DROPPED      ((ui8) 0x08)
#define SNP24_STAT_VSYNC            ((ui8) 0x10)
#define SNP24_STAT_FIELD1           ((ui8) 0x20)
#define SNP24_STAT_DATA_READY       ((ui8) 0x40)
#define SNP24_STAT_FIELD1_CAPTURED  ((ui8) 0x80)

#define SNP24_STAT_50HZ             ((ui8) 0x08)


/* Int Status Register
 * -------------------
 */
#define SNP24_STAT_VSYNC_INT        ((ui8) 0x01)
#define SNP24_STAT_DATA_RDY_INT     ((ui8) 0x02)


/* IO Status Register
 * ------------------
 */
#define SNP24_STAT_HSYNC_IP         ((ui8) 0x10)
#define SNP24_STAT_VSYNC_IP         ((ui8) 0x20)
#define SNP24_STAT_PCLK_IP          ((ui8) 0x40)
#define SNP24_STAT_TRIG_IP          ((ui8) 0x80)
#define SNP24_BITS_IO_STATUS        ((ui8) 0xF0)



/* SOFTWARE PARAMETER FLAGS
 * ========================
 */

/* Bt252s
 * ------
 */
#define SNP24_252_RED        (SNP24_PAGE_252R << 3)
#define SNP24_252_GRN        (SNP24_PAGE_252G << 3)
#define SNP24_252_BLU        (SNP24_PAGE_252B << 3)
#define SNP24_252_ALL        (SNP24_PAGE_252A << 3)


/* ICS1522 Rev
 * -----------
 */
#define SNP24_ICS1522_REV_ES     ((ui32) 1)
#define SNP24_ICS1522_REV_PROD   ((ui32) 2)



/* Pre Packing Modes Supported
 * ---------------------------
 */
#define SNP24_PRE_PACK_ON_RED    ((ui32) 0x0001L)
#define SNP24_PRE_PACK_ON_GRN    ((ui32) 0x0002L)
#define SNP24_PRE_PACK_ON_BLU    ((ui32) 0x0004L)


/* Video Format
 * ------------
 */
#define SNP24_VIDEO_FORMAT_RGB          ((ui16) 0x0001)
#define SNP24_VIDEO_FORMAT_MONO_ON_RED  ((ui16) 0x0002)
#define SNP24_VIDEO_FORMAT_MONO_ON_GRN  ((ui16) 0x0003)
#define SNP24_VIDEO_FORMAT_MONO_ON_BLU  ((ui16) 0x0004)


/*
 * SNP24_capture()
 * ---------------
 */
#define SNP24_START_AND_WAIT        ((Tparam) 0x0001)
#define SNP24_START_AND_RETURN      ((Tparam) 0x0002)
#define SNP24_ABORT_CAPTURE         ((Tparam) 0x0004)
#define SNP24_END_SEQUENCE          ((Tparam) 0x0008)
#define SNP24_START_AND_RETURN_TRIG ((Tparam) 0x0010)


/*
SNP24_configure_fpga()
----------------------
*/
#define SNP24_MODE_AREA_SCAN        ((ui16) 0x0001)
#define SNP24_MODE_LINE_SCAN        ((ui16) 0x0002)


/* 
SNP24_get_parameter()
---------------------
*/
#define SNP24_BASEBOARD_HANDLE      ((ui16) 0x0001)
#define SNP24_TIMEOUT_TRIGGER       ((ui16) 0x0002)
#define SNP24_NUMBER_OF_BITS        ((ui16) 0x0003)
#define SNP24_SUPPORTED_FORMAT      ((ui16) 0x0004)
#define SNP24_ICS1522_REV           ((ui16) 0x0005)
#define SNP24_MAPPER_INTERFACE_TYPE ((ui16) 0x0006)
#define SNP24_SUPPORTED_PRE_PACKING ((ui16) 0x0007)
#define SNP24_FPGA_STUB_NAME        ((ui16) 0x0008)
#define SNP24_ID_VALUE              ((ui16) 0x0009)
#define SNP24_IDREV_VALUE           ((ui16) 0x000A)
#define SNP24_FAMILY_VALUE          ((ui16) 0x000B)
#define SNP24_FAMILY                SNP24_FAMILY_VALUE /* For Compatibility */
#define SNP24_REV_VALUE             ((ui16) 0x000C)


/* 
SNP24_get_ptr()
---------------
*/
#define SNP24_PSNP24_STRUCT         ((ui16) 0x0001)


/*
SNP24_lm1882_prog()
-------------------
*/
#define SNP24_LM1882_INIT_EIA       ((Tparam) 0x00000001L)
#define SNP24_LM1882_INIT_CCIR      ((Tparam) 0x00000002L)
#define SNP24_LM1882_SET            ((Tparam) 0x00000004L)

#define SNP24_LM1882_INVALID_PARAM  ((Tparam) 0xFFFFFFF8L)


/*
SNP24_initialize()
------------------
*/
/* #define SNP24_EIA_DEFAULT         ((Tparam) 0x00000001L)  see SNP24_set_video_standard */
/* #define SNP24_CCIR_DEFAULT        ((Tparam) 0x00000002L)  see SNP24_set_video_standard */
/* #define SNP24_LINE_SCAN_MODE      ((Tparam) 0x00008000L)  see SNP24_set_video_standard */
#define SNP24_IPL3000                ((Tparam) 0x00008004L)
#define SNP24_CUSTOM_LINESCAN        ((Tparam) 0x80008000L)

#define SNP24_INIT_INVALID_PARAM     ((Tparam) 0x7FFF7FF8L)


/*
SNP24_reset_read_pointer()
--------------------------
*/
#define SNP24_SMASK_RD_RST_A        ((Tparam) 0x0100)
#define SNP24_SMASK_RD_RST_B        ((Tparam) 0x0200)
#define SNP24_SMASK_RD_RST_AUTO     ((Tparam) 0x0400)
#define SNP24_READ_RESET_A          ((Tparam) 0x0001 | SNP24_SMASK_RD_RST_A)
#define SNP24_READ_RESET_B          ((Tparam) 0x0002 | SNP24_SMASK_RD_RST_B)
#define SNP24_READ_RESET_AUTO       ((Tparam) 0x0004 | SNP24_SMASK_RD_RST_AUTO)


/*
SNP24_set_callback()
--------------------
*/
/* Modes: */
#define SNP24_CALLBACK_INIT          ((Tparam) 0x00000001L)
#define SNP24_CALLBACK_SET           ((Tparam) 0x00000002L)
#define SNP24_CALLBACK_INVALID_PARAM ((Tparam) 0xFFFFFFFCL)


/* 
SNP24_set_capture()
-------------------
*/
#define SNP24_CAPTURE_INIT           ((Tparam) 0x00000001L)

#define SNP24_SUB_MODE_MASK          ((Tparam) 0x0000403EL)
#define SNP24_SUB_X1                 ((Tparam) 0x00000002L)
#define SNP24_SUB_X2                 ((Tparam) 0x00000004L)
#define SNP24_SUB_X4                 ((Tparam) 0x00000008L)
#define SNP24_SUB_X1_SINGLE_FIELD    ((Tparam) 0x00000010L)
#define SNP24_SUB_X1_FIELD_DUPLICATE ((Tparam) 0x00000020L)
#define SNP24_SUB_X8                 ((Tparam) 0x00004000L) /* Linescan */

#define SNP24_START_1ST_FIELD        ((Tparam) 0x00000040L)
#define SNP24_START_NEXT_FIELD       ((Tparam) 0x00000080L)

#define SNP24_TRIG_IN_ENABLE         ((Tparam) 0x00000100L)
#define SNP24_TRIG_IN_DISABLE        ((Tparam) 0x00000200L)
#define SNP24_READOUT_DEINTERLACE    ((Tparam) 0x00000400L)
#define SNP24_READOUT_NO_DEINTERLACE ((Tparam) 0x00000800L)

#define SNP24_SINGLE_CAPTURE_MODE    ((Tparam) 0x00001000L)
#define SNP24_SEQUENCE_MODE          ((Tparam) 0x00002000L)

#define SNP24_CAPTURE_INVALID_PARAM  ((Tparam) 0xFFFF8000L)

#ifdef SNP24_COMPATIBILITY
/* Note: These #defines must match ones above */
#define SNP24_SUB_FAST               ((Tparam) 0x00000020L)  /* = FIELD_DUPLICATE  */
#define SNP24_NEXT_FRAME             ((Tparam) 0x00000040L)  /* = START_1ST_FIELD  */
#define SNP24_NEXT_1ST_FIELD         ((Tparam) 0x00000040L)  /* = START_1ST_FIELD  */
#define SNP24_NEXT_2_FIELDS          ((Tparam) 0x00000080L)  /* = START_NEXT_FIELD */
#define SNP24_NEXT_FIELD             ((Tparam) 0x00000080L)  /* = START_NEXT_FIELD */
#endif



/*
SNP24_set_clamp()
-----------------
*/
#define SNP24_CLAMP_SMASK_NUMERIC   ((Tparam) 0x00000FFFL)

#define SNP24_CLAMP_INIT            ((Tparam) 0x00001000L)
#define SNP24_CLAMP_START           ((Tparam) 0x00002000L)

#define SNP24_CLAMP_INVALID_PARAM   ((Tparam) 0xFFFFC000L)



/*
SNP24_set_clk()
---------------
*/
#define SNP24_CLK_SMASK_NUMERIC     ((Tparam) 0x00000FFFL)

#define SNP24_CLK_INIT              ((Tparam) 0x00001000L)
#define SNP24_PLL_PHASE_0           ((Tparam) 0x00002000L)
#define SNP24_PLL_PHASE_90          ((Tparam) 0x00006000L)
#define SNP24_PLL_PHASE_180         ((Tparam) 0x0000A000L)
#define SNP24_PLL_PHASE_270         ((Tparam) 0x0000E000L)
#define SNP24_PLL_PHASE_OPTION      ((Tparam) 0x00002000L)
#define SNP24_PLL_SLOW_LOCK         ((Tparam) 0x00010000L)
#define SNP24_PLL_FAST_LOCK         ((Tparam) 0x00020000L)
#define SNP24_PLL_STOP              ((Tparam) 0x00040000L)
#define SNP24_PLL_START             ((Tparam) 0x00080000L)
#define SNP24_PLL_VCO_GAIN          ((Tparam) 0x00100000L)
#define SNP24_PLL_PD_GAIN           ((Tparam) 0x00200000L)
#define SNP24_CLK_PLL               ((Tparam) 0x00800000L)
#define SNP24_CLK_EXT               ((Tparam) 0x01000000L)
#define SNP24_CLK_PCLK_IN           ((Tparam) 0x02000000L)
#define SNP24_CLK_PCLK_IN_POS       ((Tparam) 0x07000000L)
#define SNP24_CLK_PCLK_IN_NEG       ((Tparam) 0x0B000000L)
#define SNP24_CLK_PCLK_OUT          ((Tparam) 0x10000000L)
#define SNP24_CLK_PCLK_OUT_POS      ((Tparam) 0x14000000L)
#define SNP24_CLK_PCLK_OUT_NEG      ((Tparam) 0x18000000L)

#define SNP24_CLK_INVALID_PARAM     ((Tparam) 0xE0400000L)



/*
SNP24_set_ctrl_out()
--------------------
*/
#define SNP24_CTRLOUT_SMASK_NUMERIC ((Tparam) 0x000000FFL)

#define SNP24_CTRLOUT_INIT          ((Tparam) 0x00000100L)
#define SNP24_CTRLOUT_SYNC          ((Tparam) 0x00000200L)
#define SNP24_CTRLOUT_VIDSRC        ((Tparam) 0x00000400L)
#define SNP24_CTRLOUT_SET           ((Tparam) 0x00000800L)

#define SNP24_CTRLOUT_INVALID_PARAM ((Tparam) 0xFFFFF000L)


/*
SNP24_set_format()
------------------
*/
#define SNP24_FORMAT_INIT           ((Tparam) 0x00000001L)
#define SNP24_FORMAT_RGB            ((Tparam) 0x00000102L)
#define SNP24_FORMAT_Y8_ON_RED      ((Tparam) 0x00000004L)
#define SNP24_FORMAT_Y8_ON_GRN      ((Tparam) 0x00000108L)
#define SNP24_FORMAT_Y8_ON_BLU      ((Tparam) 0x00000110L)

#define SNP24_FORMAT_NOT_SNP8       ((Tparam) 0x00000100L)
#define SNP24_FORMAT_ONE_SEL_MASK   ((Tparam) 0x0000001FL)
#define SNP24_FORMAT_INVALID_PARAM  ((Tparam) 0xFFFFFEE0L)


/*
SNP24_set_interrupts()
----------------------
*/
/* Types of interrupt that can get installed */
#define SNP24_INT_INIT               ((Tparam) 0x00000001L)
#define SNP24_INT_TRANSFER_COMPLETE  ((Tparam) 0x00000002L)
#define SNP24_INT_VSYNC              ((Tparam) 0x00000004L)
#define SNP24_INT_DATA_READY         ((Tparam) 0x00000008L)
#define SNP24_INT_INVALID_PARAM      ((Tparam) 0xFFFFFFF0L)


/*
SNP24_set_levels()
------------------
*/
#define SNP24_LVL_INIT              ((Tparam) 0x00000001L)
#define SNP24_LVL_INIT_YCBCR        ((Tparam) 0x00000002L)
#define SNP24_LVL_SET               ((Tparam) 0x00000004L)
#define SNP24_LVL_INIT_601          ((Tparam) 0x00000008L)

#define SNP24_LVL_INVALID_PARAM     ((Tparam) 0xFFFFFFF0L)


#define SNP24_LVL_SMASK_NUMERIC     0xFC
#define SNP24_LVL_MAX_LEVELS        4
#define SNP24_LVL_WHITE             0
#define SNP24_LVL_BLACK             1
#define SNP24_LVL_CLAMP             2
#define SNP24_LVL_YCBCR             3

#define SNP24_LVL_WHITE_INIT        172
#define SNP24_LVL_BLACK_INIT        80
#define SNP24_LVL_CLAMP_INIT        84
#define SNP24_LVL_YCBCR_INIT        0

#define SNP24_LVL_WHITE_INIT_601    184
#define SNP24_LVL_BLACK_INIT_601    68
#define SNP24_LVL_CLAMP_INIT_601    84
#define SNP24_LVL_YCBCR_INIT_601    0

#define SNP24_LVL_WHITE_INIT_YCBCR  172
#define SNP24_LVL_BLACK_INIT_YCBCR  80
#define SNP24_LVL_CLAMP_INIT_YCBCR  160
#define SNP24_LVL_YCBCR_INIT_YCBCR  156

#define SNP24_LVL_WHITE_SCALE       ((ui16)26)
#define SNP24_LVL_BLACK_SCALE       ((ui16)20)
#define SNP24_LVL_CLAMP_SCALE       ((ui16)19)


/*
SNP24_set_linescan_ctrl()
-------------------------
*/
#define SNP24_LSCAN_INIT             ((Tparam) 0x00000001L)
#define SNP24_LSCAN_LINES_X1         ((Tparam) 0x00000002L)
#define SNP24_LSCAN_LINES_X2         ((Tparam) 0x00000004L)
#define SNP24_LSCAN_LINES_X4         ((Tparam) 0x00000008L)
#define SNP24_LSCAN_LINES_X8         ((Tparam) 0x00000010L)
#define SNP24_LSCAN_LTRIG_IN_POS     ((Tparam) 0x00000020L)
#define SNP24_LSCAN_LTRIG_IN_NEG     ((Tparam) 0x00000040L)
#define SNP24_LSCAN_LTRIG_IN_TIMER   ((Tparam) 0x00000080L)
#define SNP24_LSCAN_LSTART_IN_POS    ((Tparam) 0x00000100L)
#define SNP24_LSCAN_LSTART_IN_NEG    ((Tparam) 0x00000200L)
#define SNP24_LSCAN_LSTART_IN_NONE   ((Tparam) 0x00000400L)
#define SNP24_LSCAN_LSTART_OUT_POS   ((Tparam) 0x00000800L)
#define SNP24_LSCAN_LSTART_OUT_NEG   ((Tparam) 0x00001000L)
#define SNP24_LSCAN_LSTART_OUT_NONE  ((Tparam) 0x00002000L)
#define SNP24_LSCAN_LSTART_OUT_WIDTH ((Tparam) 0x00004000L)
#define SNP24_LSCAN_LTRIG_IN_HSYNC   ((Tparam) 0x00008000L)

#define SNP24_LSCAN_LINES_MASK       ((Tparam) 0x0000001EL)
#define SNP24_LSCAN_LTRIG_IN_MASK    ((Tparam) 0x000080E0L)
#define SNP24_LSCAN_LSTART_IN_MASK   ((Tparam) 0x00000700L)
#define SNP24_LSCAN_LSTART_OUT_MASK  ((Tparam) 0x00003800L)

#define SNP24_LSCAN_INVALID_PARAM    ((Tparam) 0xFFFF0000L)


/*
SNP24_set_LUTs()
----------------
*/
#define SNP24_LUT_INIT              ((Tparam) 0x00000001L)
#define SNP24_LUT_SET               ((Tparam) 0x00000002L)
#define SNP24_LUT_INVERSE           ((Tparam) 0x00000004L)

#define SNP24_LUT_INVALID_PARAM     ((Tparam) 0xFFFFFFF8L)

#define SNP24_SIZE_LUT252           256


#ifdef NOTE_NOT_FOR_NOW
/*
SNP24_set_multi_use()
---------------------
*/
#define SNP24_MULTI_INIT              ((Tparam) 0x00000001L)

#define SNP24_MULTI_VSYNC_OPTION      ((Tparam) 0x00000070L)
#define SNP24_MULTI_VSYNC_AS_VSYNC    ((Tparam) 0x00000010L)
#define SNP24_MULTI_VSYNC_AS_CTRL_OUT ((Tparam) 0x00000020L)
#define SNP24_MULTI_VSYNC_AS_TRIG_IN  ((Tparam) 0x00000040L)

#define SNP24_MULTI_HSYNC_OPTION      ((Tparam) 0x00000700L)
#define SNP24_MULTI_HSYNC_AS_HSYNC    ((Tparam) 0x00000100L)
#define SNP24_MULTI_HSYNC_AS_CTRL_OUT ((Tparam) 0x00000200L)
#define SNP24_MULTI_HSYNC_AS_TRIG_IN  ((Tparam) 0x00000400L)

#define SNP24_MULTI_PCLK_OPTION       ((Tparam) 0x00007000L)
#define SNP24_MULTI_PCLK_AS_PCLK      ((Tparam) 0x00001000L)
#define SNP24_MULTI_PCLK_AS_CTRL_OUT  ((Tparam) 0x00002000L)
#define SNP24_MULTI_PCLK_AS_TRIG_IN   ((Tparam) 0x00004000L)

#define SNP24_MULTI_INVALID_PARAM     ((Tparam) 0xFFFF888EL)
#endif


/*
SNP24_set_read_bank()
---------------------
*/
#define SNP24_BANK_INIT             ((Tparam) 0x00000001L)
#define SNP24_BANK_READ_MASK        ((Tparam) 0x0000000EL)
#define SNP24_BANK_READ_A           ((Tparam) 0x00000002L)
#define SNP24_BANK_READ_B           ((Tparam) 0x00000004L)
#define SNP24_BANK_READ_TOGGLE      ((Tparam) 0x00000008L)
#define SNP24_BANK_HW_DISABLE       ((Tparam) 0x00000010L)
#define SNP24_BANK_HW_ENABLE        ((Tparam) 0x00000020L)
#define SNP24_BANK_HW_VBANK_TOGGLE  ((Tparam) 0x00000040L)
#define SNP24_BANK_HW_HBANK_TOGGLE  ((Tparam) 0x00000080L)

#define SNP24_BANK_INVALID_PARAM    ((Tparam) 0xFFFFFF00L)


/*
SNP24_set_ROI()
---------------
*/
#define SNP24_ROI_SET               ((Tparam) 0x00000001L)
#define SNP24_ROI_CHECK             ((Tparam) 0x00000002L)


/*
SNP24_set_sync()
----------------
*/
#define SNP24_SYNC_INIT             ((Tparam) 0x00000001L)

#define SNP24_SYNC_INTERNAL         ((Tparam) 0x00000002L)
#define SNP24_SYNC_EXTERNAL         ((Tparam) 0x00000004L)

#define SNP24_SYNC_OFF_VIDEO        ((Tparam) 0x0000000CL)

#define SNP24_SYNC_OFF_RED_OPTION   ((Tparam) 0x08000000L)
#define SNP24_SYNC_OFF_RED1         ((Tparam) 0x08000014L)
#define SNP24_SYNC_OFF_RED2         ((Tparam) 0x08000024L)
#define SNP24_SYNC_OFF_RED3         ((Tparam) 0x08000044L)
#define SNP24_SYNC_OFF_RED4         ((Tparam) 0x08000084L)

#define SNP24_SYNC_OFF_GRN_OPTION   ((Tparam) 0x10000000L)
#define SNP24_SYNC_OFF_GRN1         ((Tparam) 0x10000014L)
#define SNP24_SYNC_OFF_GRN2         ((Tparam) 0x10000024L)
#define SNP24_SYNC_OFF_GRN3         ((Tparam) 0x10000044L)
#define SNP24_SYNC_OFF_GRN4         ((Tparam) 0x10000084L)

#define SNP24_SYNC_OFF_BLU_OPTION   ((Tparam) 0x20000000L)
#define SNP24_SYNC_OFF_BLU1         ((Tparam) 0x20000014L)
#define SNP24_SYNC_OFF_BLU2         ((Tparam) 0x20000024L)
#define SNP24_SYNC_OFF_BLU3         ((Tparam) 0x20000044L)
#define SNP24_SYNC_OFF_BLU4         ((Tparam) 0x20000084L)

#define SNP24_SYNC_OFF_CSYNC_NEG    ((Tparam) 0x00110004L)
#define SNP24_SYNC_OFF_HVSYNC_POS   ((Tparam) 0x40220004L)
#define SNP24_SYNC_OFF_HVSYNC_NEG   ((Tparam) 0x80220004L)
#define SNP24_SYNC_OFF_C_ON_H_POS   ((Tparam) 0x40440004L)
#define SNP24_SYNC_OFF_C_ON_H_NEG   ((Tparam) 0x80440004L)
#define SNP24_SYNC_OFF_HSYNC_POS_VSYNC_NEG   ((Tparam) 0x00201004L)
#define SNP24_SYNC_OFF_HSYNC_NEG_VSYNC_POS   ((Tparam) 0x00202004L)

#define SNP24_SYNC_CSYNC_IN         ((Tparam) 0x00100000L)
#define SNP24_SYNC_VSYNC_IN         ((Tparam) 0x00200000L)
#define SNP24_SYNC_HSYNC_IN         ((Tparam) 0x00400000L)
#define SNP24_SYNC_CSYNC_OUT_NEG    ((Tparam) 0x01000000L)
#define SNP24_SYNC_VSYNC_OUT_POS    ((Tparam) 0x02000000L)
#define SNP24_SYNC_VSYNC_OUT_NEG    ((Tparam) 0x04000000L)
#define SNP24_SYNC_HSYNC_OUT_POS    ((Tparam) 0x00800000L)
#define SNP24_SYNC_HSYNC_OUT_NEG    ((Tparam) 0x00080000L)
#define SNP24_SYNC_C_ON_H_OUT_POS   ((Tparam) 0x00000100L)
#define SNP24_SYNC_C_ON_H_OUT_NEG   ((Tparam) 0x00000200L)
#define SNP24_SYNC_FIELDS_SWAP      ((Tparam) 0x00000400L)
#define SNP24_SYNC_FIELDS_STD       ((Tparam) 0x00000800L)

#define SNP24_SYNC_INVALID_PARAM    ((Tparam) 0x0000C000L)

#define SNP24_SYNC_OFF_XX_MASK      ((Tparam) 0x000730F8L)


/*
SNP24_set_timer()
-----------------
*/
#define SNP24_TIMER_INIT            ((Tparam) 0x00000001L)

#define SNP24_TIMER_SET_EXPOSURE    ((Tparam) 0x00000002L)
#define SNP24_TIMER_SET_TIMER_FN    ((Tparam) 0x00000004L)

#define SNP24_TIMER_INVALID_PARAM   ((Tparam) 0xFFFFFFF8L)


/*
SNP24_set_trigger()
-------------------
* Note - Space reserved for numeric parameter - e.g. timeout value
*/
#define SNP24_TRIG_SMASK_NUMERIC    ((Tparam) 0x00000FFFL)

#define SNP24_TRIG_INIT             ((Tparam) 0x00001000L)
#define SNP24_TRIG_IN_POS           ((Tparam) 0x00002000L)
#define SNP24_TRIG_IN_NEG           ((Tparam) 0x00004000L)
#define SNP24_TRIG_OUT_HI           ((Tparam) 0x00008000L)
#define SNP24_TRIG_OUT_LO           ((Tparam) 0x00010000L)
#define SNP24_TRIG_IN_VIDSRC        ((Tparam) 0x00020000L)
#define SNP24_TRIG_IN_TRIG1         ((Tparam) 0x00040000L)
#define SNP24_TRIG_IN_TRIG2         ((Tparam) 0x00080000L)
#define SNP24_TRIG_IN_TRIG3         ((Tparam) 0x00100000L)
#define SNP24_TRIG_IN_TRIG4         ((Tparam) 0x00200000L)
#define SNP24_TRIG_OUT_TIMER_POS    ((Tparam) 0x00400000L)
#define SNP24_TRIG_OUT_TIMER_NEG    ((Tparam) 0x00800000L)
#define SNP24_TRIG_OUT_TIMER_MASK   ((Tparam) 0x01000000L)
#define SNP24_TRIG_OUT_TIMER_UNMASK ((Tparam) 0x02000000L)

#define SNP24_TRIG_INVALID_PARAM    ((Tparam) 0xFC000FFFL)
#define SNP24_TRIG_IN_OUT_MASK      ((Tparam) 0x03FFE000L)

#ifdef SNP24_COMPATIBILITY
/* These defines must match those above */
#define SNP24_TRIG_IN_RISING        ((Tparam) 0x00002000L)
#define SNP24_TRIG_IN_FALLING       ((Tparam) 0x00004000L)
#endif

/*
SNP24_set_TTL422()
------------------
*/
#define SNP24_TTL422_INIT           ((Tparam) 0x00000001L)
#define SNP24_TTL422_HSYNC_TTL      ((Tparam) 0x00000002L)
#define SNP24_TTL422_HSYNC_422      ((Tparam) 0x00000004L)
#define SNP24_TTL422_VSYNC_TTL      ((Tparam) 0x00000008L)
#define SNP24_TTL422_VSYNC_422      ((Tparam) 0x00000010L)
#define SNP24_TTL422_TRIG_TTL       ((Tparam) 0x00000020L)
#define SNP24_TTL422_TRIG_422       ((Tparam) 0x00000040L)
#define SNP24_TTL422_PCLK_TTL       ((Tparam) 0x00000080L)
#define SNP24_TTL422_PCLK_422       ((Tparam) 0x00000100L)

#define SNP24_TTL422_INVALID_PARAM  ((Tparam) 0xFFFFFE00L)


/*
SNP24_set_video_standard()
--------------------------
*/
#define SNP24_EIA_DEFAULT            ((Tparam) 0x00000001L)
#define SNP24_CCIR_DEFAULT           ((Tparam) 0x00000002L)
#define SNP24_LINE_SCAN_MODE         ((Tparam) 0x00008000L)



/*
SNP24_set_video_src()
---------------------
*/
#define SNP24_VIDSRC_INIT           ((Tparam) 0x00000001L)
#define SNP24_VIDSRC_SRC1           ((Tparam) 0x00000010L)
#define SNP24_VIDSRC_SRC2           ((Tparam) 0x00000020L)
#define SNP24_VIDSRC_SRC3           ((Tparam) 0x00000040L)
#define SNP24_VIDSRC_SRC4           ((Tparam) 0x00000080L)

#define SNP24_VIDSRC_INVALID_PARAM  ((Tparam) 0xFFFFFF0EL)


/*
SNP24_settings_load
SNP24_settings_save
----------------------
*/
#define SNP24_SETTINGS_FORMAT_FILE  ((Tparam) 0x00000001L)



/* DEFAULT IMAGE ACQUISITION COORDINATES
 * -------------------------------------
 */
#define SNP24_AREA_CCIR_X_START    ((i16)   52)
#define SNP24_AREA_CCIR_Y_START    ((i16)   44)
#define SNP24_AREA_CCIR_X_LENGTH   ((i16)  768)
#define SNP24_AREA_CCIR_Y_LENGTH   ((i16)  576)

#define SNP24_AREA_EIA_X_START     ((i16)   16)
#define SNP24_AREA_EIA_Y_START     ((i16)   32)
#define SNP24_AREA_EIA_X_LENGTH    ((i16)  640)
#define SNP24_AREA_EIA_Y_LENGTH    ((i16)  480)

#define SNP24_DEFAULT_CLAMP_START  ((ui8) 0x16)
#define SNP24_MAX_CLAMP_START      ((ui8) 0xFF)

/* STRUCTURE DEFINITIONS
 * ---------------------
 *
struct Tsnp24(ForBrief)
 */
struct Tsnp24
{
   Terr     (*Pfunc)(Thandle, ui8, ...);   /* To access BASE_module() */
   Thandle  Hbaseboard;
   ui8      ID;                             /* B0 for SNP-24, B8 for SNP-8 */
   ui8      rev;                            /* 0 / 1 / 2 etc */
   ui32     dwTimeoutTrigger;               /* Milli-seconds */
   Tboolean single_cap_next_field;          /* TRUE = need to restore NEXT FIELD mode on switching to single capture */
   Tboolean single_cap_x1_duplicate;        /* TRUE = need to restore X1 FIELD DUPLCATE mode on switching to single capture */
   Tboolean readout_deinterlace;            /* TRUE = Must deinterlace on read */
   Tboolean sync_off_video;                 /* TRUE = Automatically change sync with video */
   Tboolean trigger_off_video;              /* TRUE = Automatically change trigger with video */
   Tboolean ctrlout_off_video;              /* TRUE = Automatically change ctrl out with video */
   Tboolean TRIG_EXP;                       /* TRUE = trigger out used for exposure pulse */
   Tboolean bank_a_read;                    /* TRUE = FRAM bank A data has been read, since last written */
   Tboolean bank_b_read;                    /* TRUE = FRAM bank B data has been read, since last written */
   ui16     mode;                           /* Area scan, line scan */
   ui16     video_standard;                 /* CCIR, EIA */
   ui16     module_format;                  /* e.g. xxYx, xBGR */
   ui16     video_format;                   /* e.g. RGB, MONO_ON_RED, MONO_ON_GREEN, MONO_ON_BLUE */
   ui16     TMG_format;                     /* e.g. TMG_RGB16 */
   Tparam   video_source;                   /* Channel 1 / 2 / 3 / 4 */
   Tparam   subsample_mode;                 /* x1 / x2 / etc */
   ui32     field;                          /* TMG field ID, needed for x1 sequence */
   ui32     exposure_time;                  /* microseconds */
#if defined _WINNT || defined _WIN95 || defined _MAC_LVR || defined _MACINTOSH || defined _SOLARIS2 || defined _MACOSX || defined _LINUX
   Terr     (*timer_fn)(Thandle);           /* Function to generate exposure pulse(s) */
#else
   Terr     (EXPORT_FN *timer_fn)(Thandle); /* Function to generate exposure pulse(s) */
#endif
   i16      roi[ASL_SIZE_2D_ROI];           /* ROI setting */
   i16      active_area[ASL_SIZE_2D_ROI];   /* Max ROI setting allowed */
   ui16     x_round;                        /* Required 2^n rounding for x_length */
   ui16     y_round;                        /* Required 2^n rounding for y_length */

   ui16     ICS1522_reg[SNP24_NUM_ICS1522_REGS];  /* Shadows of registers */
   ui8      LCA_reg[SNP24_NUM_LCA_REGS];
   ui8      LM1882_reg[SNP24_NUM_LM1882_REGS];
   ui8      Bt252_cmd_red;                  /* Need to store command registers */
   ui8      Bt252_cmd_grn;                  /* because of extra level of paging */
   ui8      Bt252_cmd_blu;
   char     snp_fpga_date[RBT_ASL_DATE_LENGTH]; /* Date read from Snapper FPGA */ 
   ui16     wLinesSoFar;                     /* for read_video_data */
   ui16     wLocalAction;                    /* for read_video_data */
   IM_UI16 *Pline_buf_ui16;                  /* for read_video_data */
   IM_UI32 *Pline_buf_ui32;                  /* for read_video_data */

#define SNP24_FRAM_DUMMY_READ_BYTES ((ui32)400)  /* FRAM dummy read size - used in SNP24_capture */
   ui32 dwFramDummyReadBytes;
   ui32 dwMainReadBytes;                     /* Amount of data per image read */
   ui32 dwInterruptControl;                  /* ORable. Lower 16 bits = HW, upper 16 = SW */
};


/* MACRO DEFINITIONS not for error handling (in alphabetical order)
 * ----------------------------------------------------------------
 */

#define SNP24_check_mode(requested,check)             ( ((requested) & (check)) == (check) )

#define SNP24_check_mode_conflict(req,chk1,chk2)      ( ( ((req) & (chk1)) == (chk1) ) && ( ((req) & (chk2)) == (chk2) ) )

#define SNP24_get_offset(r)         (r & 0x07)
#define SNP24_get_page(r)           (r >> 3)


#if defined _lint
struct Tsnp24 * EXPORT_FN DUMMY_get_Psnp24(char *, Thandle);
#define SNP24_get_Psnp24(s,Ps,Hs)   Ps = DUMMY_get_Psnp24(s,Hs);

#elif defined _MALLOC_DEBUG
#define SNP24_get_Psnp24(s,Ps,Hs) {if (((Hs) < 1) || ((Hs) >= SNP24_NUM_HANDLES))\
                                      ASL_proe(ASLERR_BAD_HANDLE,"Handle out of range",s)\
                                   else if ((Ps = Psnp24_array[Hs]) == NULL)\
                                      ASL_proe(ASLERR_BAD_HANDLE,"Null pointer",s)\
                                  }
#else
#define SNP24_get_Psnp24(s,Ps,Hs)   {if ((Ps = Psnp24_array[Hs]) == NULL) ASL_proe(ASLERR_BAD_HANDLE,"",s)}
#endif



/* MACRO DEFINITIONS for error handling (in alphabetical order)
 * ------------------------------------------------------------
 * Note: SNP24_err_ret and SNP24_ret_on_err cannot be used on the line
 * after an `else' - {} are needed even on single line if-else statements.
 */
#if defined _lint
extern Terr EXPORT_FN SNP24_Dummy_err_ret(Thandle, Terr, char *, Terr, char *);
#define SNP24_err_ret(Hs,e,d,v,s)   return(SNP24_Dummy_err_ret(Hs,e,d,v,s))
#define SNP24_make_ret(Hs,e,v)      (( (Psnp24_array[Hs]->ID | Psnp24_array[Hs]->rev) << 24) | (e) | ((Terr) v & (Terr) 0xffff))

#elif defined _DOS16  /* real mode DOS - we lose some the comments to save space */
#define SNP24_err_ret(Hs,e,d,v,fn)  {ASL_err_process(fn,SNP24_make_ret(Hs,e,v),"");return(SNP24_make_ret(Hs,e,v));}
#define SNP24_make_ret(Hs,e,v)      (( (Psnp24_array[Hs]->ID | Psnp24_array[Hs]->rev) << 24) | (e) | ((Terr) v & (Terr) 0xffff))

#else
#define SNP24_err_ret(Hs,e,d,v,fn)  {ASL_err_process(fn,SNP24_make_ret(Hs,e,v),d);return(SNP24_make_ret(Hs,e,v));}
#define SNP24_make_ret(Hs,e,v)      (( (Psnp24_array[Hs]->ID | Psnp24_array[Hs]->rev) << 24) | (e) | ((Terr) v & (Terr) 0xffff))
#endif



#endif /* _SNP24_H_ */
