/**********************************************************************
 *                ACTIVE SILICON LIMITED / DATACELL LIMITED           *
 *  File name   : xpl_misc.c                                          *
 *  Function    : Miscellaneous Support Functions                     *
 *  Project     : ASLexamp                                            *
 *  Author      : Martin Bone                                         *
 *  Systems     : Ansi C - OS independent                             *
 *  Written     : 18.Aug.99                                           *
 *  Updated     :                                                     *
 *  Version     :                                                     *
 *                                                                    *
 *  Copyright (c) 1993-99 Active Silicon Ltd                          *
 **********************************************************************
 * Comments:
 */

/* Active Silicon Include files */
#include <asl_inc.h>
#include <xpl_inc.h>



/* Xillix PMI 1400 Serial Configuration Routine
 **********************************************
 */
Terr ConfigXillixPmi1400 ( struct Info *psInfo )
{
   static char *szFnName = "ConfigXillixPmi1400";
   ui32    dwRxByteCount, dwTxByteCount;
   Terr    teStatus;
   int     i;

#define NUM_XILLIX_CHARS 14
   char cUartStatus, cRxDataType;
   char cXillixStatus[NUM_XILLIX_CHARS];
   char cXillixCommand[NUM_XILLIX_CHARS] = {
      0x1A,             /* 8   Don't change offset
                           7   Don't change gain
                           6   Disable ROI
                           5,4 8MHz clock output
                           3   Don'tChange integration time
                           2   Enable continuous integrationa on readout
                           1   No changes to the hardware clock sequencer */
      0x00,             /* 8   Don't overwrite defective columns with alternative data
                           7   Don't program defective columns
                           6   Reserve
                           5   No vertical binning
                           4   Reserved
                           3   Don't force shutter open
                           2   Trigger shuter during exposure time
                           1   Disable single snapshot */
      0x00, 0x80,       /*     100ms integration time */
      0x00,             /*     No vertical or horizontal binning */
      0x00,             /*     ROI Size (disabled by Byte 1 bit 6) */
      0x00,             /*     ROI Address (disabled by Byte 1 bit 6) */
      0x00, 0x00, 0x00, /*     Gain & Offset (disabled by Byte 1 bits 7 & 8) */
      0x00,             /* 8,5 Reserved
                           4   Edge trigger mode
                           3   Active low (falling edge) trigger
                           2   Control unit generated trigger
                           1   Trigger disabled */
      0x00, 0x00,       /*     Defective address & number */
      0x00              /* 8,4 Reserved
                           3   Don't read camera config
                           2   Don't read camera temp
                           1   Disable low noise readout */
   };




   /* Send the config string to the camera */
   dwTxByteCount = NUM_XILLIX_CHARS;
   if (BASE_serial_transmit_buffer( psInfo->sSnapInfo.hBase, cXillixCommand, &dwTxByteCount, 1000 ) != ASL_OK)
   {
      DBG_WriteString( 10, "\n%s: Base_serial_transmit_buffer failed", szFnName);
      return ASL_ERROR;
   }
   if (dwTxByteCount != NUM_XILLIX_CHARS)
   {
      DBG_WriteString( 10, "\n%s: Failed to send all characters", szFnName);
      return ASL_ERROR;
   }

   /* Read the first character - UartStatus */
   dwRxByteCount = 1;
   if (BASE_serial_receive_buffer( psInfo->sSnapInfo.hBase, &cUartStatus, &dwRxByteCount, 1000 ) != ASL_OK)
   {
      DBG_WriteString( 10, "\n%s: Base_serial_receive_buffer failed reading UartStatus", szFnName);
      return ASL_ERROR;
   }

   /* Now read the camera status */
   dwRxByteCount = NUM_XILLIX_CHARS;
   if (BASE_serial_receive_buffer( psInfo->sSnapInfo.hBase, cXillixStatus, &dwRxByteCount, 1000 ) != ASL_OK)
   {
      DBG_WriteString( 10, "\n%s: Base_serial_receive_buffer failed reading Camera info", szFnName);
      return ASL_ERROR;
   }

   /* Finally read the RxDataType, ie command echo, config info, or failed command */
   dwRxByteCount = 1;
   if (BASE_serial_receive_buffer( psInfo->sSnapInfo.hBase, &cRxDataType, &dwRxByteCount, 1000 ) != ASL_OK)
   {
      DBG_WriteString( 10, "\n%s: Base_serial_receive_buffer failed reading Rx Data Type", szFnName);
      return ASL_ERROR;
   }
   
   teStatus = ASL_OK;
   if ( cUartStatus != 0x61 )
   {
      DBG_WriteString( 10, "\n%s: Communication Error", szFnName);
      if ( cRxDataType != 0x55 )
         DBG_WriteString( 10, "\n%s: Failed to receive 0x55 marker", szFnName);
      teStatus = ASL_ERROR;
   }
   else
   {
      /* Config data */
      if (cRxDataType == 0x02)
      {
         DBG_WriteString( 10, "\n%s: Config Data - ", szFnName);
         for (i = 0; i < NUM_XILLIX_CHARS; i++)
            DBG_WriteString( 10, "0x%02x ", cXillixStatus[i]);
      }

      /* Command echo */
      else if (cRxDataType == 0x01)
      {
         DBG_WriteString( 10, "\n%s: Command Echo", szFnName);
         for (i = 0; i < NUM_XILLIX_CHARS; i++)
            if (cXillixCommand[i] != cXillixStatus[i])
               DBG_WriteString( 10, "\nFailed at offset %d, Tx=0x%02x, Rx=0x%02x", i, cXillixCommand[i], cXillixStatus[i]);
      }

      /* Unknown response */
      else
      {
         DBG_WriteString( 10, "\n%s: Unknown response type from camera - 0x%02x", szFnName, cRxDataType);
         teStatus = ASL_ERROR;
      }
   }

   /* Setup output exposure pulse to occur at the start of every capture */
   DIG16_set_timer( psInfo->sSnapInfo.hSnapper, DIG16_TIMER_SET_TIMER_FN, 0, TimerFnXillixPmi1400 );

   return teStatus;
}


/* Exposure timer function for Xillix PMI 1400
 *********************************************
 */
Terr TimerFnXillixPmi1400 ( Thandle hDig16 )
{
   Thandle hBase;

   hBase = DIG16_get_parameter( hDig16, DIG16_BASEBOARD_HANDLE );

   /* Drive the OutA signal for a fixed duration.  The baseboard hardware timer
    * function is used for a more repeatable and stable time delay
    */
   DIG16_set_ctrl_io( hDig16, DIG16_CTRL_OUT_A_1 );
   BASE_set_timer( hBase, BASE_TIMER_START_AND_WAIT, 100000 );
   DIG16_set_ctrl_io( hDig16, DIG16_CTRL_OUT_A_0 );

   return ASL_OK;
}


/* Histogramming function
 ************************
 */
Terr HistogramImage ( Thandle hImage )
{
   ui32 *pdwHistogramBase;
   ui32 *pdwHistogramData;
   ui32 dwHistogramElements, dwHistogramBufferBytes;
   ui32  i, j;
   ui32  dwImagePlanes, dwImageWidth, dwImageHeight, dwImageDepth;
   ui32  dwImagePixelsPerPlane;
   ui16 *pwImageData;
   ui8  *pcImageData;
   Terr teStatus = ASL_OK;


   dwImagePlanes  = TMG_image_get_parameter( hImage, TMG_NUM_PLANES );
   dwImageDepth   = TMG_image_get_parameter( hImage, TMG_DATA_WIDTH );
   dwImageHeight  = TMG_image_get_parameter( hImage, TMG_HEIGHT );
   dwImageWidth   = TMG_image_get_parameter( hImage, TMG_WIDTH );

   dwImagePixelsPerPlane = dwImageWidth * dwImageHeight;
   dwHistogramElements   = (1 << dwImageDepth);

   dwHistogramBufferBytes = dwHistogramElements * dwImagePlanes * sizeof (ui32);
   pdwHistogramBase = (ui32 *) _ASL_Malloc( dwHistogramBufferBytes );
   memset (pdwHistogramBase, 0, dwHistogramBufferBytes);

   if (dwImageDepth <= 8)
   {
      pcImageData = (ui8 *) TMG_image_get_ptr( hImage, TMG_IMAGE_DATA );
      for (i = 0; i < dwImagePixelsPerPlane; i++) {
         for (j = 0; j < dwImagePlanes; j++) {
            if ((ui32) *pcImageData > dwHistogramElements)
               teStatus = ASL_ERROR;
            else
               (*(pdwHistogramBase + ((int) *pcImageData++ * dwImagePlanes) + j))++;
         }
      }
   }

   else if (dwImageDepth <= 16)
   {
      pwImageData = (ui16 *) TMG_image_get_ptr( hImage, TMG_IMAGE_DATA );
      for (i = 0; i < dwImagePixelsPerPlane; i++) {
         for (j = 0; j < dwImagePlanes; j++) {
            if ((ui32) *pwImageData >= dwHistogramElements)
               teStatus = ASL_ERROR;
            else
               (*(pdwHistogramBase + ((int) *pwImageData++ * dwImagePlanes) + j))++;
         }
      }
   }

   else
      // Error - Don't support pixel depths greater than 16 bit
      teStatus = ASL_ERROR;

   if ( ASL_is_ok( teStatus ) )
   {
      pdwHistogramData = pdwHistogramBase;
      for (i = 0; i < dwHistogramElements; i++) {
         DBG_WriteString( 10, "\n%5d ", i);
         for (j = 0; j < dwImagePlanes; j++)
            DBG_WriteString( 10, "%5d ", (int) *pdwHistogramData++);
      }
   }

   _ASL_Free ( pdwHistogramBase );
   return teStatus;
}
